*DECK AFM
      SUBROUTINE AFM(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* generate a macrolib using the AFM feedback model
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s):  M.T. Sissaoui
*  Modified  E. Varin, B. Dionne
*
*Parameters: input/output
* NENTRY  number of linked lists or files used by the module
* HENTRY  character*12 name of each linked list or file
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the linked list or file is created
*         =1 the linked list or file is open for modifications;
*         =2 the linked list or file is open in read-only mode
* KENTRY  =file unit number; =linked list address otherwise
*         kentry(nentry)
* LINKED LIST / XSM FILE
* HENTRY(1)  create or modification type(L_MACROLIB)
* HENTRY(2)  read only type(reactor_xsdb)
* HENTRY(3)  read only type(L_MAP)
*
*Reference:
* M. T. Sissaoui, G. Marleau and D. Rozon, "CANDU Reactor Simulations
* Using the Feedback Model with Actinide Burnup History," Nucl.
* Technology, 125, 197 (1999).
*
*-----------------------------------------------------------------------
*
      PARAMETER (NSTATE=40) 
      INTEGER  NENTRY
      CHARACTER HENTRY(NENTRY)*12,TEXT12*12,HSMG*131,HSIGN*12,
     1 TEXT*5,CTITRE*72,TINFO*72
      LOGICAL LMCR,LMAP 
      INTEGER  IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY),
     1 IPAR(NSTATE),KCHAR(18),IBASE(1),IDATA(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      LMCR=.FALSE.
      LMAP=.FALSE.
      MSFT=0
*
* PARAMETER VALIDATION.
      IF(NENTRY.LE.1) CALL XABORT('AFM: 2 PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('AFM:'
     1 //'  MACROLIB LINKED LIST OR XSM FILE EXPECTED AT LHS.')
*
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('AFM:'
     1 //' DATABASE LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(2).NE.2) CALL XABORT('AFM: DATABASE IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
*
      CALL REDGET (INDIC,NITMA,FLOTT,TEXT,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('AFM: CHARACTER DATA EXPECTED.')
      IF(TEXT.EQ.'MCR') THEN
        LMCR=.TRUE.
        WRITE(6,'(A37)') 'AFM: GENERATION OF A SINGLE MACROLIB'
        CALL REDGET (INDIC,MXSH,FLOTT,TEXT,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('AFM: INTEGER DATA EXPECTED.')
      ELSEIF(TEXT.EQ.'MAP') THEN
        LMAP=.TRUE.
        IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)) CALL XABORT('AFM:'
     1    //' FUEL MAP LINKED LIST OR XSM FILE EXPECTED AT RHS.')
        IF(JENTRY(3).NE.2) CALL XABORT('AFM: COMPO IN READ-ONLY '
     1    //'MODE EXPECTED AT RHS.')
      ELSE
         CALL XABORT('AFM: MAP OR MCR KEY WORD EXPECTED')
      ENDIF
*
      ITYPE=JENTRY(1)
      IPLIST=KENTRY(1)
      MMIX=1
*---------------------------------------------------------------*
* CHECK THE SIGNTURE OF THE LINKED LIST OR XSM FILE
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'REACTOR_XSDB') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('AFM: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. REACTOR_XSDB EXPECTED.')
      ENDIF
*---------------------------------------------------------------*
* IF L_MAP IS NOT AVAILABLE AFM GENERATE ONLY A TABLE
      IF(LMAP) THEN
        CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(HSIGN.NE.'L_MAP') THEN
          TEXT12=HENTRY(3)
          CALL XABORT('AFM: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_MAP EXPECTED.')
        ENDIF
        WRITE(6,'(A42)') 'AFM: GENARATION OF A MACROLIB USING L_MAP'
      ENDIF
*---------------------------------------------------------------*
*        READ THE INFORMATION TITLE.
      CALL REDGET (INDIC,NITMA,FLOTT,TEXT,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('AFM: CHARACTER DATA EXPECTED.')
      IF(TEXT.EQ.'INFOR') THEN
         CALL REDGET(INDIC,NITMA,FLOTT,TINFO,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('AFM: CHARACTER DATA EXPECTED.')
      ELSE
          CALL XABORT('AFM:KEY WORD INFOR EXPECTED.')
      ENDIF
*
* RECOVER SOME INFORMATIONS FROM THE DATABASE.
      TEXT12='INFORMATION'
      CALL LCMLEN(KENTRY(2),TEXT12,LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
        CALL LCMGET(KENTRY(2),'INFORMATION',KCHAR)
        WRITE(CTITRE,'(18A4)') (KCHAR(I),I=1,18)
        WRITE(6,*)'INFORMATION TITLE ',TINFO
*
        IF(CTITRE.NE.TINFO) THEN
            CALL XABORT('AFM: INCONSISTENT TITLES  '//CTITRE//
     1      '  EXPECTED. INSTEAD OF ' //TINFO// ' ')
        ENDIF
      ELSE
        CALL XABORT('AFM: DATA BASE TITLE IS NOT PROVIDED ')
      ENDIF
*---------------------------------------------------------------*
*        CHECK THE NAMES OF THE DIFFERENTS DIRECTORIES
      CALL REDGET (INDIC,NITMA,FLOTT,TEXT,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('AFM: CHARACTER DATA EXPECTED.')
      IF(TEXT.NE.'DNAME') CALL XABORT('AFM:KEY WORD DNAME EXPECTED.')
      CALL REDGET(INDIC,NUT,FLOTT,TEXT,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('AFM: INTEGER DATA EXPECTED.')
      IF(NUT.GT.1.AND.LMCR) CALL XABORT('AFM: INVALID NUMBER.')
      DO 100 IJ=1,NUT
        CALL REDGET (INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('AFM: CHARACTER DATA EXPECTED.')
        CALL LCMLEN(KENTRY(2),TEXT12,LENGT,ITYLCM)
        IF(LENGT.EQ.0) THEN
           CALL XABORT('AFM: DATA NAME  '//TEXT12//
     1        '  DO NOT EXIST ')
         ENDIF
 100  CONTINUE
*---------------------------------------------------------------*
*     RECOVER SOME INFORMATIONS FROM DATABASE.
*     TEXT12='SIGNATURE'
      CALL LCMSIX(KENTRY(2),TEXT12,1)
*     RECOVER THE TITLE.
      CALL LCMLEN(KENTRY(2),'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(KENTRY(2),'TITLE',KCHAR)
         WRITE(CTITRE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         CTITRE='*** NO TITLE PROVIDED ***'
      ENDIF
*     READ PARAMETERS DANS L_FBM 
      CALL LCMGET(KENTRY(2),'PARAM',IPAR)
      CALL LCMSIX(KENTRY(2),' ',2)
      NGRP =IPAR(1)
      NISO =IPAR(2)
      NL   =IPAR(3)
      NBURN=IPAR(4)
      IXYZ =IPAR(5)
*---------------------------------------------------------------*
      IF(LMCR) THEN
        NBCH=MXSH
        NCCO=1
        NCZO=1
        ISC=4
        MMIX=NBCH*NCCO
        MSFT =0
      ELSEIF (LMAP) THEN
* RECOVER INFORMATIONS FROM L_MAP.
*     READ PARAMETERS
        CALL LCMGET(KENTRY(3),'STATE-VECTOR',IPAR)
        NBCH =IPAR(1)
        NCCO =IPAR(2)
        NCZO =IPAR(3)
        ISC  =IPAR(5)
        MSFT =IPAR(6)
        NPARM =IPAR(8)
        MMIX=NBCH*NCCO
C HISTORY PARAMETER
        IF(IPAR(4).NE.NGRP) THEN
            WRITE(HSMG,'(A40,I5,A18,I5)') 'AFM: INCONSISTENT NB OF '
     1        //'GROUPS. IN MAP =',IPAR(4),' IN REACTOR_XSDB =',NGRP
            CALL XABORT(HSMG)
        ENDIF
      ENDIF
*  MSFT IS THE TOTAL NUMBER OF SHIFT
      NPS=MSFT+2
*  READ THE INPUT DATA.
*  TO USE THE SAME VECTOR TO GET THE REFERENCE LOCAL PARAMETER
*      IF(NISO.LT.8) THEN
*        NISM=8
      IF(NISO.LT.7) THEN
        NISM=7
      ELSE
        NISM=NISO
      ENDIF
      IF(ITYPE.NE.0) THEN
         CALL LCMGET(IPLIST,'STATE-VECTOR',IDATA)
         IF(NGRP.NE.IDATA(1)) CALL XABORT('WRONG NUMBER OF ENERGY'
     1   //' GROUPS IN UPDATED MACROLIB')
         IF(MMIX.NE.IDATA(2)) CALL XABORT('WRONG NUMBER OF MATER'
     1   //'IAL MIXTURES IN UPDATED MACROLIB')
         IF(NL.NE.IDATA(3)) CALL XABORT('WRONG ORDER OF ANISOTROPY'
     1   //'IN UPDATED MACROLIB')
      ENDIF
*---------------------------------------------------------------*
*        NTYP TYPE OF CROSS-SECTIONS CONSIDERED
         NTYP=5+NL+IXYZ*2
*---------------------------------------------------------------*
*        DRIVER TO COMPUTE THE FEEDBACK COEFFICIENTS.
*---------------------------------------------------------------*
*        DYNANIC ALLOCATION MEMORY
*        L_MACROLIB
*     DRIVER TO COMPUTE THE FEEDBACK COEFFICIENTS.
      CALL SETARA(BASE,NGRP*NGRP*NTYP*MMIX,ISIGMA)
      CALL SETARA(BASE,MMIX*NL*NGRP,IIJJ)
      CALL SETARA(BASE,MMIX*NL*NGRP,INJJ)
      CALL SETARA(BASE,NBURN*NUT,IXBURN)
      CALL SETARA(BASE,MMIX*NGRP,ICHI)
      CALL SETARA(BASE,MMIX*NGRP,IOVERV)
      CALL SETARA(BASE,MMIX*NGRP,ISIGS)
      CALL SETARA(BASE,MMIX*NGRP,IFLUX)
      CALL SETARA(BASE,MMIX,IVOL)
      CALL SETARA(BASE,NGRP+1,IENER)
      CALL SETARA(BASE,MMIX,IIPOS)
      CALL SETARA(BASE,MMIX*NGRP*NBURN,IWK)
      CALL SETARA(BASE,MMIX*NGRP,IIWK)
      CALL SETARA(BASE,MMIX*NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,MMIX*NGRP,IDIFX)
      CALL SETARA(BASE,MMIX*NGRP,IDIFY)
      CALL SETARA(BASE,MMIX*NGRP,IDIFZ)
      CALL SETARA(BASE,NGRP,IIJ)
      CALL SETARA(BASE,NGRP,INJ)
*        L_MAP
      CALL SETARA(BASE,NCCO,IZONE)
      CALL SETARA(BASE,MMIX,IBRBG)
      CALL SETARA(BASE,MMIX,IBRED)
      CALL SETARA(BASE,MMIX,IPOWER)
      CALL SETARA(BASE,MMIX*MSFT,IBSFT)
      CALL SETARA(BASE,MMIX*MSFT,IPSFT)
      CALL SETARA(IBASE,MMIX,IVSFT)
      CALL SETARA(BASE,MMIX*NGRP,IBFLUX)
      CALL SETARA(IBASE,MMIX,IINDEX)
C     COOLANT TEMPERATURE AND DENSITY PROFILES
      CALL SETARA(BASE,MMIX,ITCOOL)
      CALL SETARA(BASE,MMIX,IDCOOL)
C     FUEL TEMPERATURE PROFILES
      CALL SETARA(BASE,MMIX,ITFUEL)
*
      CALL SETARA(BASE,NPS,IBRH)
      CALL SETARA(BASE,NPS,IPW)
C TIME AVERAGE CALCULATION
      CALL SETARA(BASE,NBURN*NGRP,IFLUAV)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NBURN,ISGTV)
      CALL SETARA(BASE,NBURN*NL*NGRP*NGRP,ISCTV)
*        REACTOR_XSDB
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,ISMACB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT1FB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT2FB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT1CB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT2CB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT1MB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXT2MB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXD1CB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXD2CB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXD1MB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXD2MB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXBORB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXPURB)
* HISTORY BASE MEMORY ALLOCATION
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IXXENB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,ISM149B)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,INP239B)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IMIXFDB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IMIXMDB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IPFC1B)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IPFC2B)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IPFC1LB)
      CALL SETARA(BASE,NGRP*NGRP*NTYP*NISO*NBURN*NUT,IPFC2LB)
      CALL SETARA(BASE,NISO*NBURN*NUT,IDENSIB)
      CALL SETARA(IBASE,3*NISM,IHISO)
*
      CALL SETARA(BASE,NGRP*NBURN*NUT,IFLUXB)
      CALL SETARA(BASE,NGRP*NBURN*NUT,IOVERVB)
      CALL SETARA(BASE,NGRP*NBURN*NUT,ICHIB)
      CALL SETARA(BASE,2*NBURN*NUT,ICOWB1)
      CALL SETARA(BASE,2*NBURN*NUT,ICOWB2)
      CALL SETARA(BASE,2*NBURN*NUT,IHXEN1)
      CALL SETARA(BASE,2*NBURN*NUT,IHXEN2)
      CALL SETARA(BASE,2*NBURN*NUT,IHSAM1)
      CALL SETARA(BASE,2*NBURN*NUT,IHSAM2)
      CALL SETARA(BASE,2*NBURN*NUT,IHNEP1)
      CALL SETARA(BASE,2*NBURN*NUT,IHNEP2)
* XENON CORRECTION ALLOCATION MEMORIES
      CALL SETARA(BASE,NGRP,ISIGF)
      CALL SETARA(BASE,NGRP,ISIGX)
      CALL SETARA(BASE,NGRP,IFLR)
      CALL SETARA(IBASE,3*NUT,ITER)
      CALL SETARA(BASE,NUT,JTYP)
      CALL SETARA(IBASE,NISO,IJTAB)
*---------------------------------------------------------------*
      CALL AFMDRV(KENTRY,NPARM,ITYPE,NBURN,NGRP,NISO,ISC,NPS,
     1   NL,ILEAK,NTYP,NBCH,NCCO,NCZO,NUT,BASE(ISIGMA),BASE(IIJJ),
     1   BASE(INJJ),CTITRE,BASE(IXBURN),BASE(IVOL),BASE(IOVERV),
     1   BASE(ISIGS),BASE(IFLUX),BASE(ICHI),BASE(IENER),
     1   BASE(IIPOS),BASE(ISCAT),BASE(IWK),BASE(IDIFX),
     1   BASE(IDIFY),BASE(IDIFZ),BASE(IIJ),BASE(INJ),BASE(IZONE),
     1   BASE(IBRBG),BASE(IBRED),BASE(IPOWER),BASE(IBFLUX),
     1   BASE(IPW),BASE(IBRH),BASE(IFLUAV),BASE(ISGTV),BASE(ISCTV),
     1   BASE(IIWK),BASE(ISMACB),BASE(IXBORB),BASE(IXPURB),
     1   BASE(IXXENB),BASE(IXT1FB),BASE(IXT2FB),BASE(IXT1CB),
     1   BASE(IXT2CB),BASE(IXT1MB),
     1   BASE(IXT2MB),BASE(IXD1CB),BASE(IXD2CB),BASE(IXD1MB),
     1   BASE(IXD2MB),BASE(ISM149B),BASE(INP239B),BASE(IMIXFDB),
     1   BASE(IMIXMDB),BASE(IPFC1B),BASE(IPFC2B),BASE(IPFC1LB),
     1   BASE(IPFC2LB),BASE(IDENSIB),IBASE(IHISO),
     1   BASE(ICOWB1),BASE(ICOWB2),BASE(IHXEN1),BASE(IHXEN2),
     1   BASE(IHSAM1),BASE(IHSAM2),BASE(IHNEP1),BASE(IHNEP2),
     1   BASE(IFLUXB),BASE(IOVERVB),BASE(ICHIB),
     1   LMCR,BASE(ISIGF),BASE(ISIGX),BASE(IFLR),
     1   BASE(IBSFT),BASE(IPSFT),IBASE(IVSFT),IBASE(IINDEX),
     1   IBASE(ITER),BASE(JTYP),IBASE(IJTAB),IXYZ,
     1   BASE(IDCOOL),BASE(ITCOOL),BASE(ITFUEL))
*---------------------------------------------------------------*
       CALL RLSARA(IBASE(IJTAB))
       CALL RLSARA(BASE(JTYP))
       CALL RLSARA(IBASE(ITER))
       CALL RLSARA(BASE(IFLR))
       CALL RLSARA(BASE(ISIGX))
       CALL RLSARA(BASE(ISIGF))
*
       CALL RLSARA(BASE(IHNEP2))
       CALL RLSARA(BASE(IHNEP1))
       CALL RLSARA(BASE(IHSAM2))
       CALL RLSARA(BASE(IHSAM1))
       CALL RLSARA(BASE(IHXEN2))
       CALL RLSARA(BASE(IHXEN1))
       CALL RLSARA(BASE(ICOWB2))
       CALL RLSARA(BASE(ICOWB1))
       CALL RLSARA(BASE(ICHIB))
       CALL RLSARA(BASE(IOVERVB))
       CALL RLSARA(BASE(IFLUXB))
*
       CALL RLSARA(IBASE(IHISO))
       CALL RLSARA(BASE(IDENSIB))
       CALL RLSARA(BASE(IPFC2LB))
       CALL RLSARA(BASE(IPFC1LB))
       CALL RLSARA(BASE(IPFC2B))
       CALL RLSARA(BASE(IPFC1B))
       CALL RLSARA(BASE(IMIXMDB))
       CALL RLSARA(BASE(IMIXFDB))
       CALL RLSARA(BASE(INP239B))
       CALL RLSARA(BASE(ISM149B))
*
       CALL RLSARA(BASE(IXXENB))
       CALL RLSARA(BASE(IXPURB))
       CALL RLSARA(BASE(IXBORB))
       CALL RLSARA(BASE(IXD2MB))
       CALL RLSARA(BASE(IXD1MB))
       CALL RLSARA(BASE(IXD2CB))
       CALL RLSARA(BASE(IXD1CB))
       CALL RLSARA(BASE(IXT2MB))
       CALL RLSARA(BASE(IXT1MB))
       CALL RLSARA(BASE(IXT2CB))
       CALL RLSARA(BASE(IXT1CB))
       CALL RLSARA(BASE(IXT2FB))
       CALL RLSARA(BASE(IXT1FB))
       CALL RLSARA(BASE(ISMACB))
*
       CALL RLSARA(BASE(ISCTV))
       CALL RLSARA(BASE(ISGTV))
       CALL RLSARA(BASE(IFLUAV))
*
       CALL RLSARA(BASE(IPW))
       CALL RLSARA(BASE(IBRH))
*
       CALL RLSARA(BASE(ITFUEL))
       CALL RLSARA(BASE(IDCOOL))
       CALL RLSARA(BASE(ITCOOL))
       CALL RLSARA(IBASE(IINDEX))
       CALL RLSARA(BASE(IBFLUX))
       CALL RLSARA(IBASE(IVSFT))
       CALL RLSARA(BASE(IPSFT))
       CALL RLSARA(BASE(IBSFT))
       CALL RLSARA(BASE(IPOWER))
       CALL RLSARA(BASE(IBRED))
       CALL RLSARA(BASE(IBRBG))
       CALL RLSARA(BASE(IZONE))
*
       CALL RLSARA(BASE(INJ))
       CALL RLSARA(BASE(IIJ))
       CALL RLSARA(BASE(IDIFZ))
       CALL RLSARA(BASE(IDIFY))
       CALL RLSARA(BASE(IDIFX))
       CALL RLSARA(BASE(ISCAT))
       CALL RLSARA(BASE(IIWK))
       CALL RLSARA(BASE(IWK))
       CALL RLSARA(BASE(IIPOS))
       CALL RLSARA(BASE(IENER))
       CALL RLSARA(BASE(IVOL))
       CALL RLSARA(BASE(IFLUX))
       CALL RLSARA(BASE(ISIGS))
       CALL RLSARA(BASE(IOVERV))
       CALL RLSARA(BASE(ICHI))
       CALL RLSARA(BASE(IXBURN))
       CALL RLSARA(BASE(INJJ))
       CALL RLSARA(BASE(IIJJ))
       CALL RLSARA(BASE(ISIGMA))
*
      IF(JENTRY(1).EQ.0) THEN
        CALL XDISET(IDATA,NSTATE,0)
        HSIGN='L_MACROLIB'
        READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        CALL LCMPUT(IPLIST,'SIGNATURE',3,3,KCHAR)
        IDATA(1)=NGRP
        IDATA(2)=MMIX
        IDATA(3)=NL
        IDATA(4)=1
        IDATA(9)=ILEAK
        CALL LCMPUT(IPLIST,'STATE-VECTOR',NSTATE,1,IDATA)
      ELSE
        IDATA(1)=NGRP
        IDATA(2)=MMIX
        IDATA(3)=1
        IDATA(4)=1
        IDATA(9)=ILEAK
        CALL LCMPUT(IPLIST,'STATE-VECTOR',NSTATE,1,IDATA)
      ENDIF
      RETURN
      END
