*DECK FLPDRV
      SUBROUTINE FLPDRV(IPPOW,IPNFX,IPFLX,IPKIN,IPTRK,IPMTX,IPMAP,
     1 IPMAC,PTOT,LNEW,LMAP,JMOD,LFLX,LPOW,LRAT,IMPX,FSTH,LFSTH,LFLU,
     2 LBUN,LNRM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the powers and fluxes computations.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki, M. Guyot
*
*Parameters: input/output
* IPPOW  pointer to power information.
* IPFLX  pointer to flux information.
* IPKIN  pointer to kinetics information.
* IPTRK  pointer to tracking information.
* IPMTX  pointer to matex information.
* IPMAP  pointer to fuel-map information.
* IPMAC  pointer to macrolib information.
* PTOT   given total reactor power in mega-watts.
* LNEW   new total power flag (=.true. for new computation).
* LMAP   fuel-map printing on file flag (=.true. for print).
* JMOD   modification index for L_MAP object.
* LFLX   flux printing on file flag (=.true. for print).
* LPOW   power printing on file flag (=.true. for print).
* LRAT   flux-ratio printing on file flag (=.true. for print).
* IMPX   printing on screen index (=0 for no print).
* FSTH   thermal to fission power ratio
* LFSTH  =.true if the thermal fission ratio is specified
* LFLU   =.true. if an output flux is to be created
* LBUN   =.true. if the output flux is a flux per bundle
* LNRM   =.true. if the output flux is a flux per mesh-splitted element
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPPOW,IPNFX,IPFLX,IPKIN,IPTRK,IPMTX,IPMAP,IPMAC,JMOD,IMPX
      LOGICAL LNEW,LMAP,LFLX,LPOW,LRAT,LFSTH,LFLU,LBUN,LNRM
      REAL PTOT,FSTH
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER IBASE(1),KCHAR(3),ISTATE(NSTATE),IHFC,MPFLUX,
     1        IGR,NUN,NGRP,IGEO
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION ZNRM,VTOT,POWR
      CHARACTER HSIGN*12
*----
*  CHECK THE TYPE OF FLUX SELECTED FOR OUTPUT
*----
      IF(LFLU)THEN
        IF(LNRM.AND.LBUN) CALL XABORT('@FLPDRV: KEYWORD NORM AND BUND '
     1    //'BOTH SELECTED.')
        IF((.NOT.LNRM).AND.(.NOT.LBUN)) THEN
          LNRM=.TRUE.
          WRITE(6,*) 'FLPDRV: default option for L_FLUX object is NORM.'
        ENDIF
      ENDIF
*----
*  RECOVER INFORMATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      IF(IPFLX.NE.0) THEN
*       L_FLUX object
        CALL LCMGET(IPFLX,'STATE-VECTOR',ISTATE)
        NGRP=ISTATE(1)
        NUN=ISTATE(2)
        CALL LCMGET(IPFLX,'K-EFFECTIVE',FKEFF)
      ELSE IF(IPKIN.NE.0) THEN
*       L_KINET object
        CALL LCMGET(IPKIN,'STATE-VECTOR',ISTATE)
        NGRP=ISTATE(3)
        NUN=ISTATE(7)
        CALL LCMGET(IPKIN,'E-KEFF',FKEFF)
        CALL LCMGET(IPKIN,'E-POW',PTOT)
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NEL=ISTATE(1)
      IF(ISTATE(2).NE.NUN) CALL XABORT('@FLPDRV: INCOMPATIBLE L_TRACK '
     1 //'AND L_FLUX/L_KINET OBJECTS')
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      CALL XDISET(ISTATE,NSTATE,0)
      IF(IPMAC.EQ.0)THEN
        CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
        NB=ISTATE(1)
        NCH=ISTATE(2)
        IGEO=ISTATE(12)
        IF((IGEO.NE.7).AND.(IGEO.NE.9))CALL XABORT('@FLPDRV: INVALID'
     1   //' GEOMETRY IN FUEL MAP : ONLY 3-D CARTESIAN OR 3-D HEXAGO'
     2   //'NAL GEOMETRIES AVAILABLE') 
        IF(ISTATE(4).NE.NGRP)CALL XABORT('@FLPDRV: INVALID NUM'
     1   //'BER OF ENERGY GROUPS IN FUEL MAP OR FLUX.')
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPMTX,'STATE-VECTOR',ISTATE)
        NMIX=ISTATE(2)
        NTOT=ISTATE(5)
        IF(ISTATE(6).NE.IGEO)CALL XABORT('@FLPDRV: GEOMETRIES IN'
     1   //' MATEX AND FUEL MAP ARE DIFFERENT')
        IF(ISTATE(7).NE.NEL)CALL XABORT('@FLPDRV: INVALID TOTAL'
     1   //' NUMBER OF REGIONS IN FUEL MAP OR TRACK.')
        CALL SETARA(BASE,NMIX*NGRP,IHFC)
        CALL XDRSET(BASE(IHFC),NMIX*NGRP,0.)
        CALL LCMGET(IPMTX,'H-FACTOR',BASE(IHFC))
      ELSE
        CALL LCMGET(IPMAC,'STATE-VECTOR',ISTATE)
        IF(ISTATE(1).NE.NGRP)CALL XABORT('@FLPDRV: INVALID NUM'
     1   //'BER OF ENERGY GROUPS IN MACROLIB OR FLUX.')
        NMIX=ISTATE(2)
        CALL SETARA(BASE,NMIX*NGRP,IHFC)
        JPMAC=LCMGID(IPMAC,'GROUP')
        DO JGR=1,NGRP
          KPMAC=LCMGIL(JPMAC,JGR)
          CALL LCMLEN(KPMAC,'H-FACTOR',LENGT,ITYP)
          IF(LENGT.NE.NMIX)CALL XABORT('@FLPDRV: UNABLE TO FIND'
     1     //' H-FACTOR BLOCK DATA IN THE MACROLIB.')
          CALL LCMGET(KPMAC,'H-FACTOR',BASE(IHFC+(JGR-1)*NMIX))
        ENDDO
      ENDIF
*----
*  FLUX NORMALIZATION
*----
      CALL SETARA(BASE,NUN*NGRP,IVECT)
      CALL SETARA(BASE,NEL*NGRP,IFLUX)
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(IBASE,NEL,IIDL)
      CALL SETARA(BASE,NEL,IVOL)
      IF(LNEW)THEN
*       NEW TOTAL REACTOR POWER
        CALL LCMLEN(IPPOW,'NORM',LENGT,ITYP)
        IF(LENGT.EQ.0)CALL XABORT('@FLPDRV: UNABLE TO F'
     1   //'IND FLUX NORMALIZATION FACTOR IN L_POWER.')
        CALL LCMGET(IPPOW,'NORM',ZNRM)
        CALL FLPTOT(IPFLX,IPKIN,IPTRK,NMIX,NGRP,NEL,NUN,BASE(IVECT),
     1  BASE(IFLUX),IBASE(IMAT),BASE(IVOL),IBASE(IIDL),BASE(IHFC),POWR,
     2  ZNRM,IMPX)
      ELSE
*       GIVEN TOTAL REACTOR POWER
        POWR=DBLE(PTOT*10**6)
        IF(PTOT.EQ.0.0)CALL XABORT('@FLPDRV: PTOT IS NOT DEFINED')
        CALL FLPNRM(IPFLX,IPKIN,IPTRK,NMIX,NGRP,NEL,NUN,BASE(IVECT),
     1  BASE(IFLUX),IBASE(IMAT),BASE(IVOL),IBASE(IIDL),BASE(IHFC),POWR,
     2  ZNRM,IMPX)
        CALL LCMPUT(IPPOW,'NORM',2,4,ZNRM)
      ENDIF
      CALL RLSARA(IBASE(IIDL))
      CALL LCMPUT(IPPOW,'FLUX',NEL*NGRP,2,BASE(IFLUX))
      POWR=POWR/(10**6)
      CALL LCMPUT(IPPOW,'PTOT',2,4,POWR)
*----
*  WHOLE REACTOR
*----
      CALL SETARA(BASE,NEL*NGRP,IFXYZ)
      CALL SETARA(BASE,NEL*(NGRP-1),IRAT)
*     FLUX DISTRIBUTION
      IF(IGEO.EQ.7) THEN
        CALL FLPFLX(NGRP,NEL,LX,LY,LZ,IBASE(IMAT),BASE(IVOL),
     1   BASE(IFLUX),BASE(IFXYZ),BASE(IRAT),VTOT,IMPX,LFLX,LRAT)
      ELSEIF(IGEO.EQ.9) THEN
        CALL FLPHFX(NGRP,NEL,LX,LY,LZ,IBASE(IMAT),BASE(IVOL),
     1   BASE(IFLUX),BASE(IFXYZ),BASE(IRAT),VTOT,IMPX,LFLX,LRAT)
      ENDIF
      CALL LCMPUT(IPPOW,'VTOT',2,4,VTOT)
      CALL LCMPUT(IPPOW,'FLUX-DISTR',NEL*NGRP,2,BASE(IFXYZ))
      CALL LCMPUT(IPPOW,'FLUX-RATIO',NEL*(NGRP-1),2,BASE(IRAT))
      CALL RLSARA(BASE(IRAT))
      CALL RLSARA(BASE(IFXYZ))
*     POWER DISTRIBUTION
      CALL SETARA(BASE,NEL,IPXYZ)
      IF(IGEO.EQ.7) THEN
        CALL FLPOWR(NMIX,NGRP,NEL,LX,LY,LZ,IBASE(IMAT),BASE(IVOL),
     1   BASE(IFLUX),BASE(IHFC),BASE(IPXYZ),VTOT,IMPX,LPOW)
      ELSEIF(IGEO.EQ.9) THEN
        CALL FLPHPW(NMIX,NGRP,NEL,LX,LY,LZ,IBASE(IMAT),BASE(IVOL),
     1   BASE(IFLUX),BASE(IHFC),BASE(IPXYZ),VTOT,IMPX,LPOW)
      ENDIF
      CALL LCMPUT(IPPOW,'POWER-DISTR',NEL,2,BASE(IPXYZ))
      CALL RLSARA(BASE(IPXYZ))
      CALL LCMPUT(IPPOW,'K-EFFECTIVE',1,2,FKEFF)
*----
*  FUEL-MAP
*----
      IF((IPMAP.NE.0).AND.(IPMTX.NE.0)) THEN
        CALL SETARA(IBASE,NCH*NB,IFMIX)
        CALL SETARA(BASE,NCH*NB,IVMAP)
        CALL SETARA(BASE,NCH*NB*NGRP,IFMAP)
        CALL LCMGET(IPMAP,'FLMIX',IBASE(IFMIX))
*       COMPUTE FLUXES
        CALL FLPFLB(IPMTX,NTOT,NGRP,NEL,NCH,NB,BASE(IFLUX),BASE(IVOL),
     1  IBASE(IFMIX),BASE(IVMAP),BASE(IFMAP),IMPX,LMAP)
        CALL SETARA(BASE,NCH,IPOWC)
        CALL SETARA(BASE,NCH*NB,IPOWB)
*       COMPUTE POWERS
        CALL FLPOWB(IPPOW,IPMAP,IPMTX,NMIX,NTOT,NGRP,NCH,NB,NEL,
     1  IBASE(IMAT),BASE(IVOL),BASE(IHFC),BASE(IFLUX),BASE(IPOWB),
     2  BASE(IPOWC),IMPX,POWR,FSTH,LFSTH,IBASE(IFMIX),BASE(IFMAP),
     3  IGEO)
        CALL LCMPUT(IPPOW,'POWER-CHAN',NCH,2,BASE(IPOWC))
        CALL LCMPUT(IPPOW,'POWER-BUND',NCH*NB,2,BASE(IPOWB))
        CALL LCMPUT(IPPOW,'FLUX',NEL*NGRP,2,BASE(IFLUX))
        CALL LCMPUT(IPPOW,'VOLU-BUND',NCH*NB,2,BASE(IVMAP))
        CALL LCMPUT(IPPOW,'FLUX-BUND',NCH*NB*NGRP,2,BASE(IFMAP))
        CALL LCMPUT(IPPOW,'FLMIX',NCH*NB,1,IBASE(IFMIX))
        IF(JMOD.GE.1) THEN
          CALL LCMPUT(IPMAP,'TOT-PW',1,2,PTOT)
          CALL LCMPUT(IPMAP,'BUND-PW',NCH*NB,2,BASE(IPOWB))
          CALL LCMPUT(IPMAP,'FLUX-AV',NCH*NB*NGRP,2,BASE(IFMAP))
          IF(JMOD.EQ.2) THEN
            CALL LCMPUT(IPMAP,'BUND-PW-INI',NCH*NB,2,BASE(IPOWB))
          ENDIF
        ENDIF
        CALL RLSARA(BASE(IPOWB))
        CALL RLSARA(BASE(IPOWC))
      ENDIF
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      CALL RLSARA(BASE(IFLUX))
      CALL RLSARA(BASE(IHFC))

      IF(LFLU) THEN
*----
*  STATE-VECTOR FOR L_FLUX OBJECT
*----
        IF(LNRM) THEN
          CALL XDISET(ISTATE,NSTATE,0)
          IF(IPFLX.NE.0) THEN
            CALL LCMGET(IPFLX,'STATE-VECTOR',ISTATE)
          ELSE IF(IPKIN.NE.0) THEN
            ISTATE(1)=NGRP
            ISTATE(2)=NUN
          ENDIF
          CALL LCMPUT(IPNFX,'STATE-VECTOR',NSTATE,1,ISTATE)
          HSIGN='L_FLUX'
          READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
          CALL LCMPUT(IPNFX,'SIGNATURE',3,3,KCHAR)
          CALL SETARA(BASE,NUN*NGRP,IVECNR)
          DO 10 I=1,NUN*NGRP
            BASE(IVECNR+I-1)=BASE(IVECT+I-1)*ZNRM
   10     CONTINUE
          MPFLUX=LCMLID(IPNFX,'FLUX',NGRP)
          DO 20 IGR=1,NGRP
            IOFSET=(IGR-1)*NUN
            CALL LCMPDL(MPFLUX,IGR,NUN,2,BASE(IVECNR+IOFSET))
   20     CONTINUE        
          CALL RLSARA(BASE(IVECNR))
        ELSE
          MPFLUX=LCMLID(IPNFX,'FLUX',NGRP)
          DO 30 IGR=1,NGRP
              IOFSET=(IGR-1)*NB*NCH
              CALL LCMPDL(MPFLUX,IGR,NB*NCH,2,BASE(IFMAP+IOFSET))
   30     CONTINUE 
        ENDIF
      ENDIF
      IF(IPMAP.NE.0) THEN
        CALL RLSARA(IBASE(IFMIX))
        CALL RLSARA(BASE(IVMAP))
        CALL RLSARA(BASE(IFMAP))
      ENDIF
      CALL RLSARA(BASE(IVECT))
*----
*  STATE-VECTOR FOR L_FLUX OBJECT
*----
      IF(LFLU) THEN
        CALL XDISET(ISTATE,NSTATE,0)
        IF(IPFLX.NE.0) THEN
          CALL LCMGET(IPFLX,'STATE-VECTOR',ISTATE)
        ELSE IF(IPKIN.NE.0) THEN
          ISTATE(1)=NGRP
          ISTATE(2)=NUN
        ENDIF
        IF(LBUN) ISTATE(2)=NB*NCH
        CALL LCMPUT(IPNFX,'STATE-VECTOR',NSTATE,1,ISTATE)
        HSIGN='L_FLUX'
        READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        CALL LCMPUT(IPNFX,'SIGNATURE',3,3,KCHAR)
      ENDIF
*----
*  STATE-VECTOR FOR L_POWER OBJECT
*----
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NGRP
      ISTATE(2)=NEL
      ISTATE(3)=LX
      ISTATE(4)=LY
      ISTATE(5)=LZ
      ISTATE(6)=NCH
      ISTATE(7)=NB
      ISTATE(8)=IGEO
      CALL LCMPUT(IPPOW,'STATE-VECTOR',NSTATE,1,ISTATE)
      HSIGN='L_POWER'
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPPOW,'SIGNATURE',3,3,KCHAR)
      IF(IMPX.GT.1)CALL LCMLIB(IPPOW)
      RETURN
      END
