*DECK NCR
      SUBROUTINE NCR(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
*  recover and interpolate microlib and macrolib information from one
*  or many multicompo database objects.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert and R. Chambon
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_LIBRARY);
*         HENTRY(I): I>1 read-only type(L_MULTICOMPO) multicompo object;
*         HENTRY(NENTRY): read-only type(L_MAP) fuel-map object.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,IOUT=6,MAXISD=400)
      CHARACTER TEXT12*12,HSMG*131,HSIGN*12,NAMDIR*12
      LOGICAL LMACRO,LCUBIC,LBASE(1),LXS,LRES
      DOUBLE PRECISION DFLOTT
      INTEGER KCHAR(3),ISTATE(NSTATE),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),LBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.1) CALL XABORT('NCR: MINIMUM OF 2 OBJECTS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('NCR: MICRO'
     1 //'LIB LCM OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('NCR: MICRO'
     1 //'LIB IN CREATE OR MODIFICATION MODE EXPECTED.')
      IACCS=JENTRY(1)
      IPLIB=KENTRY(1)
      IPLIB2=0
      IPMAP=0
      NGRP=0
      NMIX=0
      IF(IACCS.EQ.1) THEN
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          TEXT12=HENTRY(1)
          CALL XABORT('NCR: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1    '. L_LIBRARY EXPECTED.')
        ENDIF
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NGRP=ISTATE(3)
        NMIX=ISTATE(1)
      ENDIF
      DO 10 I=2,NENTRY
        IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2)) CALL XABORT('NCR: '
     1 //'LCM OBJECTS EXPECTED AT RHS.')
        IF(JENTRY(I).NE.2) CALL XABORT('NCR: LCM OBJECTS IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
        CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(HSIGN.EQ.'L_LIBRARY') THEN
          IF(IPLIB2.NE.0) CALL XABORT('NCR: ONLY ONE MICROLIB EXPECTED '
     1   //'AT RHS.')
          IPLIB2=KENTRY(I)
          GO TO 10
        ELSE IF(HSIGN.EQ.'L_MAP') THEN
          IF(I.NE.NENTRY)CALL XABORT('NCR: FUEL-MAP EXPECTED TO BE THE '
     1    //'LAST OBJECT.')
          IF(NENTRY.LT.3)CALL XABORT('NCR: MISSING MULTI-COMPO OBJECT.')
          IPMAP=KENTRY(NENTRY)
          CALL LCMLEN(IPMAP,'FLMIX',NMIX,ITYP)
        ELSE IF(HSIGN.NE.'L_MULTICOMPO') THEN
          TEXT12=HENTRY(I)
          CALL XABORT('NCR: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_MULTICOMPO EXPECTED.')
        ENDIF
   10 CONTINUE
*----
*  READ THE INPUT DATA
*----
      LMACRO=.TRUE.
      LXS=.FALSE.
      LCUBIC=.TRUE.
      LRES=.FALSE.
      B2=0.0
      MAXFEL=0
      ITER=-1
      IPCPO=0
      IMPX=1
   20 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(1).')
   30 IF(TEXT12.EQ.'EDIT') THEN
*       READ THE PRINT INDEX.
        CALL REDGET(INDIC,IMPX,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('NCR: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT12.EQ.'NMIX') THEN
*       READ THE MAXIMUM NUMBER OF MATERIAL MIXTURES.
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('NCR: INTEGER DATA EXPECTED(2).')
        IF(NITMA.LT.NMIX) THEN
           WRITE(HSMG,'(20HNCR: NMIX MUST BE >=,I8)') NMIX
           CALL XABORT(HSMG)
        ENDIF
        NMIX=NITMA
      ELSE IF(TEXT12.EQ.'MACRO') THEN
        LMACRO=.TRUE.
        IF(IACCS.NE.0) CALL XABORT('NCR: LHS EXPECTED IN CREATION.')
      ELSE IF(TEXT12.EQ.'MICRO') THEN
        LMACRO=.FALSE.
      ELSE IF(TEXT12.EQ.'LINEAR') THEN
        LCUBIC=.FALSE.
      ELSE IF(TEXT12.EQ.'CUBIC') THEN
        LCUBIC=.TRUE.
      ELSE IF(TEXT12.EQ.'ALLX') THEN
        LXS=.TRUE.
        CALL REDGET(INDIC,MAXFEL,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('NCR: INTEGER DATA EXPECTED(3).')
      ELSE IF(TEXT12.EQ.'RES') THEN
        IF((IACCS.EQ.0).AND.(IPLIB2.EQ.0)) THEN
          CALL XABORT('NCR: RHS MICROLIB EXPECTED WITH RES OPTION.')
        ENDIF
        LRES=.TRUE.
      ELSE IF(TEXT12.EQ.'COMPO') THEN
        IF(NMIX.EQ.0) CALL XABORT('NCR: ZERO NUMBER OF MIXTURES.')
        IF(IPMAP.NE.0) THEN
          WRITE(IOUT,'(/43H NCR: ***WARNING*** A FUEL MAP IS SET AT RH,
     1    26HS; KEYWORD TABLE EXPECTED.)')
        ENDIF
        IF((IACCS.EQ.0).AND.(IPLIB2.NE.0)) THEN
          CALL LCMEQU(IPLIB2,IPLIB)
          IACCS=1
        ENDIF
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(2).')
        CALL REDGET(INDIC,NITMA,FLOTT,NAMDIR,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(3).')
        I0=0
        DO 50 I=2,NENTRY
          IF(KENTRY(I).EQ.IPLIB2) GO TO 50
          IF(TEXT12.EQ.HENTRY(I)) THEN
            CALL LCMGTC(KENTRY(I),'SIGNATURE',12,1,TEXT12)
            IF(TEXT12.EQ.'L_MULTICOMPO') THEN
              IPCPO=KENTRY(I)
            ELSE
              CALL XABORT('NCR: WRONG SIGNATURE ('//TEXT12//').')
            ENDIF
            ITH=I
            GO TO 60
          ENDIF
   50   CONTINUE
        CALL XABORT('NCR: COMPO '//TEXT12//' NOT FOUND.')
   60   IF(IMPX.GT.0) WRITE(IOUT,310) TEXT12,NAMDIR
        CALL LCMLEN(IPCPO,NAMDIR,ILENG,ITYLCM)
        IF((ILENG.EQ.0).OR.(ITYLCM.NE.0)) THEN
          CALL LCMLIB(IPCPO)
          CALL XABORT('NCR: NO '//NAMDIR//' DIRECTORY TO STEP.')
        ENDIF
        JPCPO=LCMGID(IPCPO,NAMDIR)
        CALL LCMGET(JPCPO,'STATE-VECTOR',ISTATE)
        IF(NGRP.EQ.0) THEN
          NGRP=ISTATE(2)
        ELSE IF(NGRP.NE.ISTATE(2)) THEN
          WRITE(HSMG,'(9H NCR: THE,I4,29H-TH COMPO HAS AN INVALID NUMB,
     1    20HER OF ENERGY GROUPS.)') ITH
          CALL XABORT(HSMG)
        ENDIF
        IF(ISTATE(12).NE.2006) CALL XABORT('NCR: 2006 COMPO SPECIF EXP'
     1   //'ECTED.')
        NMIL=ISTATE(1)
        NCAL=ISTATE(3)
        CALL SETARA(IBASE,NMIX,IMIXC)
        CALL SETARA(BASE,NCAL*NMIX,ITERP)
        CALL SETARA(IBASE,NMIX,INISO)
        CALL SETARA(IBASE,NMIX,ILISO)
        CALL SETARA(IBASE,2*NMIX*MAXISD,IHISO)
        CALL SETARA(BASE,NMIX*MAXISD,ICONC)
*
        CALL SETARA(IBASE,NMIX,IDELT)
        CALL NCRDRV(MAXISD,JPCPO,LCUBIC,NMIX,IMPX,NMIL,NCAL,ITER,
     1  MAXNIS,IBASE(IMIXC),BASE(ITERP),IBASE(INISO),IBASE(ILISO),
     2  IBASE(IHISO),BASE(ICONC),IBASE(IDELT))
        CALL RLSARA(IBASE(IDELT))
        GO TO 100
      ELSE IF(TEXT12.EQ.'TABLE') THEN
        IF(IPMAP.EQ.0) CALL XABORT('NCR: MISSING FUEL-MAP OBJECT.')
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
        NB=ISTATE(1)
        NCH=ISTATE(2)
        NCOMB=ISTATE(3)
        NGRP=ISTATE(4)
        NFUEL=ISTATE(7)
        NPARM=ISTATE(8)
        IF(NCOMB.EQ.0)CALL XABORT('NCR: NUMBER OF COMBUSTION ZONES NO'
     1   //'T YET DEFINED IN THE FUEL MAP NCOMB=0.')
        IF((IACCS.EQ.0).AND.(IPLIB2.NE.0)) THEN
          CALL LCMEQU(IPLIB2,IPLIB)
          IACCS=1
        ENDIF
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(2).')
        CALL REDGET(INDIC,NITMA,FLOTT,NAMDIR,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(3).')
        I0=0
        DO 80 I=2,NENTRY
          IF((KENTRY(I).EQ.IPLIB2).OR.(KENTRY(I).EQ.IPMAP)) GO TO 80
          IF(TEXT12.EQ.HENTRY(I)) THEN
            CALL LCMGTC(KENTRY(I),'SIGNATURE',12,1,TEXT12)
            IF(TEXT12.EQ.'L_MULTICOMPO') THEN
              IPCPO=KENTRY(I)
            ELSE
              CALL XABORT('NCR: WRONG SIGNATURE ('//TEXT12//').')
            ENDIF
            ITH=I
            GO TO 90
          ENDIF
   80   CONTINUE
        CALL XABORT('NCR: COMPO '//TEXT12//' NOT FOUND.')
   90   IF(IMPX.GT.0) WRITE(IOUT,310) TEXT12,NAMDIR
        CALL LCMLEN(IPCPO,NAMDIR,ILENG,ITYLCM)
        IF((ILENG.EQ.0).OR.(ITYLCM.NE.0)) THEN
          CALL LCMLIB(IPCPO)
          CALL XABORT('NCR: NO '//NAMDIR//' DIRECTORY TO STEP.')
        ENDIF
        JPCPO=LCMGID(IPCPO,NAMDIR)
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(JPCPO,'STATE-VECTOR',ISTATE)
        IF(ISTATE(12).NE.2006) CALL XABORT('NCR: 2006 COMPO SPECIF EXP'
     1   //'ECTED.')
        IF(NGRP.NE.ISTATE(2)) THEN
          WRITE(HSMG,'(9H NCR: THE,I4,29H-TH COMPO HAS AN INVALID NUMB,
     1    20HER OF ENERGY GROUPS.)') ITH
          CALL XABORT(HSMG)
        ENDIF
        NMIL=ISTATE(1)
        NCAL=ISTATE(3)
        CALL SETARA(IBASE,NMIX,IMIXC)
        CALL SETARA(BASE,NCAL*NMIX,ITERP)
        CALL SETARA(IBASE,NMIX,INISO)
        CALL SETARA(IBASE,NMIX,ILISO)
        CALL SETARA(IBASE,2*NMIX*MAXISD,IHISO)
        CALL SETARA(BASE,NMIX*MAXISD,ICONC)
*
        CALL SETARA(BASE,NCH*NB,IBURN0)
        CALL SETARA(BASE,NCH*NB,IBURN1)
        CALL SETARA(IBASE,NCH*NB,IFMIX)
        CALL SETARA(BASE,(NPARM+1)*NCH*NB,IWPAR)
        CALL SETARA(IBASE,(NPARM+1)*3,IHPAR)
        CALL SETARA(IBASE,NCH*NB,IZONED)
        CALL SETARA(IBASE,NCH,IZONEC)
        CALL SETARA(IBASE,NMIX,IDELT)
        CALL NCRRGR(MAXISD,JPCPO,IPMAP,LCUBIC,NMIX,IMPX,NMIL,
     1  NCAL,NCH,NB,NFUEL,IBASE(IFMIX),BASE(IBURN0),BASE(IBURN1),
     2  BASE(IWPAR),IBASE(IZONED),IBASE(IZONEC),NPARM,IBASE(IHPAR),
     3  ITER,MAXNIS,IBASE(IMIXC),BASE(ITERP),IBASE(INISO),IBASE(ILISO),
     4  IBASE(IHISO),BASE(ICONC),IBASE(IDELT))
        CALL RLSARA(IBASE(IDELT))
        CALL RLSARA(IBASE(IZONEC))
        CALL RLSARA(IBASE(IZONED))
        CALL RLSARA(IBASE(IHPAR))
        CALL RLSARA(BASE(IWPAR))
        CALL RLSARA(IBASE(IFMIX))
        CALL RLSARA(BASE(IBURN1))
        CALL RLSARA(BASE(IBURN0))
        GO TO 100
      ELSE IF(TEXT12.EQ.'LEAK') THEN
        CALL REDGET(INDIC,NITMA,B2,TEXT12,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('NCR: REAL DATA EXPECTED.')
      ELSE
        CALL XABORT('NCR: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
*----
*  BUILD THE INTERPOLATED MACROLIB OR MICROLIB
*----
 100  IF(LMACRO) THEN
*       build a macrolib
        MAX1D=40
        MAX2D=20
        MAXNL=5
        MAXNFI=50
        CALL SETARA(BASE,NGRP*2,IFLUX)
        CALL SETARA(BASE,NMIX*NGRP*MAX1D,IGAR1)
        CALL SETARA(BASE,NMIX*NGRP*MAXNFI*MAX2D,IGAR2)
        CALL SETARA(BASE,NMIX*NGRP*NGRP*MAXNL,IGAR3)
        CALL SETARA(BASE,NMIX*NGRP,IGAR4)
        CALL SETARA(BASE,NGRP+1,IENER)
        CALL SETARA(IBASE,NMIX,INJJ)
        CALL SETARA(IBASE,NMIX,IIJJ)
        CALL SETARA(IBASE,NMIX,IIPOS)
        CALL NCRMAC(MAXNIS,IPLIB,JPCPO,IACCS,NMIL,NMIX,NGRP,IMPX,NCAL,
     1  BASE(ITERP),IBASE(INISO),IBASE(ILISO),IBASE(IHISO),BASE(ICONC),
     2  IBASE(IMIXC),LRES,B2,BASE(IFLUX),BASE(IGAR1),BASE(IGAR2),
     3  BASE(IGAR3),BASE(IGAR4),BASE(IENER),IBASE(INJJ),IBASE(IIJJ),
     4  IBASE(IIPOS))
        CALL RLSARA(IBASE(IIPOS))
        CALL RLSARA(IBASE(IIJJ))
        CALL RLSARA(IBASE(INJJ))
        CALL RLSARA(BASE(IENER))
        CALL RLSARA(BASE(IGAR4))
        CALL RLSARA(BASE(IGAR3))
        CALL RLSARA(BASE(IGAR2))
        CALL RLSARA(BASE(IGAR1))
        CALL RLSARA(BASE(IFLUX))
*
        IF(IMPX.GT.0) THEN
          CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
          WRITE(IOUT,290) IMPX,(ISTATE(I),I=1,7),ISTATE(9),ISTATE(12)
        ENDIF
      ELSE
*       build a microlib
        IF(IACCS.EQ.0)THEN
          MAXISO=MAXISD*NMIX
        ELSE
          CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
          MAXISO=MAX(MAXISD*NMIX,ISTATE(2))
        ENDIF
        CALL SETARA(IBASE,3*MAXISO,IHUSE1)
        CALL SETARA(IBASE,3*MAXISO,IHNAM1)
        CALL SETARA(BASE,MAXISO,IDENS1)
        CALL SETARA(IBASE,MAXISO,IITYP1)
        CALL SETARA(IBASE,MAXISO,IITOD1)
        CALL SETARA(BASE,MAXISO,ITEMP1)
        CALL SETARA(BASE,MAXISO,IVOL1)
        CALL SETARA(IBASE,3*MAXISO,IHUSE2)
        CALL SETARA(IBASE,3*MAXISO,IHNAM2)
        CALL SETARA(BASE,MAXISO,IDENS2)
        CALL SETARA(IBASE,MAXISO,IIMIX2)
        CALL SETARA(IBASE,MAXISO,IITYP2)
        CALL SETARA(IBASE,MAXISO,IITOD2)
        CALL SETARA(BASE,MAXISO,ITEMP2)
        CALL SETARA(BASE,MAXISO,IVOL2)
        CALL SETARA(BASE,NGRP+1,IENER)
        CALL SETARA(BASE,NGRP,IDELT)
        CALL SETARA(BASE,NCAL,IFACT)
        CALL SETARA(IBASE,NMIX,IMILVO)
        CALL SETARA(BASE,NMIX,IVOLMI)
        CALL SETARA(BASE,MAXISO,IDENS3)
        CALL SETARA(BASE,MAXISO,ITEMP3)
        CALL SETARA(BASE,MAXISO,IVOL3)
        CALL NCRLIB(MAXNIS,MAXISO,MAXFEL,IPLIB,JPCPO,IACCS,NMIX,NGRP,
     1  IMPX,NCAL,BASE(ITERP),IBASE(INISO),IBASE(ILISO),IBASE(IHISO),
     2  BASE(ICONC),IBASE(IMIXC),LXS,LRES,B2,BASE(IHUSE1),BASE(IHNAM1),
     3  BASE(IDENS1),BASE(IITYP1),BASE(IITOD1),BASE(ITEMP1),BASE(IVOL1),
     4  BASE(IHUSE2),BASE(IHNAM2),BASE(IDENS2),BASE(IIMIX2),
     5  BASE(IITYP2),BASE(IITOD2),BASE(ITEMP2),BASE(IVOL2),BASE(IENER),
     6  BASE(IDELT),BASE(IFACT),BASE(IMILVO),BASE(IVOLMI),BASE(IDENS3),
     7  BASE(ITEMP3),BASE(IVOL3))
        CALL RLSARA(BASE(IVOL3))
        CALL RLSARA(BASE(ITEMP3))
        CALL RLSARA(BASE(IDENS3))
        CALL RLSARA(BASE(IVOLMI))
        CALL RLSARA(IBASE(IMILVO))
        CALL RLSARA(BASE(IFACT))
        CALL RLSARA(BASE(IDELT))
        CALL RLSARA(BASE(IENER))
        CALL RLSARA(BASE(IVOL2))
        CALL RLSARA(BASE(ITEMP2))
        CALL RLSARA(IBASE(IITOD2))
        CALL RLSARA(IBASE(IITYP2))
        CALL RLSARA(IBASE(IIMIX2))
        CALL RLSARA(BASE(IDENS2))
        CALL RLSARA(IBASE(IHNAM2))
        CALL RLSARA(IBASE(IHUSE2))
        CALL RLSARA(BASE(IVOL1))
        CALL RLSARA(BASE(ITEMP1))
        CALL RLSARA(IBASE(IITOD1))
        CALL RLSARA(IBASE(IITYP1))
        CALL RLSARA(BASE(IDENS1))
        CALL RLSARA(IBASE(IHNAM1))
        CALL RLSARA(IBASE(IHUSE1))
*
        IF(IMPX.GT.0) THEN
          CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
          WRITE(IOUT,300) IMPX,(ISTATE(I),I=1,15),(ISTATE(I),I=17,24)
        ENDIF
      ENDIF
*
      CALL RLSARA(IBASE(ILISO))
      CALL RLSARA(IBASE(INISO))
      CALL RLSARA(IBASE(IHISO))
      CALL RLSARA(BASE(ICONC))
      CALL RLSARA(BASE(ITERP))
      CALL RLSARA(IBASE(IMIXC))
*----
*  CONTINUE DATA PROCESSING
*----
      IF(ITER.EQ.0) THEN
        GO TO 200
      ELSE IF(ITER.EQ.1) THEN
        TEXT12='COMPO'
        GO TO 30
      ELSE IF(ITER.EQ.2) THEN
        TEXT12='TABLE'
        GO TO 30
      ENDIF
*----
*  LEAVE NCR:
*----
  200 IF(IMPX.GT.2) CALL LCMLIB(IPLIB)
      RETURN
*
  290 FORMAT(/8H OPTIONS/8H -------/
     1 7H IMPX  ,I6,30H   (0=NO PRINT/1=SHORT/2=MORE)/
     2 7H NGROUP,I6,28H   (NUMBER OF ENERGY GROUPS)/
     3 7H NBMIX ,I6,39H   (NUMBER OF MIXTURES IN THE MACROLIB)/
     4 7H NANISO,I6,34H   (MAXIMUM SCATTERING ANISOTROPY)/
     5 7H NIFISS,I6,45H   (MAXIMUM NUMBER OF FISSILE ISOTOPES IN A M,
     6 7HIXTURE)/
     7 7H NEDMAC,I6,34H   (NUMBER OF CROSS SECTION EDITS)/
     8 7H ITRANC,I6,45H   (0=NO TRANSPORT CORRECTION/1=APOLLO TYPE/2,
     9 43H=RECOVER FROM LIBRARY/4=LEAKAGE CORRECTION)/
     1 7H NLG   ,I6,39H   (NUMBER OF DELAYED PRECURSOR GROUPS)/
     2 7H ILEAK ,I6,40H   (1=DIFF AVAILABLE; 2=DIFFX AVAILABLE)/
     3 7H IDF   ,I6,45H   (0=NO ADF INFO/1=ALBS INFO/2=ADF GAP INFO))
  300 FORMAT(/8H OPTIONS/8H -------/
     1 7H IMPX  ,I6,30H   (0=NO PRINT/1=SHORT/2=MORE)/
     2 7H MAXMIX,I6,31H   (MAXIMUM NUMBER OF MIXTURES)/
     3 7H NBISO ,I6,36H   (NUMBER OF ISOTOPES OR MATERIALS)/
     4 7H NGRP  ,I6,28H   (NUMBER OF ENERGY GROUPS)/
     5 7H NL    ,I6,30H   (NUMBER OF LEGENDRE ORDERS)/
     6 7H ITRANC,I6,45H   (0=NO TRANSPORT CORRECTION/1=APOLLO TYPE/2,
     7 57H=RECOVER FROM LIBRARY/3=WIMS-D TYPE/4=LEAKAGE CORRECTION)/
     8 7H IPROB ,I6,23H   (0=DIRECT/1=ADJOINT)/
     9 7H ITIME ,I6,28H   (1=STEADY-STATE/2=PROMPT)/
     1 7H NLIB  ,I6,32H   (NUMBER OF SETS OF LIBRARIES)/
     2 7H NGF   ,I6,48H   (NUMBER OF FAST GROUP WITHOUT SELF-SHIELDING)/
     3 7H IGRMAX,I6,41H   (LAST GROUP INDEX WITH SELF-SHIELDING)/
     4 7H NDEPL ,I6,33H   (NUMBER OF DEPLETING ISOTOPES)/
     5 7H NCOMB ,I6,33H   (NUMBER OF DEPLETING MIXTURES)/
     6 7H NEDMAC,I6,34H   (NUMBER OF CROSS SECTION EDITS)/
     7 7H NBMIX ,I6,23H   (NUMBER OF MIXTURES)/
     8 7H NRES  ,I6,40H   (NUMBER OF SETS OF RESONANT MIXTURES)/
     9 7H IPROC ,I6,48H   (-1=SKIP LIBRARY PROCESSING/0=DILUTION INTERP,
     1 48HOLATION/1=USE PHYSICAL TABLES/2=BUILD A DRAGLIB/,
     2 55H3=COMPUTE CALENDF TABLES/4=COMPUTE SLOWING-DOWN TABLES)/
     3 7H IMAC  ,I6,45H   (0=DO NOT/1=DO BUILD AN EMBEDDED MACROLIB)/
     4 7H NDEL  ,I6,31H   (NUMBER OF PRECURSOR GROUPS)/
     5 7H NFISS ,I6,31H   (NUMBER OF FISSILE ISOTOPES)/
     6 7H ISOADD,I6,37H   (0=COMPLETE BURNUP CHAIN/1=DO NOT)/
     7 7H MAXISM,I6,40H   (MAX. NUMBER OF ISOTOPES PER MIXTURE)/
     8 7H IPRECI,I6,34H   (CALENDF ACCURACY FLAG:1/2/3/4)/
     9 7H IDF   ,I6,45H   (0=NO ADF INFO/1=ALBS INFO/2=ADF GAP INFO))
  310 FORMAT(/32H NCR: INTERPOLATING MULTICOMPO ',A12,13H' FROM DIRECT,
     1 5HORY ',A12,2H'.)
      END
