*DECK PLQ
      SUBROUTINE PLQ(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solves a linear optimization problem with a quadratic constraint.
* PLQ = Quasi Linear Programmation 
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation type(L_OPTIMIZE).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*                          
*-----------------------------------------------------------------------
*
      IMPLICIT  NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER   NITMA,ITYP,ITYP1
      DOUBLE PRECISION DFLOTT
      REAL      FLOTT
      CHARACTER TEXT12*12,HSIGN*12
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   OPTPRI(NSTATE)
      REAL      OPTPRR(NSTATE)
      INTEGER   IPOPT
      INTEGER   I,NVAR,NCST,LENGT,IPRINT,NSTPEX,IMTHD,M0,MINMAX,CNVTST,
     1          IERR,CONV
      INTEGER   GRAD,IDX,IDF,DX,GF,INEGAL,CONTR,GRAD0, VALMAX,VALMIN,
     1          DINF,DSUP,APLUS,BPLUS,INPLUS
      REAL      XDROIT,XS,XXS,EPS4,EPSIM,ECOST,DELTA,S,NORM,EPSEXT
      DOUBLE PRECISION  FCOST
      LOGICAL   LSAVE,LNORM2,LWAR
*
      INTEGER   VARVAL,VARWGT,FCSTV,VARVL2
      REAL      COST,CQUAD
*
      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*----
*  CHECK THE VALIDITY OF OBJECTS
*----
      IF(NENTRY.NE.1) CALL XABORT('PLQ:ONE OBJECT EXPECTED.')
      IF(JENTRY(1).NE.1) CALL XABORT('PLQ: OBJECT IN MODIFICATION '
     1  //'MODE ONLY')
      IPOPT=KENTRY(1)
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))CALL XABORT('PLQ:'
     1      //' LCM OBJECT EXPECTED')
      CALL LCMGTC(KENTRY(1),'SIGNATURE',12,1,HSIGN)
      IF(HSIGN.NE.'L_OPTIMIZE') THEN
         TEXT12=HENTRY(1)
         CALL XABORT('PLQ: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_OPTIMIZE EXPECTED.')
      ENDIF
*----
*  RECOVER STATE VECTOR INFORMATION	
*----
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR  =OPTPRI(1)
      NCST  =OPTPRI(2)
      MINMAX=OPTPRI(3)
      IF((MINMAX.NE.1).AND.(MINMAX.NE.-1)) CALL XABORT('PLQ: '
     1  //'MINMAX not equal to 1 or -1')
      NSTPEX=OPTPRI(5)
      IMTHD =OPTPRI(9)
      CALL LCMGET(IPOPT,'OPT-PARAM-R',OPTPRR)
      S     =OPTPRR(1)
      ECOST =OPTPRR(6)
      EPSEXT=OPTPRR(3)
      EPSIM =OPTPRR(4)
      EPS4  =OPTPRR(5)
*----
*  SET CONTROL-VARIABLE VALUES
*----
      CALL SETARA(RBASE,NVAR,VARVAL)
      CALL LCMLEN(IPOPT,'VAR-VALUE',LENGT,ITYP)
      IF(LENGT.NE.NVAR) CALL XABORT('PLQ: WRONG NUMBER OF VARIABLE')
      CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(VARVAL))
*----
*  SET CONTROL-VARIABLE WEIGHTS
*----
      CALL SETARA(RBASE,NVAR,VARWGT)
      CALL LCMLEN(IPOPT,'VAR-WEIGHT',LENGT,ITYP)
      IF(LENGT.EQ.0) THEN
         CALL XDRSET(RBASE(VARWGT),NVAR,1.0)
      ELSEIF(LENGT.EQ.NVAR) THEN
         CALL LCMGET(IPOPT,'VAR-WEIGHT',RBASE(VARWGT))
      ELSE
         CALL XABORT('PQL: NVAR - LENGT ARE NOT THE SAME')
      ENDIF
*----
*  SET SYSTEM CHARACTERISTICS (THE OBJECTIVE FUNCTION IS THE FIRST ONE)
*----
      CALL SETARA(RBASE,NCST+1,FCSTV)
      CALL LCMLEN(IPOPT,'FOBJ-CST-VAL',LENGT,ITYP)
      IF(LENGT.EQ.0) CALL XABORT('PLQ: OBJECTIVE FUNCTION AND CONSTRA'
     1  //'INTS NOT YET EVALUATED')
      CALL LCMGET(IPOPT,'FOBJ-CST-VAL',RBASE(FCSTV))
      COST=RBASE(FCSTV)
*----
*  SET GRADIENTS
*----
      CALL SETARA(RBASE,NVAR*(NCST+1),GRAD)
      CALL LCMGET(IPOPT,'GRADIENT',RBASE(GRAD))
*----
*  MEMORY ALLOCATION
*----
      CALL SETARA(RBASE,NVAR,IDX)
      CALL SETARA(RBASE,NVAR,IDF)
      IPRINT=0
      LWAR=.FALSE.
*----
*  READ USER INPUT:
*----
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
*     Edition level
   30 IF(ITYP.NE.3) CALL XABORT('PLQ: CHARACTER EXPECTED')
      IF(TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('PLQ: *IPRINT* MUST BE INTEGER')
      ELSEIF(TEXT12.EQ.'WARNING-ONLY')THEN
*        Warning Only for failure of recovery of a valid point
         LWAR=.TRUE.
      ELSEIF(TEXT12.EQ.'CALCUL-DX')THEN
*        Calculation of next point
         GO TO 100
      ELSEIF(TEXT12.EQ.'COST-EXTRAP')THEN
*        Cost extrapolation
         GO TO 200
      ELSEIF(TEXT12.EQ.'CONV-TEST') THEN
*        Convergence test
         GO TO 300
      ELSEIF( TEXT12.EQ.';' )THEN
*        End of this subroutine
         GO TO 1000
      ELSE
         CALL XABORT('PLQ: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
*----
*  NEXT STEP CALCULATION
*----
  100 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(TEXT12.EQ.'NO-STORE-OLD') THEN
         LSAVE=.TRUE.
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      ELSE
         LSAVE=.FALSE.
      ENDIF
      ITYP1=ITYP
      CALL SETARA(RBASE,NVAR,DX)
      FCOST=DBLE(COST)
      IF(NCST.GT.0) THEN
*        INEQUAL
         CALL SETARA(IBASE,NCST,INEGAL)
         CALL LCMLEN(IPOPT,'CST-TYPE',LENGT,ITYP)
         IF(LENGT.NE.NCST) CALL XABORT('PLQ: NCST - LENGT NOT EQUAL')
         CALL LCMGET(IPOPT,'CST-TYPE',IBASE(INEGAL))
*
*        CONTR
         CALL SETARA(RBASE,NCST,CONTR)
         CALL LCMLEN(IPOPT,'CST-OBJ',LENGT,ITYP)
         IF(LENGT.NE.NCST) CALL XABORT('PLQ: NCST - LENGT NOT EQUAL')
         CALL LCMGET(IPOPT,'CST-OBJ',RBASE(CONTR))
         DO 110 I=0,NCST-1
  110    RBASE(CONTR+I) = RBASE(CONTR+I)-RBASE(FCSTV+I+1)
      ENDIF
*
*   DINF AND DSUP
      CALL LCMLEN(IPOPT,'VAR-VAL-MAX',LENGT,ITYP)
      IF(LENGT.EQ.0) CALL XABORT('PLQ: NO MAXIMUM VALUE DEFINED')
      CALL SETARA(RBASE,NVAR,VALMAX)
      CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(VALMAX))
      CALL LCMLEN(IPOPT,'VAR-VAL-MIN',LENGT,ITYP)
      IF(LENGT.EQ.0) CALL XABORT('PLQ: NO MAXIMUM VALUE DEFINED')
      CALL SETARA(RBASE,NVAR,VALMIN)
      CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(VALMIN))
      CALL SETARA(RBASE,NVAR,DINF)
      CALL SETARA(RBASE,NVAR,DSUP)
      DO 115 I=0,NVAR-1
         RBASE(DINF+I) = RBASE(VALMIN+I) - RBASE(VARVAL+I)
  115    RBASE(DSUP+I) = RBASE(VALMAX+I) - RBASE(VARVAL+I)
      CALL RLSARA(RBASE(VALMAX))
      CALL RLSARA(RBASE(VALMIN))
*
      M0     = NCST
      XDROIT = S**2
      IF(IPRINT.GE.1) WRITE(6,40001) XDROIT,(RBASE(VARWGT+I-1),I=1,NVAR)
*----
*  FIND ACTIVE CONSTRAINTS FOR XK(I) LIMITS
*----
      DO 140 I=1,NVAR
         XS = SQRT(XDROIT/RBASE(VARWGT+I-1))
         XXS=-XS
         IF(RBASE(DINF+I-1).GT.XXS) THEN
            M0 = M0 + 1
         ENDIF
         IF(RBASE(DSUP+I-1).LT.XS) THEN
            M0 = M0 + 1
         ENDIF
  140    CONTINUE
*----
*  SOLUTION OF A LINEAR OPTIMIZATION PROBLEM WITH A QUADRATIC CONSTRAINT
*----
      IERR=0
      CALL SETARA(RBASE,2*NVAR,GF)
      CALL SETARA(RBASE,2*(M0+1)*((M0+1)+NVAR),APLUS)
      CALL SETARA(RBASE,M0+2,BPLUS)
      CALL SETARA(IBASE,M0+1,INPLUS)
      CALL PLDRV(IPOPT,NVAR,NCST,M0,MINMAX,IMTHD,FCOST,RBASE(DX),
     > RBASE(VARWGT),RBASE(GRAD),IBASE(INEGAL),RBASE(CONTR),RBASE(DINF),
     > RBASE(DSUP),XDROIT,EPSIM,IPRINT,IERR,RBASE(GF),RBASE(APLUS),
     > RBASE(BPLUS),IBASE(INPLUS))
      CALL RLSARA(IBASE(INPLUS))
      CALL RLSARA(RBASE(BPLUS))
      CALL RLSARA(RBASE(APLUS))
      CALL RLSARA(RBASE(GF))
*----
*  STEP-BACK IN CASE OF FAILURE
*----
      IF(IERR.GE.1) THEN
         OPTPRI(13)=OPTPRI(13)+1
         CALL LCMSIX(IPOPT,'OLD-VALUE',1)
         CALL LCMLEN(IPOPT,'VAR-VALUE2',LENGT,ITYP)
         IF(LENGT.EQ.0) THEN
            IF(LWAR) THEN
               WRITE(6,*) 'WARNING: UNABLE TO RECOVER A VALID POINT'
     1         //' WITH SUCCESSFUL "PLQ" RESOLUTION'
            ELSE
               CALL LCMLIB(IPOPT)
               CALL XABORT('PLQ: UNABLE TO RECOVER A VALID POINT WITH '
     1         //'SUCCESSFUL "PLQ" RESOLUTION')
            ENDIF
         ELSE
            CALL SETARA(RBASE,NVAR,VARVL2)
            CALL LCMGET(IPOPT,'VAR-VALUE2',RBASE(VARVL2))
            DO 156 I=0,NVAR-1
  156       RBASE(DX+I)=(RBASE(VARVL2+I)-RBASE(VARVAL+I))/2.0
            CALL RLSARA(RBASE(VARVL2))
         ENDIF
         CALL LCMSIX(IPOPT,' ',2)
         IF(IPRINT.GE.1) WRITE(6,*) 'IERR>0'
         IF(IPRINT.GE.1) WRITE(6,*) 'DX=',(RBASE(DX+I),I=0,NVAR-1)
      ELSE
         OPTPRI(13)=0
      ENDIF
*
      DO 157 I=0,NVAR-1
         RBASE(IDX+I)=RBASE(DX+I)
  157    RBASE(IDF+I)=RBASE(GRAD+I)
      CALL RLSARA(RBASE(DX))
      IF(NCST.GT.0) CALL RLSARA(IBASE(INEGAL))
      CALL RLSARA(RBASE(DINF))
      CALL RLSARA(RBASE(DSUP))
      IF(NCST.GT.0) CALL RLSARA(RBASE(CONTR))
*----
*  BACKUP VALUES OF THE PRECEDING ITERATION
*----
      IF(.NOT.LSAVE) THEN
         CALL LCMSIX(IPOPT,'OLD-VALUE',1)
         CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(VARVAL))
         CALL LCMPUT(IPOPT,'FOBJ-CST-VAL',NCST+1,2,RBASE(FCSTV))
         CALL LCMPUT(IPOPT,'GRADIENT',NVAR*(NCST+1),2,RBASE(GRAD))
         CALL LCMSIX(IPOPT,' ',2)
      ENDIF
*----
*  BACKUP VALUES OF THE NEW ITERATION
*----
      DO 180 I=0,NVAR-1
  180 RBASE(VARVAL+I)=RBASE(VARVAL+I)+RBASE(IDX+I)
      CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(VARVAL))
      ITYP=ITYP1
*----
*  EXTRAPOLATE OBJECTIVE FUNCTION
*----
      ECOST=COST
      DO 190 I=0,NVAR-1
  190 ECOST=ECOST+RBASE(IDX+I)*RBASE(IDF+I)
*----
*  REINITIALIZE GRADIENTS FOR THE NEXT ITERATION
*----
      CALL SETARA(RBASE,NVAR*(NCST+1),GRAD0)
      CALL XDRSET(RBASE(GRAD0),NVAR*(NCST+1),0.0)
      CALL LCMPUT(IPOPT,'GRADIENT',NVAR*(NCST+1),2,RBASE(GRAD0))
      CALL RLSARA(RBASE(GRAD0))
      GO TO 30
*----
* OUTPUT THE EXTRAPOLATED OBJECTIVE FUNCTION
*----
  200 CALL REDGET(ITYP,NITMA,ECOST,TEXT12,DFLOTT)
      IF(ITYP.NE.-2) CALL XABORT('PLQ: OUTPUT REAL EXPECTED')
      ITYP=2
      CALL REDPUT(ITYP,NITMA,ECOST,TEXT12,DFLOTT)
      GO TO 20
*----
*  TEST CONVERGENCE
*----
  300 LNORM2=.TRUE.
      CALL REDGET(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      IF((ITYP.EQ.3).AND.(TEXT12.EQ.'NORM-INF')) THEN
         LNORM2=.FALSE.
         CALL REDGET(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      ENDIF
      IF(ITYP.NE.-5) CALL XABORT('PLQ: OUTPUT LOGICAL EXPECTED')
      DELTA=ABS((ECOST-COST)/COST)
      NORM=0.0
      CQUAD=0.0
      IF(LNORM2) THEN
         DO 350 I=0,NVAR-1
            NORM=NORM+RBASE(VARWGT+I)*RBASE(VARVAL+I)*RBASE(VARVAL+I)
  350          CQUAD=CQUAD+RBASE(VARWGT+I)*RBASE(IDX+I)*RBASE(IDX+I)
         IF(NORM.NE.0.0) THEN
            CQUAD=SQRT(CQUAD/NORM)
         ELSE
            CQUAD=0.0
         ENDIF
      ELSE
         DO 360 I=0,NVAR-1
            NORM=MAX(NORM,ABS(RBASE(VARWGT+I)**0.5*RBASE(VARVAL+I)))
  360       CQUAD=MAX(CQUAD,ABS(RBASE(VARWGT+I)**0.5*RBASE(IDX+I)))
         IF(NORM.NE.0.0) THEN
            CQUAD=CQUAD/NORM
         ELSE
            CQUAD=0.0
         ENDIF
      ENDIF
      IF(EPSEXT.EQ.0.0) EPSEXT = 0.001
      IF(((DELTA.LT.EPSEXT).AND.(CQUAD.LE.EPSEXT)) .OR.
     1    (CQUAD.LE.(EPSEXT/10.0)))    THEN
         CNVTST=1
         CONV  =1
      ELSE
         CNVTST=-1
         CONV  =0
      ENDIF
      IF(IPRINT.GT.1) THEN
         WRITE(6,*) 'It= convergence?', DELTA,CQUAD,EPSEXT
         IF(IPRINT.GT.2) THEN
            WRITE(6,*) 'DX',(RBASE(IDX+I),I=0,NVAR-1)
            WRITE(6,*) 'X',(RBASE(VARVAL+I),I=0,NVAR-1)
         ENDIF
      ENDIF
      ITYP=5
      CALL REDPUT(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      GO TO 20
*----
*  END
*----
 1000 CALL RLSARA(RBASE(VARWGT))
      CALL RLSARA(RBASE(FCSTV))
      CALL RLSARA(RBASE(GRAD))
      CALL RLSARA(RBASE(IDX))
      CALL RLSARA(RBASE(IDF))
      CALL RLSARA(RBASE(VARVAL))
      OPTPRI(4)=CONV
      OPTPRI(5)=NSTPEX
      OPTPRR(6)=ECOST
      CALL LCMPUT(IPOPT,'OPT-PARAM-R',NSTATE,2,OPTPRR)
      CALL LCMPUT(IPOPT,'STATE-VECTOR',NSTATE,1,OPTPRI)
      RETURN
*
40001 FORMAT(//,5X,'S**2 (XDROIT) = ',1P,E13.5,
     >        /,5X,'FPOIDS  = ',/,(11X,1P,8E13.5))
      END
