*DECK SCRLIB
      SUBROUTINE SCRLIB(MAXNIS,MAXISO,IPLIB,IPSAP,IACCS,NMIX,NGRP,
     1 IMPX,HEQUI,NCAL,ITER,MY1,MY2,MD1,MD2,TERP,NISO,LISO,HISO,CONC,
     2 MIXC,LRES,B2,VTOT,YLDS,DECAYC,HUSE2,HNAM2,DENS2,DENS3,IMIX2,
     3 ITOD2,VOL2,VOLMI2,SPH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* build the Microlib by scanning the NCAL elementary calculations in
* a Saphyb and weighting them with TERP factors.
*
*Copyright:
* Copyright (C) 2012 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXNIS  maximum value of NISO(I) in user data.
* MAXISO  maximum allocated space for output Microlib TOC information.
* IPLIB   address of the output Microlib LCM object.
* IPSAP   address of the Saphyb object.
* IACCS   =0 Microlib is created; =1 ... is updated.
* NMIX    maximum number of material mixtures in the Microlib.
* NGRP    number of energy groups.
* IMPX    print parameter (equal to zero for no print).
* HEQUI   name of SPH-factor set to be recovered.
* NCAL    number of elementary calculations in the Saphyb.
* ITER    completion flag (=0/3: compute the macrolib).
* MY1     number of fissile isotopes including macroscopic sets.
* MY2     number of fission fragment.
* MD1     number of types of radioactive decay reactions.
* MD2     number of isotopes in output tables.
* TERP    interpolation factors.
* NISO    number of user-selected isotopes.
* LISO    type of treatment (=.true.: ALL; =.false.: ONLY).
* HISO    name of the user-selected isotopes.
* CONC    user-defined number density of the user-selected isotopes. A
*         value of -99.99 is set to indicate that the Saphyb value is
*         used.
* MIXC    mixture index in the Saphyb corresponding to each Microlib
*         mixture. Equal to zero if a Microlib mixture is not updated.
* LRES    =.true. if the interpolation is done without updating isotopic 
*         densities.
* B2      buckling.
* VTOT    volume of updated core.
* YLDS    fission yields.
* DECAYC  radioactive decay constants.
*
*Parameters: scratch
* HUSE2
* HNAM2
* DENS2
* DENS3
* IMIX2
* ITOD2
* VOL2
* VOLMI2
* SPH
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXNIS,MAXISO,IPLIB,IPSAP,IACCS,NMIX,NGRP,IMPX,NCAL,ITER,
     1 NISO(NMIX),HISO(2,NMIX,MAXNIS),MIXC(NMIX),HUSE2(3,MAXISO),
     2 IMIX2(MAXISO),ITOD2(MAXISO),HNAM2(3,MAXISO)
      REAL TERP(NCAL,NMIX),CONC(NMIX,MAXNIS),B2,DENS2(MAXISO),
     1 DENS3(MAXISO),VOL2(MAXISO),VOLMI2(NMIX),SPH(NGRP)
      DOUBLE PRECISION VTOT,YLDS(MY1,MY2),DECAYC(MD1,MD2)
      LOGICAL LISO(NMIX),LRES
      CHARACTER HEQUI*80
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXREA=50,MAXMAC=2,MAXDIV=3,MAXLOC=10,IOUT=6)
      CHARACTER TEXT12*12,HSMG*131,HVECT2(MAXREA)*8,NOMREA(MAXREA)*12,
     1 LOCTYP(MAXLOC)*4,LOCNAM(MAXLOC)*80,IDVAL(MAXDIV)*4,HHISO*8,
     2 NOMMAC(MAXMAC)*8,TEXT4*4,TEXT8*8,HRESID*8
      INTEGER ISTATE(NSTATE),KCHAR(3),DIMSAP(50),IBASE(1)
      REAL VALDIV(MAXDIV),TMPDAY(3)
      LOGICAL LUSER,LSPH,LSTRD,LBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),LBASE(1),IBASE(1))
*----
*  MICROLIB INITIALIZATION
*----
      IF(NMIX.GT.9999) CALL XABORT('SCRLIB: NMIX OVERFLOW.')
      IF(IACCS.EQ.0) THEN
         IF(LRES) CALL XABORT('SCRLIB: RES OPTION IS INVALID.')
         NBISO2=0
         NED2=0
         TEXT12='L_LIBRARY'
         READ(TEXT12,'(3A4)') (KCHAR(I0),I0=1,3)
         CALL LCMPUT(IPLIB,'SIGNATURE',3,3,KCHAR)
         CALL XDRSET(VOLMI2,NMIX,0.0)
         CALL XDRSET(DENS2,MAXISO,0.0)
         CALL XDRSET(VOL2,MAXISO,0.0)
         CALL XDISET(IMIX2,MAXISO,0)
         CALL XDISET(ITOD2,MAXISO,0)
      ELSE
         CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
         IF(ISTATE(1).NE.NMIX) CALL XABORT('SCRLIB: INVALID NUMBER OF '
     1   //'MATERIAL MIXTURES IN THE MICROLIB.')
         IF(ISTATE(3).NE.NGRP) CALL XABORT('SCRLIB: INVALID NUMBER OF '
     1   //'ENERGY GROUPS IN THE MICROLIB.')
         NBISO2=ISTATE(2)
         IF(NBISO2.GT.MAXISO) CALL XABORT('SCRLIB: MAXISO OVERFLOW(1).')
         NED2=ISTATE(13)
         IF(NED2.GT.MAXREA) CALL XABORT('SCRLIB: MAXREA OVERFLOW.')
         CALL LCMLEN(IPLIB,'MIXTURESVOL',ILONG,ITYLCM)
         IF(ILONG.GT.0) THEN
           CALL LCMGET(IPLIB,'MIXTURESVOL',VOLMI2)
         ELSE
           CALL XDRSET(VOLMI2,NMIX,0.0)
         ENDIF
         CALL LCMGET(IPLIB,'ISOTOPESUSED',HUSE2)
         CALL LCMGET(IPLIB,'ISOTOPERNAME',HNAM2)
         CALL LCMGET(IPLIB,'ISOTOPESDENS',DENS2)
         CALL LCMGET(IPLIB,'ISOTOPESVOL',VOL2)
         CALL LCMGET(IPLIB,'ISOTOPESMIX',IMIX2)
         CALL LCMGET(IPLIB,'ISOTOPESTODO',ITOD2)
         IF(NED2.GT.0) CALL LCMGTC(IPLIB,'ADDXSNAME-P0',8,NED2,HVECT2)
      ENDIF
*----
*  RECOVER SAPHYB CHARACTERISTICS
*----
      CALL LCMLEN(IPSAP,'DIMSAP',ILENG,ITYLCM)
      IF(ILENG.EQ.0) CALL XABORT('SCRLIB: INVALID SAPHYB.')
      CALL LCMGET(IPSAP,'DIMSAP',DIMSAP)
      IF(NGRP.NE.DIMSAP(20)) THEN
        CALL XABORT('SCRLIB: INVALID VALUE OF NGRP.')
      ENDIF
      NLAM=DIMSAP(3)   ! number of radioactive decay reactions
      NREA=DIMSAP(4)   ! number of neutron-induced reactions
      NISOP=DIMSAP(5)  ! number of particularized isotopes
      NMAC=DIMSAP(6)   ! number of macroscopic sets
      NMIL=DIMSAP(7)   ! number of mixtures in the Saphyb
      NPARL=DIMSAP(11) ! number of local variables
      NADRX=DIMSAP(18) ! number of address sets
      NCALS=DIMSAP(19) ! number of elementary calculations in the Saphyb
      NPRC=DIMSAP(31)  ! number of delayed neutron precursor groups
      NISOTS=DIMSAP(32) ! number of isotopes in edition tables
      IF(IMPX.GT.1) THEN
        WRITE(6,'(30H SCRLIB: number of reactions =,I3)') NREA
        WRITE(6,'(48H SCRLIB: number of radioactive decay reactions =,
     1  I3)') NLAM
        WRITE(6,'(46H SCRLIB: number of neutron-induced reactions =,
     1  I3)') NREA
        WRITE(6,'(44H SCRLIB: number of particularized isotopes =,I4)')
     1  NISOP
        WRITE(6,'(37H SCRLIB: number of macroscopic sets =,I2)') NMAC
        WRITE(6,'(29H SCRLIB: number of mixtures =,I5)') NMIL
        WRITE(6,'(36H SCRLIB: number of local variables =,I4)') NPARL
        WRITE(6,'(33H SCRLIB: number of address sets =,I4)') NADRX
        WRITE(6,'(33H SCRLIB: number of calculations =,I7)') NCALS
        WRITE(6,'(34H SCRLIB: number of energy groups =,I4)') NGRP
        WRITE(6,'(37H SCRLIB: number of precursor groups =,I4)') NPRC
        WRITE(6,'(46H SCRLIB: number of isotopes in output tables =,
     1  I4)') NISOTS
      ENDIF
      IF(NREA.GT.MAXREA) CALL XABORT('SCRLIB: MAXREA OVERFLOW')
      IF(NMAC.GT.MAXMAC) CALL XABORT('SCRLIB: MAXMAC OVERFLOW')
*----
*  RECOVER INFORMATION FROM constphysiq DIRECTORY.
*----
      CALL SETARA(BASE,NGRP+1,IENER)
      CALL LCMSIX(IPSAP,'constphysiq',1)
      CALL LCMGET(IPSAP,'ENRGS',BASE(IENER))
      CALL LCMSIX(IPSAP,' ',2)
      DO IGR=1,NGRP+1
        BASE(IENER+IGR-1)=BASE(IENER+IGR-1)/1.0E-6
      ENDDO
      CALL LCMPUT(IPLIB,'ENERGY',NGRP+1,2,BASE(IENER))
      DO IGR=1,NGRP
        BASE(IENER+IGR-1)=LOG(BASE(IENER+IGR-1)/BASE(IENER+IGR))
      ENDDO
      CALL LCMPUT(IPLIB,'DELTAU',NGRP,2,BASE(IENER))
      CALL RLSARA(BASE(IENER))
*----
*  RECOVER INFORMATION FROM contenu DIRECTORY.
*----
      CALL SETARA(IBASE,NMIL,ITOTM)
      CALL SETARA(IBASE,NMIL,IRESM)
      CALL LCMSIX(IPSAP,'contenu',1)
      IF(NREA.GT.0) THEN
        CALL LCMGTC(IPSAP,'NOMREA',12,NREA,NOMREA)
        IF(IMPX.GT.1) THEN
          WRITE(6,'(29H SCRLIB: Available reactions:/(1X,10A13))')
     1    (NOMREA(I),I=1,NREA)
        ENDIF
      ENDIF
      CALL LCMGET(IPSAP,'TOTMAC',IBASE(ITOTM))
      CALL LCMGET(IPSAP,'RESMAC',IBASE(IRESM))
      CALL SETARA(IBASE,(NISOP+NMAC)*2,INOMIS)
      NBISO1=NISOP
      IF(NISOP.GT.0) CALL LCMGET(IPSAP,'NOMISO',IBASE(INOMIS))
      IF(NMAC.GT.0) THEN
        CALL LCMLEN(IPSAP,'NOMMAC',ILONG,ITYLCM)
        IF(ILONG.GT.2*MAXMAC) CALL XABORT('SCRLIB: MAXMAC OVERFLOW')
        CALL LCMGTC(IPSAP,'NOMMAC',8,NMAC,NOMMAC)
        HHISO='*MAC*RES'
        READ(HHISO,'(2A4)') (IBASE(INOMIS+2*NBISO1+I0),I0=0,1)
        NBISO1=NBISO1+1
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
      IF(NBISO1.EQ.0) CALL XABORT('SCRLIB: NO CROSS SECTIONS FOUND.')
      IF(NBISO1.GT.MAXISO) CALL XABORT('SCRLIB: MAXISO OVERFLOW.')
*----
*  RECOVER INFORMATION FROM adresses DIRECTORY.
*----
      NL=0
      IF(NADRX.GT.0) THEN
        CALL SETARA(IBASE,(NREA+2)*(NISOP+NMAC)*NADRX,IADRX)
        CALL LCMSIX(IPSAP,'adresses',1)
        CALL LCMGET(IPSAP,'ADRX',IBASE(IADRX))
        CALL LCMSIX(IPSAP,' ',2)
        DO IAD=1,NADRX
          DO ISO=1,NISOP+NMAC
            IOF=(NREA+2)*(NISOP+NMAC)*(IAD-1)+(NREA+2)*(ISO-1)+NREA+1
            NL=MAX(NL,IBASE(IADRX+IOF-1))
            IOF=(NREA+2)*(NISOP+NMAC)*(IAD-1)+(NREA+2)*(ISO-1)+NREA+2
            NL=MAX(NL,IBASE(IADRX+IOF-1))
          ENDDO
        ENDDO
      ENDIF
      IF(IMPX.GT.1) THEN
        WRITE(6,'(36H SCRLIB: number of Legendre orders =,I4)') NL
      ENDIF
*----
*  RECOVER INFORMATION FROM geom DIRECTORY.
*----
      CALL LCMSIX(IPSAP,'geom',1)
      CALL SETARA(BASE,NMIL,IXVOLM)
      CALL LCMGET(IPSAP,'XVOLMT',BASE(IXVOLM))
      CALL LCMSIX(IPSAP,' ',2)
*----
*  LOOP OVER SAPHYB MIXTURES TO COMPUTE DENS0(NMIL,NCAL,NBISO1)
*----
      CALL SETARA(BASE,NBISO1*NCAL*NMIL,IDENS0)
      IF(NISOTS.GT.0) CALL SETARA(IBASE,NISOTS*2,IISOTS)
      CALL XDRSET(BASE(IDENS0),NBISO1*NCAL*NMIL,0.0)
      CALL SETARA(BASE,NISOTS,ICONCE)
      DO 30 IBMOLD=1,NMIL
      DO ICAL=1,NCAL
        DO IBM=1,NMIX
          IF((TERP(ICAL,IBM).NE.0.0).AND.(MIXC(IBM).EQ.IBMOLD)) GO TO 10
        ENDDO
        CYCLE
  10    WRITE(TEXT12,'(4Hcalc,I8)') ICAL
        CALL LCMLEN(IPSAP,TEXT12,ILENG,ITYLCM)
        IF(ILENG.EQ.0) THEN
          WRITE(HSMG,'(29HSCRLIB: MISSING CALCULATION '',A12,2H''.)')
     1    TEXT12
          CALL XABORT(HSMG)
        ENDIF
        CALL LCMSIX(IPSAP,TEXT12,1) ! step up to calc
        CALL LCMSIX(IPSAP,'info',1)
        IF(NISOTS.GT.0) CALL LCMGET(IPSAP,'ISOTS',IBASE(IISOTS))
        CALL LCMSIX(IPSAP,' ',2)
        WRITE(TEXT12,'(4Hmili,I8)') IBMOLD
        CALL LCMLEN(IPSAP,TEXT12,ILENG,ITYLCM)
        IF(ILENG.EQ.0) THEN
          WRITE(HSMG,'(29HSCRLIB: MISSING MIXTURE '',A12,2H''.)') TEXT12
          CALL XABORT(HSMG)
        ENDIF
        CALL LCMSIX(IPSAP,TEXT12,1) ! step up to mili
        CALL LCMGET(IPSAP,'CONCES',BASE(ICONCE))
        DO ISO=1,NISOP
          IOF=INOMIS+2*(ISO-1)
          DO IS2=1,NISOTS
            ISOKEP=IS2
            IF(IBASE(IOF+0).NE.IBASE(IISOTS+2*(IS2-1)+0)) CYCLE
            IF(IBASE(IOF+1).NE.IBASE(IISOTS+2*(IS2-1)+1)) CYCLE
            GO TO 20
          ENDDO
          CYCLE
   20     IOFD=(IBMOLD-1)*NCAL*NBISO1+(ICAL-1)*NBISO1+ISO
          BASE(IDENS0+IOFD-1)=BASE(ICONCE+ISOKEP-1)
        ENDDO
        CALL LCMSIX(IPSAP,' ',2) ! step down from mili
        CALL LCMSIX(IPSAP,' ',2) ! step down from calc
      ENDDO
   30 CONTINUE
      CALL RLSARA(BASE(ICONCE))
*----
*  LOOP OVER MICROLIB MIXTURES
*----
      CALL SETARA(IBASE,NMIL,ISADRX)
      CALL SETARA(IBASE,NMIL,LENGDX)
      CALL SETARA(IBASE,NMIL,LENGDP)
      CALL SETARA(BASE,NGRP,INWT0)
      CALL SETARA(BASE,NGRP*NL*(NISOP+NMAC),ISIGS)
      CALL SETARA(BASE,NGRP*NGRP*NL*(NISOP+NMAC),ISS2D)
      CALL SETARA(BASE,NGRP*NREA*(NISOP+NMAC),IXS)
      CALL SETARA(LBASE,NREA,ILXS)
      CALL SETARA(BASE,NPRC,ILAMB)
      CALL SETARA(BASE,NGRP*NPRC,ICHIRS)
      CALL SETARA(BASE,NPRC,IBETAR)
      CALL SETARA(BASE,NGRP,INVELS)
      CALL XDRSET(BASE(ILAMB),NPRC,0.0)
      CALL XDRSET(BASE(ICHIRS),NGRP*NPRC,0.0)
      CALL XDRSET(BASE(IBETAR),NPRC,0.0)
      CALL XDRSET(BASE(INVELS),NGRP,0.0)
      CALL SETARA(BASE,NGRP*NPRC,ICHIRSB)
      CALL SETARA(BASE,NPRC,IBETARB)
      CALL SETARA(BASE,NGRP,INVELSB)
      CALL SETARA(BASE,NBISO1*NCAL,IDENS1)
      CALL SETARA(BASE,NBISO1*NCAL,IFACT)
      DO 100 IBM=1,NMIX
      IBMOLD=MIXC(IBM)
      IF(IBMOLD.EQ.0) GO TO 100
      IF(NISO(IBM).GT.MAXNIS) CALL XABORT('SCRLIB: MAXNIS OVERFLOW.')
      VOLMI2(IBM)=BASE(IXVOLM+IBMOLD-1)
      IMAC=IBASE(ITOTM+IBMOLD-1)
      IRES=IBASE(IRESM+IBMOLD-1)
*----
*  COMPUTE THE NUMBER DENSITIES OF EACH ELEMENTARY CALCULATION.
*----
      CALL XDRSET(BASE(IDENS1),NBISO1*NCAL,0.0)
      CALL XDRSET(DENS3,NBISO1,0.0)
      DO ICAL=1,NCAL
        WEIGHT=TERP(ICAL,IBM)
        IF(WEIGHT.EQ.0.0) CYCLE
        DO ISO=1,NISOP
          LUSER=.FALSE.
          WRITE(TEXT8,'(2A4)') (IBASE(INOMIS+2*(ISO-1)+I0),I0=0,1)
          KSO1=0
          DO KSO=1,NISO(IBM) ! user-selected isotope
            WRITE(HHISO,'(2A4)') (HISO(I0,IBM,KSO),I0=1,2)
            IF(TEXT8.EQ.HHISO) THEN
              KSO1=KSO
              LUSER=(CONC(IBM,KSO1).NE.-99.99)
              GO TO 35
            ENDIF
          ENDDO
   35     IF(LUSER) THEN
            BASE(IDENS1+(ICAL-1)*NBISO1+ISO-1)=CONC(IBM,KSO1)
            CYCLE
          ENDIF
          IF(.NOT.LISO(IBM)) CYCLE
          IOFD=(IBMOLD-1)*NCAL*NBISO1+(ICAL-1)*NBISO1+ISO
          BASE(IDENS1+(ICAL-1)*NBISO1+ISO-1)=BASE(IDENS0+IOFD-1)
        ENDDO
        IF(NMAC.GT.0) BASE(IDENS1+(ICAL-1)*NBISO1+(NISOP+1)-1)=1.0
        DO ISO=1,NBISO1
          DENS3(ISO)=DENS3(ISO)+WEIGHT*
     1               BASE(IDENS1+(ICAL-1)*NBISO1+ISO-1)
        ENDDO
      ENDDO
      DO ICAL=1,NCAL
      DO ISO=1,NBISO1
        IF(DENS3(ISO).GT.BASE(IDENS1+(ICAL-1)*NBISO1+ISO-1)*1.0E-9) THEN
          BASE(IFACT+(ICAL-1)*NBISO1+ISO-1)=
     1    BASE(IDENS1+(ICAL-1)*NBISO1+ISO-1)/DENS3(ISO)
        ELSE
          BASE(IFACT+(ICAL-1)*NBISO1+ISO-1)=1.0
        ENDIF
      ENDDO
      ENDDO
*----
*  ALLOCATE WORKING ARRAYS.
*----
      CALL XDRSET(BASE(INWT0),NGRP,0.0)
      CALL XDRSET(BASE(ISIGS),NGRP*NL*NBISO1,0.0)
      CALL XDRSET(BASE(ISS2D),NGRP*NGRP*NL*NBISO1,0.0)
      CALL XDRSET(BASE(IXS),NGRP*NREA*NBISO1,0.0)
      CALL XDLSET(LBASE(ILXS),NREA,.FALSE.)
*----
*  MAIN LOOP OVER ELEMENTARY CALCULATIONS
*----
      LSTRD=.FALSE.
      DO 60 ICAL=1,NCAL
      WEIGHT=TERP(ICAL,IBM)
      IF(WEIGHT.EQ.0.0) GO TO 60
*----
*  RECOVER INFORMATION FROM caldir DIRECTORY.
*----
      WRITE(TEXT12,'(4Hcalc,I8)') ICAL
      CALL LCMSIX(IPSAP,TEXT12,1) ! step up to calc
      IF(NPRC.GT.0) THEN
        CALL XDRSET(BASE(ICHIRSB),NGRP*NPRC,0.0)
        CALL XDRSET(BASE(IBETARB),NPRC,0.0)
        CALL XDRSET(BASE(INVELSB),NGRP,0.0)
      ENDIF
      CALL LCMSIX(IPSAP,'info',1)
      LSPH=.FALSE.
      IF(NPARL.GT.0) THEN
        CALL LCMGET(IPSAP,'NLOC',NLOC)
        IF(NLOC.GT.MAXLOC) CALL XABORT('SCRLIB: MAXLOC OVERFLOW')
        CALL LCMGTC(IPSAP,'LOCTYP',4,NLOC,LOCTYP)
        CALL LCMGTC(IPSAP,'LOCNAM',80,NLOC,LOCNAM)
        CALL SETARA(IBASE,NLOC+1,ILOCAD)
        CALL LCMGET(IPSAP,'LOCADR',IBASE(ILOCAD))
        DO ILOC=1,NLOC
          LSPH=((LOCTYP(ILOC).EQ.'EQUI').AND.(LOCNAM(ILOC).EQ.HEQUI))
          IF(LSPH) EXIT
        ENDDO
      ENDIF
      CALL LCMGET(IPSAP,'ISADRX',IBASE(ISADRX))
      CALL LCMGET(IPSAP,'LENGDX',IBASE(LENGDX))
      CALL LCMGET(IPSAP,'LENGDP',IBASE(LENGDP))
      IF(NISOTS.GT.0) CALL LCMGET(IPSAP,'ISOTS',IBASE(IISOTS))
      CALL LCMSIX(IPSAP,' ',2)
      CALL LCMSIX(IPSAP,'divers',1)
      CALL LCMLEN(IPSAP,'NVDIV',ILENG,ITYLCM)
      IF(ILENG.EQ.0) THEN
        NVDIV=0
      ELSE
        CALL LCMGET(IPSAP,'NVDIV',NVDIV)
      ENDIF
      B2SAP=0.0
      IF(NVDIV.GT.0) THEN
        IF(NVDIV.GT.MAXDIV) CALL XABORT('SCRLIB: MAXDIV OVERFLOW.')
        CALL LCMGTC(IPSAP,'IDVAL',4,NVDIV,IDVAL)
        CALL LCMGET(IPSAP,'VALDIV',VALDIV)
        DO I=1,NVDIV
          IF(IMPX.GT.3) THEN
            WRITE(6,'(9H SCRLIB: ,I3,2X,A,1H=,1P,E13.5)') I,IDVAL(I),
     1      VALDIV(I)
          ENDIF
          IF(IDVAL(I).EQ.'B2') B2SAP=VALDIV(I)
        ENDDO
      ENDIF
      LSTRD=LSTRD.OR.(B2SAP.EQ.0.0)
*
      CALL LCMLEN(IPSAP,'NPR',ILONG,ITYLCM)
      IF((NPRC.GT.0).AND.(ILONG.EQ.1)) THEN
        CALL LCMGET(IPSAP,'NPR',NPR)
        IF(NPR.NE.NPRC) CALL XABORT('SCRLIB: NPR INCONSISTENCY(1).')
        CALL LCMGET(IPSAP,'LAMBRS',BASE(ILAMB))
        CALL LCMGET(IPSAP,'CHIRS',BASE(ICHIRSB))
        CALL LCMGET(IPSAP,'BETARS',BASE(IBETARB))
        CALL LCMGET(IPSAP,'INVELS',BASE(INVELSB))
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
*----
*  SELECT SAPHYB MIXTURE IBMOLD.
*----
      IF(NADRX.EQ.0) CALL XABORT('SCRLIB: NO ADDRESS SETS AVAILABLE.')
      WRITE(TEXT12,'(4Hmili,I8)') IBMOLD
      CALL LCMLEN(IPSAP,TEXT12,ILENG,ITYLCM)
      IF(ILENG.EQ.0) THEN
        WRITE(HSMG,'(29HSCRLIB: MISSING MIXTURE '',A12,2H''.)') TEXT12
        CALL XABORT(HSMG)
      ENDIF
      CALL LCMSIX(IPSAP,TEXT12,1) ! step up to mili
      CALL XDRSET(SPH,NGRP,1.0)
      IF(LSPH) THEN
        CALL SETARA(BASE,IBASE(ILOCAD+NLOC)-1,IRVALO)
        CALL LCMGET(IPSAP,'RVALOC',BASE(IRVALO))
        DO ILOC=1,NLOC
          IF((LOCTYP(ILOC).EQ.'EQUI').AND.(LOCNAM(ILOC).EQ.HEQUI)) THEN
            IF(IBASE(ILOCAD+ILOC)-IBASE(ILOCAD+ILOC-1).NE.NGRP) THEN
              CALL XABORT('SCRLIB: INVALID NUMBER OF COMPONENTS FOR '
     1        //'SPH FACTORS')
            ENDIF
            DO IGR=1,NGRP
              SPH(IGR)=BASE(IRVALO+IBASE(ILOCAD+ILOC-1)+IGR-2)
            ENDDO
          ENDIF
        ENDDO
        CALL RLSARA(BASE(IRVALO))
      ENDIF
      IF(NPARL.GT.0) CALL RLSARA(IBASE(ILOCAD))
      IAD=IBASE(ISADRX+IBMOLD-1)
      NDATAX=IBASE(LENGDX+IBMOLD-1)
      NDATAP=IBASE(LENGDP+IBMOLD-1)
      CALL SETARA(BASE,NGRP,IFLUXS)
      CALL SETARA(BASE,NDATAX,IRDATA)
      CALL SETARA(IBASE,NDATAP,IIDATA)
      CALL LCMGET(IPSAP,'FLUXS',BASE(IFLUXS))
      CALL LCMGET(IPSAP,'RDATAX',BASE(IRDATA))
      CALL LCMGET(IPSAP,'IDATAP',IBASE(IIDATA))
      DO I=1,NGRP
        BASE(INWT0+I-1)=BASE(INWT0+I-1)+WEIGHT*BASE(IFLUXS+I-1)/
     1  (BASE(IXVOLM+IBMOLD-1)*SPH(I))
      ENDDO
      CALL SETARA(BASE,NGRP*NL,ISIGSB)
      CALL SETARA(BASE,NGRP*NGRP*NL,ISS2DB)
      CALL SETARA(BASE,NGRP*NREA,IXSB)
      IF(NISOP.NE.0) THEN
        DO ISO=1,NISOP
          FACT0=BASE(IFACT+(ICAL-1)*NBISO1+ISO-1)
          CALL SPHSXS(NREA,NISOP+NMAC,NADRX,NGRP,NL,NDATAX,NDATAP,
     1    ISO,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),NOMREA,
     2    BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
          DO IGR=1,NGRP
            DO IL=1,NL
              I=(IL-1)*NGRP+IGR
              J=(ISO-1)*NGRP*NL+I
              BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+FACT0*SPH(IGR)*WEIGHT*
     1        BASE(ISIGSB+I-1)
            ENDDO
          ENDDO
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              DO IL=1,NL
                I=(IL-1)*NGRP*NGRP+(JGR-1)*NGRP+IGR
                J=(ISO-1)*NGRP*NGRP*NL+I
                BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+FACT0*SPH(JGR)*WEIGHT*
     1          BASE(ISS2DB+I-1)
              ENDDO
            ENDDO
          ENDDO
          DO IGR=1,NGRP
            DO IREA=1,NREA
              I=(IREA-1)*NGRP+IGR
              J=(ISO-1)*NGRP*NREA+I
              IF(NOMREA(IREA).EQ.'SPECTRE') THEN
                BASE(IXS+J-1)=BASE(IXS+J-1)+WEIGHT*BASE(IXSB+I-1)
              ELSE IF(NOMREA(IREA).EQ.'FUITES') THEN
                IF(B2SAP.NE.0.0) THEN
                 XSECT=BASE(IXSB+I-1)/B2SAP
                 BASE(IXS+J-1)=BASE(IXS+J-1)+FACT0*WEIGHT*XSECT/SPH(IGR)
                ENDIF
              ELSE
                BASE(IXS+J-1)=BASE(IXS+J-1)+FACT0*SPH(IGR)*WEIGHT*
     1          BASE(IXSB+I-1)
              ENDIF
            ENDDO
          ENDDO
        ENDDO
        IF(IRES.NE.0) THEN
          CALL SPHSXS(NREA,NISOP+NMAC,NADRX,NGRP,NL,NDATAX,NDATAP,
     1    NISOP+IRES,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),
     2    NOMREA,BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
          DO IGR=1,NGRP
            DO IL=1,NL
              I=(IL-1)*NGRP+IGR
              J=NISOP*NGRP*NL+I
              BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+SPH(IGR)*WEIGHT*
     1        BASE(ISIGSB+I-1)
            ENDDO
          ENDDO
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              DO IL=1,NL
                I=(IL-1)*NGRP*NGRP+(JGR-1)*NGRP+IGR
                J=NISOP*NGRP*NGRP*NL+I
                BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+SPH(JGR)*WEIGHT*
     1          BASE(ISS2DB+I-1)
              ENDDO
            ENDDO
          ENDDO
          DO IGR=1,NGRP
            DO IREA=1,NREA
              I=(IREA-1)*NGRP+IGR
              J=NISOP*NGRP*NREA+I
              IF(NOMREA(IREA).EQ.'SPECTRE') THEN
                BASE(IXS+J-1)=BASE(IXS+J-1)+WEIGHT*BASE(IXSB+I-1)
              ELSE IF(NOMREA(IREA).EQ.'FUITES') THEN
                IF(B2SAP.NE.0.0) THEN
                  XSECT=BASE(IXSB+I-1)/B2SAP
                  BASE(IXS+J-1)=BASE(IXS+J-1)+WEIGHT*XSECT/SPH(IGR)
                ENDIF
              ELSE
                BASE(IXS+J-1)=BASE(IXS+J-1)+SPH(IGR)*WEIGHT*
     1          BASE(IXSB+I-1)
              ENDIF
            ENDDO
          ENDDO
        ENDIF
      ELSE IF(IMAC.NE.0) THEN
        CALL SPHSXS(NREA,NISOP+NMAC,NADRX,NGRP,NL,NDATAX,NDATAP,
     1  NISOP+IMAC,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),
     2  NOMREA,BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
        DO IGR=1,NGRP
          DO IL=1,NL
            I=(IL-1)*NGRP+IGR
            J=NISOP*NGRP*NL+I
            BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+SPH(IGR)*WEIGHT*
     1      BASE(ISIGSB+I-1)
          ENDDO
        ENDDO
        DO IGR=1,NGRP
          DO JGR=1,NGRP
            DO IL=1,NL
              I=(IL-1)*NGRP*NGRP+(JGR-1)*NGRP+IGR
              J=NISOP*NGRP*NGRP*NL+I
              BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+SPH(JGR)*WEIGHT*
     1        BASE(ISS2DB+I-1)
            ENDDO
          ENDDO
        ENDDO
        DO IGR=1,NGRP
          DO IREA=1,NREA
            I=(IREA-1)*NGRP+IGR
            J=NISOP*NGRP*NREA+I
            IF(NOMREA(IREA).EQ.'SPECTRE') THEN
              BASE(IXS+J-1)=BASE(IXS+J-1)+WEIGHT*BASE(IXSB+I-1)
            ELSE IF(NOMREA(IREA).EQ.'FUITES') THEN
              IF(B2SAP.NE.0.0) THEN
                XSECT=BASE(IXSB+I-1)/B2SAP
                BASE(IXS+J-1)=BASE(IXS+J-1)+WEIGHT*XSECT/SPH(IGR)
              ENDIF
            ELSE
              BASE(IXS+J-1)=BASE(IXS+J-1)+SPH(IGR)*WEIGHT*
     1        BASE(IXSB+I-1)
            ENDIF
          ENDDO
        ENDDO
      ELSE
        CALL XABORT('SCRLIB: NO MACROSCOPIC SET.')
      ENDIF
      CALL RLSARA(BASE(IXSB))
      CALL RLSARA(BASE(ISS2DB))
      CALL RLSARA(BASE(ISIGSB))
      CALL RLSARA(IBASE(IIDATA))
      CALL RLSARA(BASE(IRDATA))
      CALL RLSARA(BASE(IFLUXS))
*
      CALL LCMLEN(IPSAP,'cinetique',ILONG,ITYLCM)
      IF((NPRC.GT.0).AND.(ILONG.NE.0)) THEN
        CALL LCMSIX(IPSAP,'cinetique',1)
        CALL LCMGET(IPSAP,'NPR',NPR)
        IF(NPR.NE.NPRC) CALL XABORT('SCRLIB: NPR INCONSISTENCY(2).')
        CALL LCMGET(IPSAP,'LAMBRS',BASE(ILAMB))
        CALL LCMGET(IPSAP,'CHIRS',BASE(ICHIRSB))
        CALL LCMGET(IPSAP,'BETARS',BASE(IBETARB))
        CALL LCMGET(IPSAP,'INVELS',BASE(INVELSB))
        CALL LCMSIX(IPSAP,' ',2)
      ENDIF
      IF(NPRC.GT.0) THEN
        DO IGR=1,NGRP
          BASE(INVELS+IGR-1)=BASE(INVELS+IGR-1)+SPH(IGR)*WEIGHT*
     1    BASE(INVELSB+IGR-1)
          DO IPRC=1,NPRC
            IOF=(IPRC-1)*NGRP+IGR
            BASE(ICHIRS+IOF-1)=BASE(ICHIRS+IOF-1)+WEIGHT*
     1      BASE(ICHIRSB+IOF-1)
          ENDDO
        ENDDO
        DO IPRC=1,NPRC
          BASE(IBETAR+IPRC-1)=BASE(IBETAR+IPRC-1)+WEIGHT*
     1    BASE(IBETARB+IPRC-1)
        ENDDO
      ENDIF
*----
*  IDENTIFY SPECIAL FLUX EDITS
*----
      DO IREA=1,NREA
        IF(.NOT.LBASE(ILXS+IREA-1)) CYCLE
        IF(NOMREA(IREA).EQ.'TOTALE') CYCLE
        IF(NOMREA(IREA).EQ.'TOTALE P1') CYCLE
        IF(NOMREA(IREA).EQ.'EXCESS') CYCLE
        IF(NOMREA(IREA).EQ.'FISSION') CYCLE
        IF(NOMREA(IREA).EQ.'SPECTRE') CYCLE
        IF(NOMREA(IREA).EQ.'NU*FISSION') CYCLE
        IF(NOMREA(IREA).EQ.'ENERGIE') CYCLE
        IF(NOMREA(IREA).EQ.'SELF') CYCLE
        IF(NOMREA(IREA).EQ.'TRANSP-CORR') CYCLE
        IF(NOMREA(IREA).EQ.'FUITES') CYCLE
        IF(NOMREA(IREA).EQ.'DIFFUSION') CYCLE
        IF(NOMREA(IREA).EQ.'TRANSFERT') CYCLE
        DO 40 IED2=1,NED2
        IF(NOMREA(IREA)(:8).EQ.HVECT2(IED2)) GO TO 50
   40   CONTINUE
        NED2=NED2+1
        HVECT2(NED2)=NOMREA(IREA)(:8)
   50   CONTINUE
      ENDDO
*
      CALL LCMSIX(IPSAP,' ',2) ! step down from mili
      CALL LCMSIX(IPSAP,' ',2) ! step down from calc
   60 CONTINUE ! end of loop over elementary calculations.
*----
*  SAVE CROSS SECTIONS IN MICROLIB FOR MIXTURE IBM
*----
      CALL SETARA(BASE,NL,IITYPR)
      CALL SETARA(BASE,NGRP,ISTRD)
      HRESID=' '
      IF(NISOP.NE.0) THEN
        DO ISO=1,NISOP
          JSIGS=ISIGS+(ISO-1)*NGRP*NL
          JSS2D=ISS2D+(ISO-1)*NGRP*NGRP*NL
          JXS=IXS+(ISO-1)*NGRP*NREA
          WRITE(TEXT12,'(2A4,I4.4)') (IBASE(INOMIS+2*(ISO-1)+I0),
     1    I0=0,1),IBM
          CALL LCMSIX(IPLIB,TEXT12,1)
          CALL SCRISO(IPLIB,IMPX,NREA,NGRP,NL,NPRC,NOMREA,BASE(INWT0),
     1    BASE(JSIGS),BASE(JSS2D),BASE(JXS),LBASE(ILXS),BASE(ILAMB),
     2    BASE(ICHIRS),BASE(IBETAR),BASE(INVELS),LSTRD,ITRANC,IFISS,
     3    BASE(IITYPR),BASE(ISTRD))
          CALL LCMSIX(IPLIB,' ',2)
        ENDDO
        IF(IRES.NE.0) THEN
          HRESID=NOMMAC(IRES)
          JSIGS=ISIGS+NISOP*NGRP*NL
          JSS2D=ISS2D+NISOP*NGRP*NGRP*NL
          JXS=IXS+NISOP*NGRP*NREA
          WRITE(TEXT12,'(8H*MAC*RES,I4.4)') IBM
          CALL LCMSIX(IPLIB,TEXT12,1)
          CALL SCRISO(IPLIB,IMPX,NREA,NGRP,NL,NPRC,NOMREA,BASE(INWT0),
     1    BASE(JSIGS),BASE(JSS2D),BASE(JXS),LBASE(ILXS),BASE(ILAMB),
     2    BASE(ICHIRS),BASE(IBETAR),BASE(INVELS),LSTRD,ITRANC,IFISS,
     3    BASE(IITYPR),BASE(ISTRD))
          CALL LCMSIX(IPLIB,' ',2)
        ENDIF
      ELSE IF(IMAC.NE.0) THEN
        HRESID=NOMMAC(IMAC)
        JSIGS=ISIGS+NISOP*NGRP*NL
        JSS2D=ISS2D+NISOP*NGRP*NGRP*NL
        JXS=IXS+NISOP*NGRP*NREA
        WRITE(TEXT12,'(8H*MAC*RES,I4.4)') IBM
        CALL LCMSIX(IPLIB,TEXT12,1)
        CALL SCRISO(IPLIB,IMPX,NREA,NGRP,NL,NPRC,NOMREA,BASE(INWT0),
     1  BASE(JSIGS),BASE(JSS2D),BASE(JXS),LBASE(ILXS),BASE(ILAMB),
     2  BASE(ICHIRS),BASE(IBETAR),BASE(INVELS),LSTRD,ITRANC,IFISS,
     3  BASE(IITYPR),BASE(ISTRD))
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
      CALL RLSARA(BASE(ISTRD))
      CALL RLSARA(BASE(IITYPR))
*----
*  SET NUMBER DENSITIES AND VOLUMES TO ZERO.
*----
      IF(LRES) THEN
        DO JSO=1,NBISO2 ! microlib isotope
          IF(IBM.EQ.IMIX2(JSO)) THEN
            DO KSO=1,NISO(IBM) ! user-selected isotope
              IF((HISO(1,IBM,KSO).EQ.HUSE2(1,JSO)).AND.
     1           (HISO(2,IBM,KSO).EQ.HUSE2(2,JSO))) THEN
                DENS2(JSO)=0.0
                VOL2(JSO)=0.0
                CYCLE
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ELSE
        DO JSO=1,NBISO2 ! microlib isotope
          IF(IBM.EQ.IMIX2(JSO)) THEN
            DO ISO=1,NBISO1 ! Saphyb isotope
              IF((IBASE(INOMIS+2*(ISO-1)+0).EQ.HUSE2(1,JSO)).AND.
     1           (IBASE(INOMIS+2*(ISO-1)+1).EQ.HUSE2(2,JSO))) THEN
                DENS2(JSO)=0.0
                VOL2(JSO)=0.0
                CYCLE
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
*----
*  SET ISOTOPES NAMES IN OUTPUT MICROLIB
*----
      IF(.NOT.LRES) THEN
*       -- Number densities are interpolated or not according to
*       -- ALL/ONLY option
        DO 80 ISO=1,NBISO1 ! Saphyb isotope
        WRITE(TEXT12,'(2A4,I4.4)') (IBASE(INOMIS+2*(ISO-1)+I0),I0=0,1),
     1  IBM
        IF(.NOT.LISO(IBM)) THEN
*         --ONLY option
          DO KSO=1,NISO(IBM) ! user-selected isotope
            WRITE(HHISO,'(2A4)') (HISO(I0,IBM,KSO),I0=1,2)
            IF(TEXT12(:8).EQ.HHISO) GO TO 70
          ENDDO
          GO TO 80
        ENDIF
   70   DO JSO=1,NBISO2 ! microlib isotope
          IF(IMIX2(JSO).NE.IBM) CYCLE
          IF((IBASE(INOMIS+2*(ISO-1)+0).EQ.HUSE2(1,JSO)).AND.
     1       (IBASE(INOMIS+2*(ISO-1)+1).EQ.HUSE2(2,JSO))) THEN
            DENS2(JSO)=DENS2(JSO)+DENS3(ISO)
            VOL2(JSO)=VOL2(JSO)+BASE(IXVOLM+IBMOLD-1)
            GO TO 80
          ENDIF
        ENDDO
        NBISO2=NBISO2+1
        IF(NBISO2.GT.MAXISO) THEN
          WRITE(IOUT,'(/16H SCRLIB: NBISO2=,I6,8H MAXISO=,I6)') NBISO2,
     1    MAXISO
          CALL XABORT('SCRLIB: MAXISO OVERFLOW(2).')
        ENDIF
        READ(TEXT12,'(3A4)') (HUSE2(I0,NBISO2),I0=1,3)
        HNAM2(1,NBISO2)=HUSE2(1,NBISO2)
        HNAM2(2,NBISO2)=HUSE2(2,NBISO2)
        IF(TEXT12(:8).EQ.'*MAC*RES') THEN
          READ(HRESID,'(2A4)') (HNAM2(I0,NBISO2),I0=1,2)
        ENDIF
        TEXT4=' '
        READ(TEXT4,'(3A4)') IIII
        HNAM2(3,NBISO2)=IIII
        IMIX2(NBISO2)=IBM
        DENS2(NBISO2)=DENS3(ISO)
        VOL2(NBISO2)=BASE(IXVOLM+IBMOLD-1)
   80   CONTINUE
      ELSE IF(NISO(IBM).GT.0) THEN
        DO 90 JSO=1,NBISO2 ! microlib isotope
          IF(IMIX2(JSO).NE.IBM) CYCLE
          DO KSO=1,NISO(IBM) ! user-selected isotope
            IF(CONC(IBM,KSO).EQ.-99.99) THEN
*             -- Only number densities of isotopes set with "MICR" and
*             -- "*" keywords are interpolated
              DO ISO=1,NBISO1 ! Saphyb isotope
                IF((IBASE(INOMIS+2*(ISO-1)+0).EQ.HUSE2(1,JSO)).AND.
     1            (IBASE(INOMIS+2*(ISO-1)+1).EQ.HUSE2(2,JSO))) THEN
                  DENS2(JSO)=DENS2(JSO)+DENS3(ISO)
                  VOL2(JSO)=VOL2(JSO)+BASE(IXVOLM+IBMOLD-1)
                  GO TO 90
                ENDIF
              ENDDO
              WRITE(TEXT12,'(2A4,I4.4)') HUSE2(1,JSO),HUSE2(2,JSO),IBM
              CALL XABORT('SCRLIB: UNABLE TO PROCESS ISOTOPE '//TEXT12)
            ELSE IF(CONC(IBM,KSO).NE.-99.99) THEN
*             -- Number densities of isotopes set with "MICR" and
*             -- fixed value are forced to this value
              IF((HISO(1,IBM,KSO).EQ.HUSE2(1,JSO)).AND.
     1           (HISO(2,IBM,KSO).EQ.HUSE2(2,JSO))) THEN
                   DENS2(JSO)=CONC(IBM,KSO)
                   GO TO 90
                 ENDIF
            ENDIF
          ENDDO
   90   CONTINUE
      ENDIF
  100 CONTINUE ! end of loop over microlib mixtures.
*----
*  RELEASE MEMORY
*----
      CALL RLSARA(BASE(IFACT))
      CALL RLSARA(BASE(IDENS1))
      CALL RLSARA(BASE(INVELSB))
      CALL RLSARA(BASE(IBETARB))
      CALL RLSARA(BASE(ICHIRSB))
      CALL RLSARA(BASE(INVELS))
      CALL RLSARA(BASE(IBETAR))
      CALL RLSARA(BASE(ICHIRS))
      CALL RLSARA(BASE(ILAMB))
      CALL RLSARA(LBASE(ILXS))
      CALL RLSARA(BASE(IXS))
      CALL RLSARA(BASE(ISS2D))
      CALL RLSARA(BASE(ISIGS))
      CALL RLSARA(BASE(INWT0))
      CALL RLSARA(IBASE(LENGDP))
      CALL RLSARA(IBASE(LENGDX))
      CALL RLSARA(IBASE(ISADRX))
      IF(NISOTS.GT.0) CALL RLSARA(IBASE(IISOTS))
      CALL RLSARA(BASE(IDENS0))
      CALL RLSARA(BASE(IXVOLM))
      IF(NADRX.GT.0) CALL RLSARA(IBASE(IADRX))
      CALL RLSARA(IBASE(INOMIS))
      CALL RLSARA(IBASE(IRESM))
      CALL RLSARA(IBASE(ITOTM))
*----
*  COMPUTE DEPLETION CHAIN DATA
*----
      CALL XDDSET(YLDS,MY1*MY2,0.0)
      CALL XDDSET(DECAYC,MD1*MD2,0.0)
      VTOT=0.0D0
      DO 110 IBM=1,NMIX
      IF(MIXC(IBM).NE.0) VTOT=VTOT+VOLMI2(NMIX)
  110 CONTINUE
      DO 120 IBM=1,NMIX
      IBMOLD=MIXC(IBM)
      DO 120 ICAL=1,NCAL
      WEIGHT=TERP(ICAL,IBM)
      IF(WEIGHT.EQ.0.0) GO TO 120
      WRITE(TEXT12,'(4Hcalc,I8)') ICAL
      CALL LCMSIX(IPSAP,TEXT12,1) ! step up to calc
      CALL LCMSIX(IPSAP,'info',1)
      CALL LCMGET(IPSAP,'NISF',NISF)
      IF(NISF+NMAC.GT.MY1) CALL XABORT('SCRLIB: MY1 OVERFLOW')
      CALL LCMGET(IPSAP,'NISP',NISP)
      IF(NISP.GT.MY2) CALL XABORT('SCRLIB: MY2 OVERFLOW')
      CALL LCMGET(IPSAP,'NISOTS',NISOTS2)
      IF(NISOTS2.GT.NISOTS) CALL XABORT('SCRLIB: NISOTS OVERFLOW')
      CALL LCMSIX(IPSAP,' ',2)
      WRITE(TEXT12,'(4Hmili,I8)') IBMOLD
      CALL LCMSIX(IPSAP,TEXT12,1) ! step up to mili
      IF(MY1*MY2.GT.0) THEN
        CALL LCMLEN(IPSAP,'YLDS',ILONG,ITYLCM)
        IF(ILONG.NE.(NISF+NMAC)*NISP) CALL XABORT('SCRLIB: BAD YLDS.')
        CALL SETARA(BASE,ILONG,IYLDS2)
        CALL LCMGET(IPSAP,'YLDS',BASE(IYLDS2))
        DO IY1=1,NISF+NMAC
          DO IY2=1,NISP
            IOF=(NISF+NMAC)*(IY2-1)+IY1
            YLDS(IY1,IY2)=YLDS(IY1,IY2)+WEIGHT*BASE(IYLDS2+IOF-1)*
     >      VOLMI2(IBM)/VTOT
          ENDDO
        ENDDO
        CALL RLSARA(BASE(IYLDS2))
      ENDIF
      IF(MD1*MD2.GT.0) THEN
        CALL LCMLEN(IPSAP,'DECAYC',ILONG,ITYLCM)
        IF(ILONG.NE.NLAM*NISOTS2) CALL XABORT('SCRLIB: BAD DECAYC.')
        CALL SETARA(BASE,ILONG,IDECAY2)
        CALL LCMGET(IPSAP,'DECAYC',BASE(IDECAY2))
        DO ID1=1,NLAM
          DO ID2=1,NISOTS2
            IOF=NLAM*(ID2-1)+ID1
            DECAYC(ID1,ID2)=DECAYC(ID1,ID2)+WEIGHT*BASE(IDECAY2+IOF-1)*
     >      VOLMI2(IBM)/VTOT
          ENDDO
        ENDDO
        CALL RLSARA(BASE(IDECAY2))
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
      CALL LCMSIX(IPSAP,' ',2)
  120 CONTINUE
*----
*  MICROLIB FINALIZATION
*----
      IF(.NOT.LRES) THEN
        CALL XDISET(ISTATE,NSTATE,0)
        ISTATE(1)=NMIX
        ISTATE(2)=NBISO2
        ISTATE(3)=NGRP
        ISTATE(4)=NL
        ISTATE(5)=ITRANC
        ISTATE(7)=1
        IF(ITER.EQ.3) ISTATE(12)=NMIX
        ISTATE(13)=NED2
        ISTATE(14)=NMIX
        ISTATE(18)=1
        ISTATE(19)=NPRC
        ISTATE(20)=0
        ISTATE(22)=MAXISO/NMIX
        IF(NBISO2.EQ.0) CALL XABORT('SCRLIB: NBISO2=0.')
        CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
        CALL LCMPUT(IPLIB,'MIXTURESVOL',NMIX,2,VOLMI2)
        CALL LCMPUT(IPLIB,'ISOTOPESUSED',3*NBISO2,3,HUSE2)
        CALL LCMPUT(IPLIB,'ISOTOPERNAME',3*NBISO2,3,HNAM2)
        CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO2,2,DENS2)
        CALL LCMPUT(IPLIB,'ISOTOPESMIX',NBISO2,1,IMIX2)
        CALL LCMPUT(IPLIB,'ISOTOPESVOL',NBISO2,2,VOL2)
        IF(NED2.GT.0) CALL LCMPTC(IPLIB,'ADDXSNAME-P0',8,NED2,HVECT2)
        CALL LCMPUT(IPLIB,'ISOTOPESTODO',NBISO2,1,ITOD2)
      ELSE IF(LRES.AND.(NISOP.GT.0)) THEN
        CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO2,2,DENS2)
        CALL LCMPUT(IPLIB,'ISOTOPESVOL',NBISO2,2,VOL2)
      ENDIF
      IF(IMPX.GT.5) CALL LCMLIB(IPLIB)
      IACCS=1
*----
*  COMPUTE THE MACROSCOPIC X-SECTIONS
*----
      IF((ITER.NE.0).AND.(ITER.NE.3)) GO TO 280
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      MAXMIX=ISTATE(1)
      IF(MAXMIX.NE.NMIX) CALL XABORT('SPHLIB: INVALID NMIX.')
      NBISO=ISTATE(2)
      CALL SETARA(LBASE,MAXMIX,IMASK)
      CALL SETARA(LBASE,NGRP,IMASKL)
      CALL LCMIOF(IPLIB,'ISOTOPESUSED',IISONA)
      CALL LCMIOF(IPLIB,'ISOTOPESMIX',IISOMI)
      CALL LCMIOF(IPLIB,'ISOTOPESDENS',IDENIS)
      CALL XDLSET(LBASE(IMASK),MAXMIX,.TRUE.)
      CALL XDLSET(LBASE(IMASKL),NGRP,.TRUE.)
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
      CALL LIBMIX(IPLIB,MAXMIX,NGRP,NBISO,IBASE(IISONA),IBASE(IISOMI),
     1         BASE(IDENIS),LBASE(IMASK),LBASE(IMASKL),ITSTMP,TMPDAY)
      CALL RLSARA(LBASE(IMASKL))
      CALL RLSARA(LBASE(IMASK))
*----
*  INCLUDE LEAKAGE IN THE MACROLIB (USED ONLY FOR NON-REGRESSION TESTS)
*----
      IF(B2.NE.0.0) THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        JPLIB=LCMGID(IPLIB,'GROUP')
        CALL SETARA(BASE,NMIX,IGAR1)
        CALL SETARA(BASE,NMIX,IGAR2)
        DO 270 IGR=1,NGRP
          KPLIB=LCMGIL(JPLIB,IGR)
          CALL LCMGET(KPLIB,'NTOT0',BASE(IGAR1))
          CALL LCMGET(KPLIB,'DIFF',BASE(IGAR2))
          DO 260 IBM=1,NMIX
            IF(MIXC(IBM).NE.0) 
     >        BASE(IGAR1+IBM-1)=BASE(IGAR1+IBM-1)+B2*BASE(IGAR2+IBM-1)
  260     CONTINUE
          CALL LCMPUT(KPLIB,'NTOT0',NMIX,2,BASE(IGAR1))
  270   CONTINUE
        CALL RLSARA(BASE(IGAR2))
        CALL RLSARA(BASE(IGAR1))
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
  280 RETURN
      END
