*DECK SIM
      SUBROUTINE SIM(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* PWR fuelling simulator according to the time-linear model.
*
*Copyright:
* Copyright (C) 2013 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert, V. Salino
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): modification type(L_MAP);
*         HENTRY(2): read-only type(L_POWER).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6,MAXIAS=30)
      CHARACTER TEXT*12,HSIGN*12,TEXT4*4,HCYCL*12,HOLD*12,HHX(26)*1,
     > TEXT4B*4,TEXT1*1,TEXT1B*1,HSMG*131,PNAME*12,ASMB1(MAXIAS)*4,
     > HC1*12,HC2*12
      INTEGER KCHAR(3),IMPX,IHY(26),ISTATE(NSTATE),SIMIND
      DOUBLE PRECISION DFLOT
      DIMENSION  IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IPMAP=0
      IPPOW=0
      DO IEN=1,NENTRY
        IF(IENTRY(IEN).GT.2) THEN
          WRITE(HSMG,'(12H@SIM: ENTRY ,A12,19H IS NOT OF LCM TYPE)')
     >    HENTRY(IEN)
          CALL XABORT(HSMG)
        ENDIF
        CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        IF(HSIGN.EQ.'L_MAP') THEN
          IPMAP=KENTRY(IEN)
          IF(JENTRY(IEN).NE.1) CALL XABORT('@SIM: MODIFICATION MODE '
     >    //'FOR L_MAP EXPECTED')
        ELSEIF(HSIGN.EQ.'L_POWER') THEN
          IPPOW=KENTRY(IEN)
          IF(JENTRY(IEN).NE.2) CALL XABORT('@SIM: READ-ONLY MODE FOR'
     >    //' L_POWER EXPECTED')
        ELSE
          CALL XABORT('@SIM: UNKNOWN SIGNATURE ('//HSIGN//')')
        ENDIF
      ENDDO
      IF(IPMAP.EQ.0) CALL XABORT('@SIM: NO FUEL MAP OBJECT FOUND')
*----
*  RECOVER INFORMATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
      NB=ISTATE(1)
      NCH=ISTATE(2)
      NCOMB=ISTATE(3)
      IMOD=ISTATE(5)
      NF=ISTATE(7)
      NPARM=ISTATE(8)
      NSIMS=ISTATE(13)
      LX=NSIMS/100
      LY=MOD(NSIMS,100)
      IF(NF.EQ.0)CALL XABORT('@SIM: NO FUEL IN MAP OBJECT.')
*----
*  ONLY TIME INSTANTANEOUS CALCULATIONS IN SIM:
*----
      IF(IMOD.NE.2)CALL XABORT('@SIM: INST-BURN OPTION SHOULD BE '
     + //'USED IN RESINI.')
*----
*  READ INPUT DATA
*----
      IMPX=0
      TTIME=0.0
      CALL SETARA(BASE,NCH,IRFCHAN)
      CALL XDRSET(BASE(IRFCHAN),NCH,0.0)
      TIME=0.0
      BURNSTEP=0.0
      HCYCL=' '
*     READ KEYWORD
  10  CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(1).')
      IF(TEXT.EQ.'EDIT')THEN
*       PRINTING INDEX
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@SIM: INTEGER DATA EXPECTED(1).')
        IMPX=MAX(0,NITMA)
        IF(IMPX.GT.0) WRITE(6,190) NB,NCH,LX,LY
      ELSEIF(TEXT.EQ.'CYCLE')THEN
        CALL REDGET(ITYP,NITMA,FLOT,HCYCL,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(2).')
        IF(NSIMS.EQ.0)CALL XABORT('@SIM: SIM DATA NOT DEFINED IN RESIN'
     >  //'I: MODULE.')
        CALL SETARA(IBASE,NCH,IZONE)
        CALL SETARA(BASE,NCH*NB,IBURNUP)
        CALL SETARA(IBASE,NCH*NB,IFMIX)
        CALL SETARA(IBASE,3*NCH,INAME)
        CALL SETARA(IBASE,3*NCH,IONAME)
        CALL SETARA(BASE,NCH*NB,IOBURNU)
        CALL SETARA(IBASE,NCH*NB,IOFMIX)
        CALL XDRSET(BASE(IBURNUP),NCH*NB,-999.0)
        CALL LCMGET(IPMAP,'S-ZONE',IBASE(IZONE))
        WRITE(TEXT4,'(A4)') IBASE(IZONE)
        READ(TEXT4,'(A1,I2)') TEXT1,INTG2
        L=0
        DO K=1,NCH
          WRITE(TEXT4,'(A4)') IBASE(IZONE+K-1)
          READ(TEXT4,'(A1,I2)') TEXT1B,INTG2B
          IF(TEXT1B.EQ.TEXT1) THEN
            L=L+1
            IF(L.GT.LY)CALL XABORT('@SIM: INCOHERENCE IN BASIC'
     >      //'ASSEMBLY LAYOUT GIVEN IN RESINI: (1).')
            IHY(L)=INTG2B
          ENDIF
        ENDDO
        L=0
        DO K=1,NCH
          WRITE(TEXT4,'(A4)') IBASE(IZONE+K-1)
          READ(TEXT4,'(A1,I2)') TEXT1B,INTG2B
          IF(INTG2B.EQ.IHY((LY+1)/2)) THEN
            L=L+1
            IF(L.GT.LX)CALL XABORT('@SIM: INCOHERENCE IN BASIC'
     >      //'ASSEMBLY LAYOUT GIVEN IN RESINI: (2).')
            HHX(L)=TEXT1B
          ENDIF
        ENDDO
        HOLD=' '
        INDCY=-1
        BURNCY=-999.0
  30    CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(3).')
  40    IF(TEXT.EQ.'FROM') THEN
          CALL REDGET(ITYP,NITMA,FLOT,HOLD,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(4).')
          GO TO 30
        ELSE IF(TEXT.EQ.'BURN') THEN
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.EQ.1) THEN
            INDCY=NITMA
          ELSE IF(ITYP.EQ.2) THEN
            BURNCY=FLOT
          ELSE
            CALL XABORT('@SIM: INTEGER OR REAL DATA EXPECTED.')
          ENDIF
          GO TO 30
        ELSE IF((TEXT.EQ.'MAP').OR.(TEXT.EQ.'QMAP')) THEN
          CALL LCMLEN(IPMAP,HCYCL,ILONG,ITYLCM)
          IF(ILONG.NE.0) THEN
            WRITE(HSMG,'(12H@SIM: CYCLE ,A12,8H EXISTS.)') HCYCL
            CALL XABORT(HSMG)
          ENDIF
          CALL SETARA(IBASE,LX*LY,ICYCLE)
          IF(TEXT.EQ.'MAP') THEN
            DO 45 I=1,LX
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(5).')
   45       CONTINUE
            DO 50 J=1,LY
            CALL REDGET(ITYP,IHY(J),FLOT,TEXT,DFLOT)
            IF(ITYP.NE.1)CALL XABORT('@SIM: INTEGER DATA EXPECTED(2).')
            DO 50 I=1,LX
            CALL REDGET(ITYP,NITMA,FLOT,TEXT4,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(6).')
            READ(TEXT4,'(A4)') NITMA
            WRITE(TEXT4B,'(A1,I2.2)') HHX(I),IHY(J)
            IBASE(ICYCLE+(J-1)*LX+I-1)=NITMA
   50       CONTINUE
          ELSE IF(TEXT.EQ.'QMAP') THEN
            LXMIN=LX/2+1
            LYMIN=LY/2+1
            DO 55 I=LXMIN,LX
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(7).')
   55       CONTINUE
            DO 60 J=LYMIN,LY
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.1)CALL XABORT('@SIM: INTEGER DATA EXPECTED(3).')
            DO 60 I=LXMIN,LX
            CALL REDGET(ITYP,NITMA,FLOT,TEXT4,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(8).')
            READ(TEXT4,'(A4)') NITMA
            WRITE(TEXT4B,'(A1,I2.2)') HHX(I),IHY(J)
            IBASE(ICYCLE+(J-1)*LX+I-1)=NITMA
   60       CONTINUE
            CALL SIMQMP(LX,LY,LXMIN,LYMIN,HHX,IHY,IBASE(ICYCLE))
          ENDIF
          IF(IMPX.GE.2) THEN
            CALL SETARA(IBASE,LX*LY,ICYCLE2)
            DO I=1,LX
              DO J=1,LY
                IBASE(ICYCLE2+(J-1)*LX+I-1)=IBASE(ICYCLE+(J-1)*LX+I-1)
                WRITE(TEXT4,'(A4)') IBASE(ICYCLE2+(J-1)*LX+I-1)
                IF(TEXT4.EQ.'|') TEXT4=' |'
                IF(TEXT4.EQ.'-') TEXT4=' -'
                READ(TEXT4,'(A4)') IBASE(ICYCLE2+(J-1)*LX+I-1)
              ENDDO
            ENDDO
            WRITE (6,195)
            WRITE (6,196) (HHX(I),I=1,LX)
            DO J=1,LY
              WRITE (6,197) IHY(J),(IBASE(ICYCLE2+(J-1)*LX+I-1),I=1,LX)
            ENDDO
            CALL RLSARA(IBASE(ICYCLE2))
          ENDIF
          CALL XDRSET(BASE(IOBURNU),NCH*NB,-999.0)
          IF(HOLD.NE.' ') THEN
            JNDCY=SIMIND(IPMAP,IMPX,HOLD,INDCY,BURNCY)
            JPMAP=LCMGID(IPMAP,HOLD)
            KPMAP=LCMGIL(JPMAP,JNDCY)
            CALL LCMGET(KPMAP,'NAME',IBASE(IONAME))
            CALL LCMGET(KPMAP,'BURN-INST',BASE(IOBURNU))
            CALL LCMGET(KPMAP,'FLMIX',IBASE(IOFMIX))
          ENDIF
          CALL SETARA(IBASE,NCH*NB,IINFMIX)
          CALL LCMGET(IPMAP,'FLMIX-INI',IBASE(IINFMIX))
          CALL SETARA(BASE,NCH,IZONE2)
          CALL SIMPOS(LX,LY,NCH,NB,HCYCL,HOLD,HHX,IHY,IBASE(IZONE),
     >    IBASE(IINFMIX),IBASE(ICYCLE),IBASE(INAME),BASE(IBURNUP),
     >    IBASE(IFMIX),IBASE(IONAME),BASE(IOBURNU),IBASE(IOFMIX),
     >    BASE(IZONE2))
          CALL RLSARA(BASE(IZONE2))
          CALL RLSARA(IBASE(IINFMIX))
          JPMAP=LCMLID(IPMAP,HCYCL,1)
          KPMAP=LCMDIL(JPMAP,1)
          CALL LCMPUT(KPMAP,'CYCLE',LX*LY,3,IBASE(ICYCLE))
          CALL LCMPUT(KPMAP,'NAME',3*NCH,3,IBASE(INAME))
          CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
          CALL LCMPUT(KPMAP,'FLMIX',NCH*NB,1,IBASE(IFMIX))
          CALL RLSARA(IBASE(ICYCLE))
          GO TO 30
        ELSE IF(TEXT.EQ.'SPEC') THEN
          JPMAP=LCMGID(IPMAP,HCYCL)
          KPMAP=LCMGIL(JPMAP,1)
          CALL LCMGET(KPMAP,'NAME',IBASE(INAME))
          CALL LCMGET(KPMAP,'BURN-INST',BASE(IBURNUP))
          CALL LCMGET(KPMAP,'FLMIX',IBASE(IFMIX))
          IASMB1=0
          INDCY=-1
          BURNCY=-999.0
   70     CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(9).')
   80     IF(TEXT.EQ.'ENDCYCLE') THEN
            JPMAP=LCMGID(IPMAP,HCYCL)
            KPMAP=LCMGIL(JPMAP,1)
            CALL LCMPUT(KPMAP,'NAME',3*NCH,3,IBASE(INAME))
            CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
            CALL LCMPUT(KPMAP,'FLMIX',NCH*NB,1,IBASE(IFMIX))
            GO TO 120
          ELSE IF((TEXT.EQ.'DIST-AX').OR.(TEXT.EQ.'BURN-STEP')) THEN
            JPMAP=LCMGID(IPMAP,HCYCL)
            KPMAP=LCMGIL(JPMAP,1)
            CALL LCMPUT(KPMAP,'NAME',3*NCH,3,IBASE(INAME))
            CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
            CALL LCMPUT(KPMAP,'FLMIX',NCH*NB,1,IBASE(IFMIX))
            GO TO 40
          ELSE IF(TEXT.EQ.'SET') THEN
            BURN=-999.0
            IFUEL=0
            IF(IASMB1.EQ.0) CALL XABORT('@SIM: ASMB1 NOT SET.')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(10).')
            IF(TEXT.EQ.'AVGB') THEN
              CALL REDGET(ITYP,NITMA,BURN,TEXT,DFLOT)
              IF(ITYP.NE.2)CALL XABORT('@SIM: REAL DATA EXPECTED(3).')
            ELSE IF(TEXT.EQ.'FUEL') THEN
              CALL REDGET(ITYP,IFUEL,FLOT,TEXT,DFLOT)
              IF(ITYP.NE.1)CALL XABORT('@SIM: INTEGER DATA EXPECTED(4)')
            ELSE
              CALL XABORT('@SIM: AVGB OR FUEL KEYWORD EXPECTED')
            ENDIF
            CALL SIMSET(NCH,NB,HCYCL,IASMB1,ASMB1,BURN,IFUEL,
     >      IBASE(IZONE),IBASE(INAME),BASE(IBURNUP),IBASE(IFMIX))
            IASMB1=0
          ELSE IF(TEXT.EQ.'FROM') THEN
            IF(IASMB1.EQ.0) CALL XABORT('@SIM: ASMB1 NOT SET.')
            CALL REDGET(ITYP,NITMA,FLOT,HOLD,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(11).')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(12).')
            IF(TEXT.NE.'AT')CALL XABORT('@SIM: AT KEYWORD EXPECTED')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT4B,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(13).')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(14).')
            IF(TEXT.EQ.'BURN') THEN
              CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
              IF(ITYP.EQ.1) THEN
                INDCY=NITMA
              ELSE IF(ITYP.EQ.2) THEN
                BURNCY=FLOT
              ELSE
                CALL XABORT('@SIM: INTEGER OR REAL DATA EXPECTED.')
              ENDIF
            ELSE
              JNDCY=SIMIND(IPMAP,IMPX,HOLD,INDCY,BURNCY)
              JPMAP=LCMGID(IPMAP,HOLD)
              KPMAP=LCMGIL(JPMAP,JNDCY)
              CALL LCMGET(KPMAP,'NAME',IBASE(IONAME))
              CALL LCMGET(KPMAP,'BURN-INST',BASE(IOBURNU))
              CALL LCMGET(KPMAP,'FLMIX',IBASE(IOFMIX))
              CALL SETARA(BASE,NCH,IZONE2)
              CALL SIMCPY(NCH,NB,HCYCL,IASMB1,ASMB1,TEXT4B,IBASE(IZONE),
     >        IBASE(INAME),BASE(IBURNUP),IBASE(IFMIX),IBASE(IONAME),
     >        BASE(IOBURNU),IBASE(IOFMIX),BASE(IZONE2))
              CALL RLSARA(BASE(IZONE2))
              IASMB1=0
              GO TO 80
            ENDIF
          ELSE
            IASMB1=IASMB1+1
            IF(IASMB1.GT.MAXIAS) CALL XABORT('@SIM: MAXIAS OVERFLOW.')
            ASMB1(IASMB1)=TEXT(:4)
          ENDIF
          GO TO 70
        ELSE IF(TEXT.EQ.'DIST-AX') THEN
          IF(HCYCL.EQ.' ') CALL XABORT('@SIM: HCNEW NOT DEFINED.')
          CALL SETARA(BASE,NB,IFORM)
          JPMAP=LCMGID(IPMAP,HCYCL)
          KPMAP=LCMGIL(JPMAP,1)
          CALL LCMGET(KPMAP,'BURN-INST',BASE(IBURNUP))
          IASMB1=0
          INDCY=-1
          BURNCY=-999.0
   90     CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(15).')
  100     IF(TEXT.EQ.'ENDCYCLE') THEN
            CALL RLSARA(BASE(IFORM))
            JPMAP=LCMGID(IPMAP,HCYCL)
            KPMAP=LCMGIL(JPMAP,1)
            CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
            GO TO 120
          ELSE IF((TEXT.EQ.'SPEC').OR.(TEXT.EQ.'BURN-STEP')) THEN
            CALL RLSARA(BASE(IFORM))
            JPMAP=LCMGID(IPMAP,HCYCL)
            KPMAP=LCMGIL(JPMAP,1)
            CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
            GO TO 40
          ELSE IF(TEXT.EQ.'SET') THEN
            DO IB=1,NB
              CALL REDGET(ITYP,NITMA,BASE(IFORM+IB-1),TEXT,DFLOT)
              IF(ITYP.NE.2) CALL XABORT('@SIM: REAL AXN EXPECTED.')
            ENDDO
            CALL SETARA(BASE,NCH,IZONE2)
            CALL SIMDIS(.TRUE.,NCH,NB,HCYCL,IASMB1,ASMB1,BASE(IFORM),
     >      TEXT4B,IBASE(IZONE),BASE(IBURNUP),BASE(IBURNUP),
     >      BASE(IZONE2))
            CALL RLSARA(BASE(IZONE2))
            IASMB1=0
          ELSE IF(TEXT.EQ.'FROM') THEN
            IF(IASMB1.EQ.0) CALL XABORT('@SIM: ASMB1 NOT SET.')
            CALL REDGET(ITYP,NITMA,FLOT,HOLD,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(16).')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(17).')
            IF(TEXT.NE.'AT')CALL XABORT('@SIM: AT KEYWORD EXPECTED')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT4B,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(18).')
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED'
     >      //'(19).')
            IF(TEXT.EQ.'BURN') THEN
              CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
              IF(ITYP.EQ.1) THEN
                INDCY=NITMA
              ELSE IF(ITYP.EQ.2) THEN
                BURNCY=FLOT
              ELSE
                CALL XABORT('@SIM: INTEGER OR REAL DATA EXPECTED.')
              ENDIF
            ELSE
              JNDCY=SIMIND(IPMAP,IMPX,HOLD,INDCY,BURNCY)
              JPMAP=LCMGID(IPMAP,HOLD)
              KPMAP=LCMGIL(JPMAP,JNDCY)
              CALL LCMGET(KPMAP,'BURN-INST',BASE(IOBURNU))
              CALL SETARA(BASE,NCH,IZONE2)
              CALL SIMDIS(.FALSE.,NCH,NB,HCYCL,IASMB1,ASMB1,BASE(IFORM),
     >        TEXT4B,IBASE(IZONE),BASE(IBURNUP),BASE(IOBURNU),
     >        BASE(IZONE2))
              CALL RLSARA(BASE(IZONE2))
              IASMB1=0
              GO TO 100
            ENDIF
          ELSE
            IASMB1=IASMB1+1
            IF(IASMB1.GT.MAXIAS) CALL XABORT('@SIM: MAXIAS OVERFLOW.')
            ASMB1(IASMB1)=TEXT(:4)
          ENDIF
          GO TO 90
        ELSEIF(TEXT.EQ.'ENDCYCLE')THEN
          GO TO 120
        ELSEIF(TEXT.EQ.'TIME')THEN
*         TIME VALUE
          IF(TIME.NE.0.0)CALL XABORT('@SIM: TIME ALREADY SPECIFIED(1).')
          IF(BURNSTEP.NE.0.0)CALL XABORT('@SIM: BURNSTEP ALREADY //
     >    //SPECIFIED(1).')
          CALL REDGET(ITYP,NITMA,TIME,TEXT,DFLOT)
          IF(ITYP.NE.2)CALL XABORT('@SIM: REAL DATA EXPECTED(1).')
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(20).')
          IF(TIME.LT.0.)CALL XABORT('@SIM: EXPECTING REAL>0 (1).')
          IF(TEXT.EQ.'DAY')THEN
            TIME=TIME
          ELSEIF(TEXT.EQ.'HOUR')THEN
            TIME=TIME/24.
          ELSEIF(TEXT.EQ.'MINUTE')THEN
            TIME=TIME/(24.*60.)
          ELSEIF(TEXT.EQ.'SECOND')THEN
            TIME=TIME/(24.*60.*60.)
          ELSE
            CALL XABORT('@SIM: EXPECTING DAY|HOUR|MINUTE|SECOND.')
          ENDIF
          GOTO 130
        ELSEIF(TEXT.EQ.'BURN-STEP')THEN
*         BURN-STEP
          IF(TIME.NE.0.)CALL XABORT('@SIM: TIME ALREADY SPECIFIED(2).')
          IF(BURNSTEP.NE.0.)CALL XABORT('@SIM: BURNSTEP ALREADY '
     >    //'SPECIFIED(2).')
          CALL REDGET(ITYP,NITMA,BURNSTEP,TEXT,DFLOT)
          IF(ITYP.NE.2)CALL XABORT('@SIM: REAL DATA EXPECTED(2).')
          IF(BURNSTEP.LE.0.)CALL XABORT('@SIM: EXPECTING REAL>0 (2).')
          GO TO 130
        ELSE
          CALL XABORT('@SIM: WRONG KEYWORD: '//TEXT//' (1).')
        ENDIF
      ELSE IF(TEXT.EQ.'COMPARE') THEN
*       Compare two fields of values
        IF(HCYCL.NE.' ') CALL XABORT('@SIM: HCNEW STILL ACTIVE.')
        INDCY1=-1
        BURNCY1=-999.0
        INDCY2=-1
        BURNCY2=-999.0
        IMODE=0
        CALL REDGET(ITYP,NITMA,FLOT,HC1,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(21).')
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(22).')
        IF(TEXT.EQ.'BURN') THEN
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.EQ.1) THEN
            INDCY1=NITMA
          ELSE IF(ITYP.EQ.2) THEN
            BURNCY1=FLOT
          ELSE
            CALL XABORT('@SIM: INTEGER OR REAL DATA EXPECTED.')
          ENDIF
          CALL REDGET(ITYP,NITMA,FLOT,HC2,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(23).')
        ELSE
          HC2=TEXT
        ENDIF
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(24).')
        IF(TEXT.EQ.'BURN') THEN
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.EQ.1) THEN
            INDCY2=NITMA
          ELSE IF(ITYP.EQ.2) THEN
            BURNCY2=FLOT
          ELSE
            CALL XABORT('@SIM: INTEGER OR REAL DATA EXPECTED.')
          ENDIF
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(25).')
        ENDIF
        IF(TEXT.EQ.'DIST-BURN') THEN
          IMODE=1
        ELSE IF(TEXT.EQ.'DIST-POWR') THEN
          IMODE=2
        ELSE
          CALL XABORT('@SIM: DIST-BURN OR DIST-POWR KEYWORD EXPECTED.')
        ENDIF
        CALL SETARA(BASE,NCH*NB,IDIST1)
        CALL SETARA(BASE,NCH*NB,IDIST2)
        CALL SIMCOM(IPMAP,IMPX,IMODE,NCH,NB,HC1,HC2,INDCY1,INDCY2,
     >  BURNCY1,BURNCY2,ERROR,BASE(IDIST1),BASE(IDIST2))
        CALL RLSARA(BASE(IDIST2))
        CALL RLSARA(BASE(IDIST1))
        CALL REDGET(ITYP,NITMA,ERROR,TEXT,DFLOT)
        IF(ITYP.NE.-2) CALL XABORT('SIM: OUTPUT REAL EXPECTED')
        ITYP=2
        CALL REDPUT(ITYP,NITMA,ERROR,TEXT,DFLOT)
      ELSE IF(TEXT.EQ.'SET-PARAM') THEN
*       Reset a global parameter
        IF(HCYCL.NE.' ') CALL XABORT('@SIM: HCNEW STILL ACTIVE.')
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(26).')
        CALL REDGET(ITYP,NITMA,VALUE,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@SIM: REAL FOR VALUE EXPECTED.')
        JPMAP=LCMGID(IPMAP,'PARAM')
        DO 110 IPAR=1,NPARM
        KPMAP=LCMGIL(JPMAP,IPAR)
        CALL LCMGTC(KPMAP,'P-NAME',12,1,PNAME)
        CALL LCMGET(KPMAP,'P-TYPE',ITYPE)
        IF(ITYPE.EQ.1) THEN
          IF(PNAME.EQ.TEXT) THEN
            CALL LCMPUT(KPMAP,'P-VALUE',1,2,VALUE)
            IF(IMPX.GT.0) WRITE(6,200) PNAME,VALUE
            GO TO 10
          ELSE
            GO TO 110
          ENDIF
        ELSE IF(ITYPE.EQ.2) THEN
          CALL XABORT('@SIM: CANNOT RESET LOCAL PARAMETER: '//TEXT)
        ENDIF
  110   CONTINUE
        CALL XABORT('@SIM: GLOBAL PARAMETER NAME NOT FOUND: '//TEXT)
      ELSEIF(TEXT.EQ.';')THEN
        GO TO 140
      ELSE
*       KEYWORD DOES NOT MATCH
        CALL XABORT('@SIM: WRONG KEYWORD: '//TEXT//' (2).')
      ENDIF
      GO TO 10
*----
*  RECOVER THE BURNUP AND FUEL TYPE DISTRIBUTIONS IN DIRECTORY HCYCL
*  AFTER REFUELLING
*----
  120 IF(HCYCL.EQ.' ') CALL XABORT('@SIM: HCNEW NOT DEFINED.')
      JPMAP=LCMGID(IPMAP,HCYCL)
      KPMAP=LCMGIL(JPMAP,1)
      CALL LCMGET(KPMAP,'BURN-INST',BASE(IBURNUP))
      CALL LCMGET(KPMAP,'FLMIX',IBASE(IFMIX))
      DO ICH=1,NCH
        DO IB=1,NB
          IOF=(IB-1)*NCH+ICH
          IF(BASE(IBURNUP+IOF-1).EQ.-999.0) THEN
            WRITE(HSMG,'(30HSIM: BURNUP NOT SET IN CHANNEL,I4,
     >      11H AND BUNDLE,I4,1H.)') ICH,IB
            CALL XABORT(HSMG)
          ENDIF
        ENDDO
      ENDDO
      CALL LCMPUT(IPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNUP))
      CALL LCMPUT(IPMAP,'FLMIX',NCH*NB,1,IBASE(IFMIX))
      CALL RLSARA(IBASE(IOFMIX))
      CALL RLSARA(BASE(IOBURNU))
      CALL RLSARA(IBASE(IONAME))
      CALL RLSARA(IBASE(INAME))
      CALL RLSARA(IBASE(IFMIX))
      CALL RLSARA(BASE(IBURNUP))
      CALL RLSARA(IBASE(IZONE))
      HCYCL=' '
      GO TO 10
*----
*  PERFORM CALCULATION
*----
  130 IF(HCYCL.EQ.' ') CALL XABORT('@SIM: HCNEW NOT DEFINED.')
      CALL SETARA(IBASE,NCH*NB,IBUNDMIX)
      CALL SETARA(IBASE,NF,IFLMIX)
      CALL SETARA(BASE,NF,IFLWEIGHT)
      CALL SETARA(BASE,NCH*NB,IBUNDPOW)
      IF(IPPOW.EQ.0) THEN
        CALL LCMLEN(IPMAP,'BUND-PW',LENGT,ITYP)
        IF(LENGT.EQ.0)CALL XABORT('@SIM: MISSING BUND-PW DATA IN '
     >  //'L_MAP OBJECT.')
        CALL LCMGET(IPMAP,'BUND-PW',BASE(IBUNDPOW))
      ELSE 
        CALL LCMLEN(IPPOW,'POWER-CHAN',LENGT,ITYP)
        IF(LENGT.EQ.0)CALL XABORT('@SIM: MISSING POWER-CHAN DATA I'
     >  //'N L_POWER OBJECT.')
        CALL LCMGET(IPPOW,'POWER-BUND',BASE(IBUNDPOW))
      ENDIF
      TTIME=TTIME+TIME
      CALL SETARA(BASE,NCH*NB,IBURNINST)
      IF(IMPX.GE.8) THEN
        CALL SIMOUT(IPMAP,IMPX,BASE(IBURNINST),IBASE(IZONE),NCH,NB,LX,
     >  LY,HHX,IHY,'BEGIN')
      ENDIF
      CALL TINSTB(IPMAP,IPPOW,TIME,BURNSTEP,NCH,NB,NF,IMPX,BURNAVG,
     > IBASE(IBUNDMIX),IBASE(IFLMIX),BASE(IFLWEIGHT),BASE(IBUNDPOW),
     > BASE(IBURNINST))
      CALL RLSARA(IBASE(IBUNDMIX))
      CALL RLSARA(IBASE(IFLMIX))
      CALL RLSARA(BASE(IFLWEIGHT))
*
      CALL LCMLEN(IPMAP,HCYCL,ILONG,ITYLCM)
      IF(IMPX.GT.0) WRITE(6,300) ILONG+1,HCYCL,BURNAVG
      JPMAP=LCMLID(IPMAP,HCYCL,ILONG+1)
      KPMAP=LCMGIL(JPMAP,1)
      CALL LCMGET(KPMAP,'NAME',IBASE(INAME))
      CALL LCMGET(KPMAP,'FLMIX',IBASE(IFMIX))
      KPMAP=LCMDIL(JPMAP,ILONG+1)
      CALL LCMPUT(KPMAP,'TIME',1,2,TTIME)
      CALL LCMPUT(KPMAP,'BURNAVG',1,2,BURNAVG)
      CALL LCMPUT(KPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNINST))
      CALL LCMPUT(KPMAP,'POWER-BUND',NCH*NB,2,BASE(IBUNDPOW))
      CALL LCMPUT(KPMAP,'NAME',3*NCH,3,IBASE(INAME))
      CALL LCMPUT(KPMAP,'FLMIX',NCH*NB,1,IBASE(IFMIX))
      CALL LCMPUT(IPMAP,'BURN-INST',NCH*NB,2,BASE(IBURNINST))
      CALL SIMOUT(IPMAP,IMPX,BASE(IBURNINST),IBASE(IZONE),NCH,NB,LX,LY,
     > HHX,IHY,'END  ')
      CALL RLSARA(BASE(IBUNDPOW))
      CALL RLSARA(BASE(IBURNINST))
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@SIM: CHARACTER DATA EXPECTED(27).')
      IF(TEXT.NE.'ENDCYCLE')CALL XABORT('@SIM: ENDCYCLE KEYWORD EXPECT'
     > //'ED.')
      CALL RLSARA(IBASE(IOFMIX))
      CALL RLSARA(BASE(IOBURNU))
      CALL RLSARA(IBASE(IONAME))
      CALL RLSARA(IBASE(INAME))
      CALL RLSARA(IBASE(IFMIX))
      CALL RLSARA(BASE(IBURNUP))
      CALL RLSARA(IBASE(IZONE))
      HCYCL=' '
      GOTO 10

  140 IF(HCYCL.NE.' ') CALL XABORT('@SIM: HCNEW STILL ACTIVE.')
      CALL LCMSIX(IPMAP,' ',0)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
      CALL LCMPUT(IPMAP,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPMAP,'DEPL-TIME',1,2,TTIME)
      CALL LCMPUT(IPMAP,'REF-CHAN',NCH,2,BASE(IRFCHAN))
      CALL RLSARA(BASE(IRFCHAN))
      RETURN
*
  190 FORMAT(/38H SIM: NUMBER OF ASSEMBLY SUBDIVISIONS=,I4/
     1 6X,25HNUMBER OF FUEL CHANNELS =,I4/
     2 6X,34HNUMBER OF ASSEMBLIES ALONG X AXIS=,I3/
     3 6X,34HNUMBER OF ASSEMBLIES ALONG Y AXIS=,I3/)
  195 FORMAT(' SIM: RELOADING PATTERN :')
  196 FORMAT(2X,32A4)
  197 FORMAT(1X,I2,1X,32A4)
  200 FORMAT(/27H SIM: SET GLOBAL PARAMETER ,A,2H =,1P E12.4)
  300 FORMAT(/36H SIM: STORE INFORMATION IN LIST ITEM,I3,10H OF CYCLE ,
     > A12,10H AT BURNUP,1P,E12.4,8H MW-D/T./)
      END
