*DECK CHAB04
      SUBROUTINE CHAB04(IPLIB,IMPX,IRHS,NGRP,NLEG,IMOD,IL,IGM,IGP,
     1 VALUE,DELTA,FMULT,NJJ,IJJ,GAR1,GAR2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* modify scattering information in a microlib or in a draglib.
*
*Copyright:
* Copyright (C) 2011 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   LCM pointer to the Microlib or Draglib.
* IMPX    print index.
* IRHS    type of IPLIB: =1: Microlib; =2: Draglib.
* NGRP    number of energy groups.
* NLEG    max Legendre order of scattering anisotropy (1=isotropic,
*         etc.).
* IMOD    type of modification: =1,2: replace the value; =3: increase by
*         VALUE; =4: multiply by VALUE.
* IL      Legendre order under consideration.
* IGM     first energy group to modify.
* IGP     last energy group to modify.
* VALUE   value array used in scattering modification operation.
*
*Parameters: output
* DELTA   difference in scattering cross section value.
* FMULT   multiplicative modification factors.
*
*Parameters: scratch
* NJJ
* IJJ
* GAR1
* GAR2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IMPX,IRHS,NGRP,NLEG,IMOD,IL,IGM,IGP,NJJ(NGRP),
     1 IJJ(NGRP)
      REAL VALUE(NGRP),DELTA(NGRP),FMULT(NGRP),GAR1(NGRP),
     1 GAR2(NGRP,NGRP)
*----
*  LOCAL VARIABLES
*----
      CHARACTER CM*2
      COMMON BASE(1)
*      
      CALL XDRSET(DELTA,NGRP,0.0)
      CALL XDRSET(FMULT,NGRP,1.0)
*----
*  RECOVER SCATTERING INFORMATION
*----
      IF(IL.GE.NLEG) CALL XABORT('CHAB04: LEGENDRE INDEX OVERFLOW.')
      CALL SETARA(BASE,NGRP*NGRP,IXSSCM)
      CALL SETARA(BASE,NLEG,ITYPRO)
      IF(IRHS.EQ.1) THEN
         CALL XDRLGS(IPLIB,-1,IMPX,IL,IL,1,NGRP,GAR1,GAR2,BASE(ITYPRO),
     1   BASE(IXSSCM),NJJ,IJJ)
      ELSE IF(IRHS.EQ.2) THEN
         WRITE (CM,'(I2.2)') IL
         CALL XDRSET(GAR1,NGRP,0.0)
         CALL XDRSET(GAR2,NGRP*NGRP,0.0)
         CALL LCMGET(IPLIB,'NJJS'//CM,NJJ)
         CALL LCMGET(IPLIB,'IJJS'//CM,IJJ)
         LENGT=0
         DO 10 I=1,NGRP
   10    LENGT=LENGT+NJJ(I)
         CALL XDRSET(BASE(IXSSCM),LENGT,0.0)
         CALL LCMGET(IPLIB,'SCAT'//CM,BASE(IXSSCM))
         IGAR=0
         DO 20 IG2=1,NGRP
         DO 20 IG1=IJJ(IG2),IJJ(IG2)-NJJ(IG2)+1,-1
         IGAR=IGAR+1
         GAR2(IG2,IG1)=BASE(IXSSCM+IGAR-1)
   20    GAR1(IG1)=GAR1(IG1)+GAR2(IG2,IG1)
      ENDIF
*----
*  MODIFY SCATTERING INFORMATION
*----
      DO 40 IG2=IGM,IGP
      IF(GAR1(IG2).NE.0.0) THEN
         DO 30 IG1=1,NGRP  
   30    GAR2(IG1,IG2)=GAR2(IG1,IG2)/GAR1(IG2)
      ENDIF
      IF((IMOD.EQ.1).OR.(IMOD.EQ.2)) THEN
         IF(GAR1(IG2).EQ.0.0) THEN
            FMULT(IG2)=1.0
         ELSE
            FMULT(IG2)=VALUE(IG2)/GAR1(IG2)
         ENDIF
         DELTA(IG2)=VALUE(IG2)-GAR1(IG2)
         GAR1(IG2)=VALUE(IG2)
      ELSE IF(IMOD.EQ.3) THEN
         IF(GAR1(IG2).EQ.0.0) THEN
            FMULT(IG2)=1.0
         ELSE
            FMULT(IG2)=1.0+VALUE(IG2)/GAR1(IG2)
         ENDIF
         DELTA(IG2)=VALUE(IG2)
         GAR1(IG2)=GAR1(IG2)+VALUE(IG2)
      ELSE IF(IMOD.EQ.4) THEN
         FMULT(IG2)=VALUE(IG2)
         DELTA(IG2)=GAR1(IG2)*(VALUE(IG2)-1.0)
         GAR1(IG2)=GAR1(IG2)*VALUE(IG2)
      ENDIF
      DO 40 IG1=1,NGRP
   40 GAR2(IG1,IG2)=GAR2(IG1,IG2)*GAR1(IG2)
*----
*  SAVE SCATTERING INFORMATION
*----
      IF(IRHS.EQ.1) THEN
         CALL XDRLGS(IPLIB,1,IMPX,IL,IL,1,NGRP,GAR1,GAR2,BASE(ITYPRO),
     1   BASE(IXSSCM),NJJ,IJJ)
      ELSE IF(IRHS.EQ.2) THEN
         IGAR=0
         DO 50 IG2=1,NGRP
         DO 50 IG1=IJJ(IG2),IJJ(IG2)-NJJ(IG2)+1,-1
         IGAR=IGAR+1
   50    BASE(IXSSCM+IGAR-1)=GAR2(IG2,IG1)
         CALL LCMPUT(IPLIB,'SCAT'//CM,IGAR,2,BASE(IXSSCM))
      ENDIF
      CALL RLSARA(BASE(ITYPRO))
      CALL RLSARA(BASE(IXSSCM))
      RETURN
      END
