*DECK DOORAV
      SUBROUTINE DOORAV (CDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1 NBMIX,NANI,MAT,VOL,KNORM,LEAKSW,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of system matrices. Vectorial version.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* CDOOR   name of the geometry/solution operator.
* JPSYS   pointer to the PIJ LCM object (L_PIJ signature). JPSYS is
*         a list of directories.
* NPSYS   index array pointing to the JPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* NANI    number of Legendre orders.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   normalization scheme.
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW
      INTEGER JPSYS,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,NANI,
     > MAT(NREG),KNORM
      REAL VOL(NREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER ISTATE(NSTATE)
      LOGICAL LBIHET
      COMMON BASE(1)
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      LBIHET=ISTATE(40).NE.0
      IF(LBIHET) THEN
         CALL SETARA(BASE,NREG,IMAT)
         CALL SETARA(BASE,NREG,IVOL)
         DO I=1,NREG
            BASE(IMAT+I-1)=MAT(I)
            BASE(IVOL+I-1)=VOL(I)
         ENDDO
         NREGAR=NREG
         NBMIXG=NBMIX
         CALL DOORAB(CDOOR,JPSYS,NPSYS,IPTRK,IMPX,NGRP,NREG,NBMIX,NANI,
     1   MAT,VOL)
      ENDIF
*
      IF (CDOOR.EQ.'SYBIL') THEN
         CALL SETARA(BASE,NBMIX+1,ISGAR)
         CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
         DO 100 IGR=1,NGRP
            IOFSET=NPSYS(IGR)
            IF(IOFSET.NE.0) THEN
               KPSYS=LCMGIL(JPSYS,IOFSET)
               IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
               CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
               IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TXSC '
     1         //'LENGTH(1).')
               CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
               CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
               CALL SYBILA(KPSYS,IPTRK,IMPX,NREG,NBMIX,MAT,BASE(ISGAR),
     1                     BASE(ISGAS))
               IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
            ENDIF
 100     CONTINUE
         CALL RLSARA(BASE(ISGAS))
         CALL RLSARA(BASE(ISGAR))
      ELSE IF (CDOOR.EQ.'SN') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         ITYPE=ISTATE(6)
         IF(ISTATE(19).EQ.1) THEN
*           SYNTHETIC ACCELERATION.
            IF(IMPX.GT.0) WRITE (6,'(/29H DOORAV: SYNTHETIC ACCELERATI,
     1      20HON ASSEMBLY FOLLOWS:)')
            CALL LCMSIX(IPTRK,'DSA',1)
            CALL SETARA(BASE,(NBMIX+1)*2,ISGAR)
            CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
            CALL SETARA(BASE,NBMIX+1,ISGAD)
            DO 150 IGR=1,NGRP
               IOFSET=NPSYS(IGR)
               IF(IOFSET.NE.0) THEN
                  KPSYS=LCMGIL(JPSYS,IOFSET)
                  CALL XDISET(ISTATE,NSTATE,0)
                  CALL LCMPUT(KPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
                  IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
                  CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
                  IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TX'
     1            //'SC LENGTH(2).')
                  CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
                  CALL LCMLEN(KPSYS,'DRAGON-T1XSC',ILONG,ITYLCM)
                  IF(ILONG.NE.0) THEN
                   CALL LCMGET(KPSYS,'DRAGON-T1XSC',BASE(ISGAR+NBMIX+1))
                  ELSE
                   CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR+NBMIX+1))
                  ENDIF
                  CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
                  IF(ITYPE.EQ.7) THEN
                     CALL TRIVA(KPSYS,IPTRK,IMPX,NREG,NBMIX,NANI,MAT,
     1                          VOL,BASE(ISGAR),BASE(ISGAS),BASE(ISGAD))
                  ELSE
                     CALL BIVAA(KPSYS,IPTRK,IMPX,NREG,NBMIX,NANI,MAT,
     1                          VOL,BASE(ISGAR),BASE(ISGAS),BASE(ISGAD))
                  ENDIF
                  IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
               ENDIF
 150        CONTINUE
            CALL RLSARA(BASE(ISGAD))
            CALL RLSARA(BASE(ISGAS))
            CALL RLSARA(BASE(ISGAR))
            CALL LCMSIX(IPTRK,' ',2)
         ENDIF
      ELSE IF (CDOOR.EQ.'BIVAC') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         NLF=ISTATE(14)
         ISCAT=ISTATE(16)
         CALL SETARA(BASE,(NBMIX+1)*2,ISGAR)
         CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
         CALL SETARA(BASE,NBMIX+1,ISGAD)
         DO 190 IGR=1,NGRP
            IOFSET=NPSYS(IGR)
            IF(IOFSET.NE.0) THEN
               KPSYS=LCMGIL(JPSYS,IOFSET)
               IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
               CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
               IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TXSC '
     1         //'LENGTH(3).')
               CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
               IF(NLF.EQ.0) THEN
                  CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(ISGAD))
               ELSE IF(ISCAT.LT.0) THEN
                  CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(ISGAD))
                  BASE(ISGAR+NBMIX+1)=1.0E10
                  DO 180 IMIX=1,NBMIX
 180              BASE(ISGAR+NBMIX+1+IMIX)=1.0/(3.0*BASE(ISGAD+IMIX))
               ELSE IF(ISCAT.GT.0) THEN
                  CALL LCMLEN(KPSYS,'DRAGON-T1XSC',ILONG,ITYLCM)
                  IF(ILONG.NE.0) THEN
                   CALL LCMGET(KPSYS,'DRAGON-T1XSC',BASE(ISGAR+NBMIX+1))
                  ELSE
                   CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR+NBMIX+1))
                  ENDIF
               ENDIF
               CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
               CALL BIVAA(KPSYS,IPTRK,IMPX,NREG,NBMIX,NANI,MAT,VOL,
     1                    BASE(ISGAR),BASE(ISGAS),BASE(ISGAD))
               IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
            ENDIF
 190     CONTINUE
         CALL RLSARA(BASE(ISGAD))
         CALL RLSARA(BASE(ISGAS))
         CALL RLSARA(BASE(ISGAR))
      ELSE IF (CDOOR.EQ.'TRIVAC') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         NLF=ISTATE(30)
         ISCAT=ISTATE(32)
         CALL SETARA(BASE,(NBMIX+1)*2,ISGAR)
         CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
         CALL SETARA(BASE,NBMIX+1,ISGAD)
         DO 210 IGR=1,NGRP
            IOFSET=NPSYS(IGR)
            IF(IOFSET.NE.0) THEN
               KPSYS=LCMGIL(JPSYS,IOFSET)
               CALL XDISET(ISTATE,NSTATE,0)
               CALL LCMPUT(KPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
               IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
               CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
               IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TXSC '
     1         //'LENGTH(4).')
               CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
               IF(NLF.EQ.0) THEN
                  CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(ISGAD))
               ELSE IF(ISCAT.LT.0) THEN
                  CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(ISGAD))
                  BASE(ISGAR+NBMIX+1)=1.0E10
                  DO 200 IMIX=1,NBMIX
 200              BASE(ISGAR+NBMIX+1+IMIX)=1.0/(3.0*BASE(ISGAD+IMIX))
               ELSE IF(ISCAT.GT.0) THEN
                  CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR+NBMIX+1))
               ENDIF
               CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
               CALL TRIVA(KPSYS,IPTRK,IMPX,NREG,NBMIX,NANI,MAT,VOL,
     1                    BASE(ISGAR),BASE(ISGAS),BASE(ISGAD))
               IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
            ENDIF
 210     CONTINUE
         CALL RLSARA(BASE(ISGAD))
         CALL RLSARA(BASE(ISGAS))
         CALL RLSARA(BASE(ISGAR))
      ELSE IF ((CDOOR.EQ.'MCCG').OR.(CDOOR.EQ.'EXCELL')) THEN
         CALL MCCGA(JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,
     1              NANI,MAT,VOL,KNORM,LEAKSW,TITR)
      ELSE
         CALL XABORT('DOORAV: UNKNOWN DOOR:'//CDOOR//'.')
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         NREG=NREGAR
         NBMIX=NBMIXG
         DO I=1,NREG
            MAT(I)=BASE(IMAT+I-1)
            VOL(I)=BASE(IVOL+I-1)
         ENDDO
         CALL RLSARA(BASE(IMAT))
         CALL RLSARA(BASE(IVOL))
      ENDIF
      RETURN
      END
