*DECK DUO
      SUBROUTINE DUO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* perturbative analysis of two systems and determination of the origins
* of Keff discrepancies.
*
*Copyright:
* Copyright (C) 2013 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): read-only type(L_LIBRARY) first system;
*         HENTRY(2): read-only type(L_FLUX) direct flux;
*         HENTRY(3): read-only type(L_FLUX) adjoint flux;
*         HENTRY(4): read-only type(L_LIBRARY) second system;
*         HENTRY(5): read-only type(L_FLUX) direct flux.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER HSIGN*12,CARLIR*12
      INTEGER KCHAR(3),ISTATE(NSTATE)
      REAL REALIR
      DOUBLE PRECISION DBLLIR
      LOGICAL LENER,LISOT,LMIXT,LREAC
      COMMON BASE(1)
*----
*  PARAMETER VALIDATION.
*----
      IF(NENTRY.NE.2) CALL XABORT('DUO: TWO PARAMETERS EXPECTED.')
      DO IEN=1,2
        IF((IENTRY(IEN).NE.1).AND.(IENTRY(IEN).NE.2)) CALL XABORT('DUO'
     1  //': LCM OBJECT EXPECTED AT LHS.')
        IF(JENTRY(IEN).NE.2) CALL XABORT('DUO: ENTRY IN READ-ONLY MODE'
     1  //' EXPECTED.')
        CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          CARLIR=HENTRY(IEN)
          CALL XABORT('DUO: SIGNATURE OF '//CARLIR//' IS '//HSIGN//
     1    '. L_LIBRARY EXPECTED.')
        ENDIF
      ENDDO
      IPLIB1=KENTRY(1)
      IPLIB2=KENTRY(2)
      CALL LCMGET(IPLIB1,'STATE-VECTOR',ISTATE)
      NG=ISTATE(3)
      CALL LCMGET(IPLIB2,'STATE-VECTOR',ISTATE)
      IF(ISTATE(3).NE.NG) CALL XABORT('DUO: INVALID NUMBER OF ENERGY G'
     1 //'ROUPS.')
*---
*  READ DATA
*---
      IPRINT=1
      LENER=.FALSE.
      LISOT=.FALSE.
      LMIXT=.FALSE.
      LREAC=.FALSE.
   10 CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU.EQ.10) GO TO 100
      IF(ITYPLU.NE.3) CALL XABORT('DUO: READ ERROR - CHARACTER VARI'
     > //'ABLE EXPECTED')
      IF(CARLIR.EQ.';') THEN
        GO TO 100
      ELSE IF(CARLIR.EQ.'EDIT') THEN
        CALL REDGET(ITYPLU,IPRINT,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT('DUO: READ ERROR - INTEGER VARI'
     >  //'ABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'ENERGY') THEN
        LENER=.TRUE.
      ELSE IF(CARLIR.EQ.'ISOTOPE') THEN
        LISOT=.TRUE.
      ELSE IF(CARLIR.EQ.'MIXTURE') THEN
        LMIXT=.TRUE.
      ELSE IF(CARLIR.EQ.'REAC') THEN
        LREAC=.TRUE.
        GO TO 100
      ELSE
        CALL XABORT('DUO: ILLEGAL KEYWORD '//CARLIR)
      ENDIF
      GO TO 10
  100 CALL LCMGET(IPLIB1,'STATE-VECTOR',ISTATE)
      NMIX=ISTATE(1)
      NISOT=ISTATE(2)
      NGRP=ISTATE(3)
      CALL LCMGET(IPLIB2,'STATE-VECTOR',ISTATE)
      IF(ISTATE(1).NE.NMIX) CALL XABORT('DUO: THE TWO MICROLIBS HAVE A'
     > //' DIFFERENT NUMBER OF MIXTURES.')
      IF(ISTATE(2).NE.NISOT) CALL XABORT('DUO: THE TWO MICROLIBS HAVE '
     > //'A DIFFERENT NUMBER OF ISOTOPES.')
      IF(ISTATE(3).NE.NGRP) CALL XABORT('DUO: THE TWO MICROLIBS HAVE A'
     > //' DIFFERENT NUMBER OF GROUPS.')
*
      CALL SETARA(BASE,NGRP*NGRP*NMIX,IRHS1)
      CALL SETARA(BASE,NGRP*NGRP*NMIX,ILHS1)
      CALL SETARA(BASE,NGRP*NMIX,IFLUX1)
      CALL SETARA(BASE,NGRP*NMIX,IAFLU1)
      CALL SETARA(BASE,NGRP*NGRP*NMIX,IRHS2)
      CALL SETARA(BASE,NGRP*NGRP*NMIX,ILHS2)
      CALL SETARA(BASE,NGRP*NMIX,IFLUX2)
      CALL SETARA(BASE,NGRP*NMIX,IAFLU2)
      CALL SETARA(BASE,NGRP*NGRP*(NISOT+NMIX),IRHSI1)
      CALL SETARA(BASE,NGRP*NGRP*(NISOT+NMIX),ILHSI1)
      CALL SETARA(BASE,NGRP*(NISOT+NMIX),IFLUXI1)
      CALL SETARA(BASE,NGRP*(NISOT+NMIX),IAFLUI1)
      CALL SETARA(BASE,NGRP*NGRP*(NISOT+NMIX),IRHSI2)
      CALL SETARA(BASE,NGRP*NGRP*(NISOT+NMIX),ILHSI2)
      CALL SETARA(BASE,NGRP*(NISOT+NMIX),IFLUXI2)
      CALL SETARA(BASE,NGRP*(NISOT+NMIX),IAFLUI2)
      CALL DUODRV(IPLIB1,IPLIB2,IPRINT,LENER,LISOT,LMIXT,LREAC,
     > NMIX,NISOT,NGRP,BASE(IRHS1),BASE(ILHS1),BASE(IFLUX1),
     > BASE(IAFLU1),BASE(IRHS2),BASE(ILHS2),BASE(IFLUX2),BASE(IAFLU2),
     > BASE(IRHSI1),BASE(ILHSI1),BASE(IFLUXI1),BASE(IAFLUI1),
     > BASE(IRHSI2),BASE(ILHSI2),BASE(IFLUXI2),BASE(IAFLUI2))
      CALL RLSARA(BASE(IAFLUI2))
      CALL RLSARA(BASE(IFLUXI2))
      CALL RLSARA(BASE(ILHSI2))
      CALL RLSARA(BASE(IRHSI2))
      CALL RLSARA(BASE(IAFLUI1))
      CALL RLSARA(BASE(IFLUXI1))
      CALL RLSARA(BASE(ILHSI1))
      CALL RLSARA(BASE(IRHSI1))
      CALL RLSARA(BASE(IAFLU2))
      CALL RLSARA(BASE(IFLUX2))
      CALL RLSARA(BASE(ILHS2))
      CALL RLSARA(BASE(IRHS2))
      CALL RLSARA(BASE(IAFLU1))
      CALL RLSARA(BASE(IFLUX1))
      CALL RLSARA(BASE(ILHS1))
      CALL RLSARA(BASE(IRHS1))
      RETURN
      END
