*DECK DUO003
      SUBROUTINE DUO003(IPLIB,IPRINT,NMIX,NISOT,NGRP,IDIV,ZKEFF,RHS,LHS,
     > FLUX,AFLUX,IHUSED,IMIX,DENS,VOL,TOTAL,ZNUSF,CHI,SCAT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* processing one of the two microlibs and return isotope-dependent
* RHS and LHS matrices.
*
*Copyright:
* Copyright (C) 2013 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   microlib
* IPRINT  print parameter
* NMIX    number of mixtures
* NISOT   number of isotopes
* NGRP    number of energy groups
* IDIV    type of divergence term processing (=0: no processing;
*         =1: direct processing; =2: adjoint processing;
*         =3: direct-adjoint processing)
*
*Parameters: output
* ZKEFF   effective multiplication factor
* RHS     absorption macroscopic cross-section matrix
* LHS     production macroscopic cross-section matrix
* FLUX    integrated direct flux
* AFLUX   integrated adjoint flux
*
*Parameters: scratch
* IHUSED
* IMIX
* DENS
* VOL
* TOTAL
* ZNUSF
* CHI
* SCAT
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB
      INTEGER IPRINT,NMIX,NISOT,NGRP,IDIV,IHUSED(3,NISOT),IMIX(NISOT)
      REAL ZKEFF,RHS(NGRP,NGRP,NISOT+NMIX),LHS(NGRP,NGRP,NISOT+NMIX),
     > FLUX(NGRP,NISOT+NMIX),AFLUX(NGRP,NISOT+NMIX),DENS(NISOT),
     > VOL(NISOT),TOTAL(NGRP),ZNUSF(NGRP),CHI(NGRP),SCAT(NGRP,NGRP)
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXT12*12
      DOUBLE PRECISION SUM
      COMMON BASE(1)
*----
*  COMPUTE THE RHS AND LHS MATRICES
*----
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IHUSED)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',IMIX)
      CALL LCMGET(IPLIB,'ISOTOPESDENS',DENS)
      CALL LCMGET(IPLIB,'ISOTOPESVOL',VOL)
      CALL LCMGET(IPLIB,'K-EFFECTIVE',ZKEFF)
      IF(IPRINT.GT.1) WRITE(6,'(35H DUO003: EFFECTIVE MULTIPLICATION F,
     > 6HACTOR=,1P,E12.5)') ZKEFF
      CALL XDRSET(RHS,NGRP*NGRP*(NISOT+NMIX),0.0)
      CALL XDRSET(LHS,NGRP*NGRP*(NISOT+NMIX),0.0)
      DO ISOT=1,NISOT
        WRITE(TEXT12,'(3A4)') (IHUSED(I0,ISOT),I0=1,3)
        IF(IPRINT.GT.4) WRITE(6,'(29H DUO003: PROCESSING ISOTOPE '',
     >  A12,2H''.)') TEXT12
        CALL LCMSIX(IPLIB,TEXT12,1)
        CALL LCMGET(IPLIB,'NWT0',FLUX(1,ISOT))
        CALL LCMLEN(IPLIB,'NWAT0',ILON,ITYLCM)
        IF(ILON.NE.0) THEN
          CALL LCMGET(IPLIB,'NWAT0',AFLUX(1,ISOT))
        ELSE
          CALL XDRSET(AFLUX(1,ISOT),NGRP,1.0)
        ENDIF
        DO IGR=1,NGRP
          FLUX(IGR,ISOT)=FLUX(IGR,ISOT)*VOL(ISOT)
          AFLUX(IGR,ISOT)=AFLUX(IGR,ISOT)*VOL(ISOT)
        ENDDO
        CALL LCMGET(IPLIB,'NTOT0',TOTAL)
        CALL LCMLEN(IPLIB,'NUSIGF',ILON,ITYLCM)
        IF(ILON.GT.0) THEN
          CALL LCMGET(IPLIB,'NUSIGF',ZNUSF)
          CALL LCMGET(IPLIB,'CHI',CHI)
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              LHS(JGR,IGR,ISOT)=LHS(JGR,IGR,ISOT)+DENS(ISOT)*CHI(JGR)*
     >        ZNUSF(IGR)
            ENDDO
          ENDDO
        ENDIF
        CALL SETARA(BASE,NGRP,ISIGS)
        CALL SETARA(BASE,NGRP*NGRP,IXSSCM)
        CALL SETARA(BASE,NGRP,INJJ)
        CALL SETARA(BASE,NGRP,IIJJ)
        CALL XDRLGS(IPLIB,-1,IPRINT,0,0,1,NGRP,BASE(ISIGS),SCAT,ITYPRO,
     >  BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
        CALL RLSARA(BASE(IIJJ))
        CALL RLSARA(BASE(INJJ))
        CALL RLSARA(BASE(IXSSCM))
        CALL RLSARA(BASE(ISIGS))
        DO IGR=1,NGRP
          DO JGR=1,NGRP
            RHS(JGR,IGR,ISOT)=RHS(JGR,IGR,ISOT)-DENS(ISOT)*SCAT(JGR,IGR)
          ENDDO
          RHS(IGR,IGR,ISOT)=RHS(IGR,IGR,ISOT)+DENS(ISOT)*TOTAL(IGR)
        ENDDO
        CALL LCMSIX(IPLIB,' ',2)
      ENDDO
*----
*  INTRODUCE THE DIRECT OR ADJOINT DIVERGENCE COMPONENT IN THE RHS
*  MATRIX
*----
      DO IBM=1,NMIX
        IF(IDIV.EQ.1) THEN
          DO JGR=1,NGRP
            SUM=0.0D0
            DO ISOT=1,NISOT
              IF(IMIX(ISOT).EQ.IBM) THEN
                FLUMIX=FLUX(JGR,ISOT)
                AFLUMI=AFLUX(JGR,ISOT)
                DO IGR=1,NGRP
                  SUM=SUM+(RHS(JGR,IGR,ISOT)-LHS(JGR,IGR,ISOT)/ZKEFF)*
     >            FLUX(IGR,ISOT)
                ENDDO
              ENDIF
            ENDDO
            RHS(JGR,JGR,NISOT+IBM)=-REAL(SUM)/FLUMIX
            FLUX(JGR,NISOT+IBM)=FLUMIX
            AFLUX(JGR,NISOT+IBM)=AFLUMI
          ENDDO
        ELSE IF(IDIV.EQ.2) THEN
          DO IGR=1,NGRP
            SUM=0.0D0
            DO ISOT=1,NISOT
              IF(IMIX(ISOT).EQ.IBM) THEN
                FLUMIX=FLUX(IGR,ISOT)
                AFLUMI=AFLUX(IGR,ISOT)
                DO JGR=1,NGRP
                  SUM=SUM+(RHS(JGR,IGR,ISOT)-LHS(JGR,IGR,ISOT)/ZKEFF)*
     >            AFLUX(JGR,ISOT)
                ENDDO
              ENDIF
            ENDDO
            RHS(IGR,IGR,NISOT+IBM)=-REAL(SUM)/AFLUMI
            FLUX(IGR,NISOT+IBM)=FLUMIX
            AFLUX(IGR,NISOT+IBM)=AFLUMI
          ENDDO
        ELSE IF(IDIV.EQ.3) THEN
          CALL SETARA(BASE,NGRP,IDLK)
          CALL SETARA(BASE,NGRP,IALK)
          DO JGR=1,NGRP
            SUM=0.0D0
            DO ISOT=1,NISOT
              IF(IMIX(ISOT).EQ.IBM) THEN
                FLUMIX=FLUX(JGR,ISOT)
                AFLUMI=AFLUX(JGR,ISOT)
                DO IGR=1,NGRP
                  SUM=SUM+(RHS(JGR,IGR,ISOT)-LHS(JGR,IGR,ISOT)/ZKEFF)*
     >            FLUX(IGR,ISOT)
                ENDDO
              ENDIF
            ENDDO
            BASE(IDLK+JGR-1)=REAL(SUM)
            FLUX(JGR,NISOT+IBM)=FLUMIX
          ENDDO
          DO IGR=1,NGRP
            SUM=0.0D0
            DO ISOT=1,NISOT
              IF(IMIX(ISOT).EQ.IBM) THEN
                FLUMIX=FLUX(IGR,ISOT)
                AFLUMI=AFLUX(IGR,ISOT)
                DO JGR=1,NGRP
                  SUM=SUM+(RHS(JGR,IGR,ISOT)-LHS(JGR,IGR,ISOT)/ZKEFF)*
     >            AFLUX(JGR,ISOT)
                ENDDO
              ENDIF
            ENDDO
            BASE(IALK+IGR-1)=REAL(SUM)
            AFLUX(IGR,NISOT+IBM)=AFLUMI
          ENDDO
          CALL SETARA(BASE,NGRP,IV)
          CALL SETARA(BASE,NGRP,IW)
          CALL SETARA(BASE,2*(2*NGRP+1)*(2*NGRP),IDFF)
          CALL SETARA(BASE,2*(2*NGRP)*(2*NGRP+1),IDFF2)
          CALL DUO005(NGRP,BASE(IDLK),BASE(IALK),FLUX(1,NISOT+IBM),
     >    AFLUX(1,NISOT+IBM),BASE(IV),BASE(IW),BASE(IDFF),BASE(IDFF2))
          CALL RLSARA(BASE(IDFF2))
          CALL RLSARA(BASE(IDFF))
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              RHS(IGR,JGR,NISOT+IBM)=RHS(IGR,JGR,NISOT+IBM)-
     >        BASE(IV+IGR-1)-BASE(IW+JGR-1)
            ENDDO
          ENDDO
          CALL RLSARA(BASE(IW))
          CALL RLSARA(BASE(IV))
          CALL RLSARA(BASE(IALK))
          CALL RLSARA(BASE(IDLK))
        ENDIF
      ENDDO
      RETURN
      END
