*DECK EDI
      SUBROUTINE EDI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* edition operator for Dragon.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and G. Marleau
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file (order is arbitrary for
*         objects 2,3,4):
*         HENTRY(1): create or modification type(L_EDIT);
*         HENTRY(2): read-only type(L_FLUX);
*         HENTRY(3): read-only type(L_MACROLIB OR L_LIBRARY);
*         HENTRY(4): read-only type(L_TRACK);
*         The following three LCM objects are required with a SPH
*         calculation (order is arbitrary for objects 5,6,7). The
*         object 5 is required if the "MERG CELL" option is used.
*         HENTRY(5): optional read-only type(L_GEOM) containing the
*         original geometry;
*         HENTRY(6): optional read-only type(L_GEOM) containing the SPH
*         macrogeometry;
*         HENTRY(7): optional read-only type(L_SYS) containing the
*         L_PIJ object of the original geometry in cases where a
*         Selengut normalization is required.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXED=100,NSTATE=40,IOUT=6)
      CHARACTER*12 TEXT12,CDOOR,OLDGEO,MACGEO,CARISO(MAXED),CURNAM,
     > OLDNAM,HSIGN
      CHARACTER TITLE*72,HSMG*131
      INTEGER IGP(NSTATE),KCHAR(18),IDATA(NSTATE),ISTATE(NSTATE),
     > IBASE(1)
      LOGICAL LNEWGE,LISO
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
* PARAMETER VALIDATION.
*----
      IF(NENTRY.LT.2) CALL XABORT('EDI: MORE RHS LCM OBJECTS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('EDI: LC'
     > //'M OBJECT EXPECTED AT LHS.')

      IPEDIT=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_EDIT'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_EDIT') THEN
          TEXT12=HENTRY(1)
          CALL XABORT('EDI: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >    '. L_EDIT EXPECTED.')
        ENDIF
      ELSE
        CALL XABORT('EDI: EDITING LCM OBJECT IN CREATE OR MODIFY MODE '
     >  //'EXPECTED.')
      ENDIF
*----
*  SCAN READ-ONLY MODE DATA STRUCTURE ENTRY(2) TO ENTRY(4)
*  FOR FLUX, TRACK AND LIB
*----
      IF(JENTRY(2).NE.2) CALL XABORT('EDI: LCM OBJECT IN READ-ONLY MOD'
     > //'E EXPECTED AT RHS.')
      IPFLUX=0
      IKFLUX=0
      IPTRK1=0
      IKTRK1=0
      IPLIB=0
      IKLIB=0
      DO 10 IEN=2,MIN(4,NENTRY)
        IF((IENTRY(IEN).EQ.1).OR.(IENTRY(IEN).EQ.2)) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF((HSIGN.EQ.'L_FLUX').AND.(IKFLUX.EQ.0)) THEN
            IPFLUX=KENTRY(IEN)
            IKFLUX=IEN
          ELSE IF((HSIGN.EQ.'L_TRACK').AND.(IKTRK1.EQ.0)) THEN
            IPTRK1=KENTRY(IEN)
            IKTRK1=IEN
          ELSE IF((HSIGN.EQ.'L_LIBRARY').AND.(IKLIB.EQ.0)) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ELSE IF((HSIGN.EQ.'L_MACROLIB').AND.(IKLIB.EQ.0)) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=-IEN
          ENDIF
        ENDIF
   10 CONTINUE
*----
*  READ MACROLIB INFORMATION
*----
      IF(IKLIB.EQ.0) CALL XABORT('EDI: NO MACROLIB OR MICROLIB LCM OBJ'
     > //'ECT FOUND.')
      IF(IKLIB.GT.0) THEN
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        MAXISM=ISTATE(22)
        CALL LCMSIX(IPLIB,'MACROLIB',1)
      ELSE
        MAXISM=1
      ENDIF
      CALL LCMGET(IPLIB,'STATE-VECTOR',IDATA)
      NGRP=IDATA(1)
      NBMIX=IDATA(2)
      NL=IDATA(3)
      NIFISS=IDATA(4)
      NEDMAC=IDATA(5)
      ITRANC=IDATA(6)
      NDEL=IDATA(7)
*----
*  BUILD L_TRACK AND L_FLUX OBJECTS FROM EXTENDED MACROLIB
*----
      IF((IKTRK1.EQ.0).AND.(IKFLUX.EQ.0)) THEN
         CALL LCMOP(IPTRK1,'PSEUDO_TRACK',0,1,0)
         CALL LCMOP(IPFLUX,'PSEUDO_FLUX',0,1,0)
         HSIGN='L_TRACK'
         CALL LCMPTC(IPTRK1,'SIGNATURE',12,1,HSIGN)
         HSIGN='L_FLUX'
         CALL LCMPTC(IPFLUX,'SIGNATURE',12,1,HSIGN)
         TEXT12='DUMMY'
         CALL LCMPTC(IPTRK1,'TRACK-TYPE',12,1,TEXT12)
         CALL SETARA(IBASE,NBMIX,IKEYFL)
         DO 20 IBM=1,NBMIX
   20    IBASE(IKEYFL+IBM-1)=IBM
         CALL LCMPUT(IPTRK1,'MATCOD',NBMIX,1,IBASE(IKEYFL))
         CALL LCMPUT(IPTRK1,'KEYFLX',NBMIX,1,IBASE(IKEYFL))
         CALL RLSARA(IBASE(IKEYFL))
         CALL SETARA(BASE,NBMIX,IVOL)
         CALL LCMLEN(IPLIB,'VOLUME',ILONG,ITYLCM)
         IF(ILONG.EQ.0) CALL XABORT('EDI: NO VOLUME IN MACROLIB.')
         CALL LCMGET(IPLIB,'VOLUME',BASE(IVOL))
         CALL LCMPUT(IPTRK1,'VOLUME',NBMIX,2,BASE(IVOL))
         CALL XDISET(ISTATE,NSTATE,0)
         ISTATE(1)=NBMIX
         ISTATE(2)=NBMIX
         ISTATE(4)=NBMIX
         CALL LCMPUT(IPTRK1,'STATE-VECTOR',NSTATE,1,ISTATE)
         CALL SETARA(BASE,NBMIX,IFLINT)
         JPMAC=LCMGID(IPLIB,'GROUP')
         JPFLUX=LCMLID(IPFLUX,'FLUX',NGRP)
         DO 40 IGR=1,NGRP
            KPMAC=LCMGIL(JPMAC,IGR)
            CALL LCMLEN(KPMAC,'FLUX-INTG',ILONG,ITYLCM)
            IF(ILONG.EQ.0) CALL XABORT('EDI: NO FLUX-INTG IN MACROLIB.')
            CALL LCMGET(KPMAC,'FLUX-INTG',BASE(IFLINT))
            DO 30 IBM=1,NBMIX
   30       BASE(IFLINT+IBM-1)=BASE(IFLINT+IBM-1)/BASE(IVOL+IBM-1)
            CALL LCMPDL(JPFLUX,IGR,NBMIX,2,BASE(IFLINT))
   40    CONTINUE
         CALL RLSARA(BASE(IFLINT))
         CALL RLSARA(BASE(IVOL))
         CALL LCMLEN(IPLIB,'K-EFFECTIVE',ILONG,ITYLCM)
         IF(ILONG.EQ.1) THEN
            CALL LCMGET(IPLIB,'K-EFFECTIVE',FLOAT)
            CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,FLOAT)
         ENDIF
         CALL LCMLEN(IPLIB,'K-INFINITY',ILONG,ITYLCM)
         IF(ILONG.EQ.1) THEN
            CALL LCMGET(IPLIB,'K-INFINITY',FLOAT)
            CALL LCMPUT(IPFLUX,'K-INFINITY',1,2,FLOAT)
         ENDIF
         CALL XDISET(ISTATE,NSTATE,0)
         ISTATE(1)=NGRP
         ISTATE(2)=NBMIX
         CALL LCMPUT(IPFLUX,'STATE-VECTOR',NSTATE,1,ISTATE)
      ENDIF
      IF(IKLIB.GT.0) CALL LCMSIX(IPLIB,' ',2)
      IF(IPFLUX.EQ.0) CALL XABORT('EDI: NO REFERENCE FLUX AVAILABLE.')
      CALL LCMGET(IPFLUX,'STATE-VECTOR',ISTATE)
      ILEAKC=ISTATE(7)
*----
*  READ GEOMETRIES AND SYSTEM (SPH ONLY)
*----
      IPGEO1=0
      IKGEO1=0
      IPGEO2=0
      IKGEO2=0
      IPSYS=0
      IKSYS=0
      OLDGEO=' '
      IF(NENTRY.GT.4) THEN
        DO 70 IEN=5,NENTRY
          IF((IENTRY(IEN).EQ.1).OR.(IENTRY(IEN).EQ.2)) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(HSIGN.EQ.'L_GEOM') THEN
              IF(IKGEO1.EQ.0) THEN
                IPGEO1=KENTRY(IEN)
                OLDGEO=HENTRY(IEN)
                IKGEO1=IEN
              ELSE IF(IKGEO2.EQ.0) THEN
                IPGEO2=KENTRY(IEN)
                IKGEO2=IEN
              ENDIF
            ELSE IF((HSIGN.EQ.'L_PIJ').AND.(IKSYS.EQ.0)) THEN
              IPSYS=KENTRY(IEN)
              IKSYS=IEN
            ENDIF
          ELSE
            CALL XABORT('EDI: INVALID TYPE AT RHS.')
          ENDIF
   70   CONTINUE
      ENDIF
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
      IF(IPTRK1.EQ.0) CALL XABORT('EDI: NO REFERENCE TRACKING AVAILABL'
     > //'E.')
      CALL LCMGET(IPTRK1,'STATE-VECTOR',IGP)
      NREG=IGP(1)
      CALL LCMGET(IPTRK1,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
      IF(CDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK1,'KEYFLX',LKFL,ITYLCM)
         NFUNL=LKFL/NREG
      ELSE
         NFUNL=1
      ENDIF
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG*NFUNL,IIDL)
      CALL LCMGET(IPTRK1,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK1,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK1,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK1,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK1,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
         CALL LCMPUT(IPEDIT,'TITLE',18,3,KCHAR)
      ELSE
         TITLE='*** NO TITLE PROVIDED FOR THE REFERENCE CASE ***'
      ENDIF
*----
*  READ GROUP STRUCTURE
*----
      CALL SETARA(BASE,2*NGRP+1,IENERG)
      CALL SETARA(BASE,NGRP,IENERV)
      CALL LCMLEN(IPLIB,'ENERGY',NTENER,ITYLCM)
      IF(NTENER.EQ.NGRP+1) THEN
        CALL LCMGET(IPLIB,'ENERGY',BASE(IENERG))
      ELSE IF(NTENER.NE.0) THEN
        CALL XABORT('EDI: INVALID NUMBER OF GROUP ON MACROLIB.')
      ENDIF
*----
*  READ EDITION OPTIONS PARAMETERS
*----
      CALL SETARA(IBASE,NGRP,IGCOND)
      CALL SETARA(IBASE,NREG,IMERGE)
      CALL SETARA(IBASE,NBMIX,IACTI)
      CALL SETARA(IBASE,NBMIX+1,IMIXME)
      ICALL=0
      CURNAM=' '
      IF(JENTRY(1).EQ.0) THEN
         HSIGN='L_EDIT'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
         OLDNAM=' '
         MACGEO=' '
         NMERGE=NREG
         NGCOND=NGRP
         IHF=1
         IFFAC=0
         ILUPS=0
         NACTI=0
         NSTATS=0
         IADF=0
         NBMICR=0
         IPRINT=1
         NSAVES=0
         NWGTH=0
         IF(ILEAKC.GE.5) NWGTH=1
         MAXISK=0
         MAXCND=0
         MAXMRG=0
         IXEDI=0
         IADJ=0
         IEUR=0
         MAXPTS=NREG
         DO 90 IGROUP=1,NGRP
   90    IBASE(IGCOND+IGROUP-1)=IGROUP
         DO 100 IREGIO=1,NREG
  100    IBASE(IMERGE+IREGIO-1)=IREGIO
      ELSE IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_EDIT') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('EDI: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >      '. L_EDIT EXPECTED.')
         ENDIF
         CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
         NMERGE=ISTATE(1)
         NGCOND=ISTATE(2)
         IFFAC=ISTATE(3)
         ILUPS=ISTATE(4)
         NACTI=ISTATE(5)
         NSTATS=ISTATE(6)
         IADF=ISTATE(7)
         IEUR=ISTATE(8)
         NBMICR=ISTATE(9)
         IPRINT=ISTATE(10)
         NSAVES=ISTATE(11)
         NWGTH=ISTATE(12)
         MAXISK=ISTATE(13)
         MAXCND=ISTATE(14)
         MAXMRG=ISTATE(15)
         IXEDI=ISTATE(16)
         MAXPTS=ISTATE(17)
         IHF=ISTATE(18)
         IF(ISTATE(19).NE.NDEL) CALL XABORT('EDI: BAD VALUE OF NDEL')
         IADJ=ISTATE(21)
         CALL LCMLEN(IPEDIT,'LAST-EDIT',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPEDIT,'LAST-EDIT',KCHAR)
            WRITE(OLDNAM,'(3A4)') (KCHAR(I1),I1=1,3)
            INTLIR=0
            READ(OLDNAM,'(8X,I4)',ERR=105) INTLIR
  105       ICALL=MAX(ICALL,INTLIR)
         ENDIF
         CALL LCMLEN(IPEDIT,'REF:IMERGE',LENGT,ITYLCM)
         IF(LENGT.EQ.NREG) THEN
            CALL LCMGET(IPEDIT,'REF:IMERGE',IBASE(IMERGE))
         ELSE
            DO 106 IREGIO=1,NREG
  106       IBASE(IMERGE+IREGIO-1)=IREGIO
         ENDIF
         CALL LCMLEN(IPEDIT,'REF:IGCOND',LENGT,ITYLCM)
         IF(LENGT.EQ.NGCOND) THEN
            CALL LCMGET(IPEDIT,'REF:IGCOND',IBASE(IGCOND))
         ELSE
            DO 107 IGROUP=1,NGRP
  107       IBASE(IGCOND+IGROUP-1)=IGROUP
         ENDIF
         CALL LCMLEN(IPEDIT,'LINK.MACGEOM',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPEDIT,'LINK.MACGEOM',KCHAR)
            WRITE(MACGEO,'(3A4)') (KCHAR(I),I=1,3)
         ELSE
            MACGEO=' '
         ENDIF
         IF(NBMICR.GT.0) THEN
            CALL SETARA(BASE,3*NBMICR,IJCEDM)
            CALL LCMGET(IPEDIT,'CARISO',BASE(IJCEDM))
            IDEB=IJCEDM
            DO 110 J=1,NBMICR
              WRITE(CARISO(J),'(3A4)') (BASE(IDEB+I1),I1=0,2)
              IDEB=IDEB+3
  110       CONTINUE
            CALL RLSARA(BASE(IJCEDM))
         ENDIF
         IF(NACTI.GT.0) CALL LCMGET(IPEDIT,'IACTI',IBASE(IACTI))
      ENDIF
      NGCR=0
      LISO=.FALSE.
      MAXISK=MAX(MAXISK,MAXISM)
      CALL SETARA(IBASE,NGRP+1,IGCR)
      CALL SETARA(BASE,NGRP+1,IECR)
      CALL XDISET(IBASE(IGCR),NGRP+1,NGRP)
      CALL XDRSET(BASE(IECR),NGRP+1,0.0)
      CALL EDIGET(IPEDIT,NGRP,NGCR,NREG,NBMIX,IBASE(IMAT),NMERGE,IHF,
     1 IFFAC,ILUPS,NSAVES,NSTATS,IBASE(IGCR),BASE(IECR),IBASE(IMERGE),
     2 CURNAM,OLDNAM,IADF,NWGTH,NBMICR,CARISO,NACTI,IBASE(IACTI),IPRINT,
     3 MAXPTS,ICALL,ISOTXS,LISO,IADJ,MACGEO,IEUR,IBASE(IMIXME))
      CALL RLSARA(IBASE(IMIXME))
*----
*  CALL EDIMRG TO FIND MERGE INDEX ASSOCIATED WITH THE SECOND GEOMETRY
*  OR TRACK FILE (EQUIGEOM CAPABILITIES)
*----
      TEXT12=' '
      LNEWGE=.FALSE.
      IF(NMERGE.EQ.-1) THEN
        IF(IKGEO2.GT.0) THEN
          ITM=-1
          IPMRG=IPGEO2
          TEXT12=HENTRY(IKGEO2)
        ELSE IF(IKGEO1.GT.0) THEN
          ITM=-1
          IPMRG=IPGEO1
          TEXT12=HENTRY(IKGEO1)
        ELSE
          ITM=0
          IPMRG=IPTRK1
        ENDIF
        CALL EDIMRG(IPTRK1,IPMRG,IPRINT,TEXT12,ITM,NREG,NMERGE,
     >              IBASE(IMERGE))
*----
*  BUILD A MACRO-GEOMETRY FROM REFERENCE GEOMETRY OLDGEO (CELL OPTION)
*----
      ELSE IF(NMERGE.EQ.-2) THEN
        IF(((CDOOR.EQ.'EXCELL').OR.(CDOOR.EQ.'MCCG')).AND.
     >  (IGP(7).EQ.4)) THEN
          CALL EDIMRC(IPTRK1,IPRINT,NREG,NMERGE,IBASE(IMERGE))
        ELSE
          IF(IPGEO1.EQ.0) CALL XABORT('EDI: NO REFERENCE GEOMETRY AVAI'
     >    //'LABLE.')
          IF(IPGEO2.NE.0) CALL XABORT('EDI: INPUT MACRO-GEOMETRY NOT E'
     >    //'XPECTED WITH CELL OPTION.')
          IF(IPRINT.GT.0) WRITE(IOUT,190) OLDGEO,CDOOR
          CALL LCMGET(IPGEO1,'STATE-VECTOR',ISTATE)
          MAXGEO=MAX(MAXPTS,ISTATE(6))
          LNEWGE=.TRUE.
          MACGEO='MACRO$GEO'
          CALL LCMOP(IPGEO2,'MACRO$GEO',0,1,9)
          MAXMER=MIN(NREG,MAXGEO)
          CALL SETARA(BASE,MAXGEO,IXX)
          CALL SETARA(BASE,MAXGEO,IYY)
          CALL SETARA(BASE,2*MAXGEO,IINUM)
          CALL SETARA(BASE,MAXGEO,IIGEN)
          CALL SETARA(BASE,2*MAXGEO,IIORI)
          CALL SETARA(BASE,2*MAXGEO,IIMAT)
          CALL SETARA(BASE,MAXGEO+1,INMC)
          CALL SETARA(BASE,NREG,IIBI)
          CALL EDIGEO(MAXGEO,MAXMER,IPGEO1,IPGEO2,IPRINT,NREG,IEUR,
     1    NMERGE,IBASE(IMERGE),BASE(IXX),BASE(IYY),BASE(IINUM),
     2    BASE(IIGEN),BASE(IIORI),BASE(IIMAT),BASE(INMC),BASE(IIBI))
          CALL RLSARA(BASE(IIBI))
          CALL RLSARA(BASE(INMC))
          CALL RLSARA(BASE(IIMAT))
          CALL RLSARA(BASE(IIORI))
          CALL RLSARA(BASE(IIGEN))
          CALL RLSARA(BASE(IINUM))
          CALL RLSARA(BASE(IYY))
          CALL RLSARA(BASE(IXX))
*
*         COPY THE MACRO-GEOMETRY INTO THE EDITION OBJECT.
          CALL LCMSIX(IPEDIT,'MACRO-GEOM',1)
          CALL LCMEQU(IPGEO2,IPEDIT)
          CALL LCMSIX(IPEDIT,' ',2)
        ENDIF
      ELSE IF(NMERGE.EQ.-3) THEN
*----
*  CALL EDIMRG TO FIND MERGE INDEX ASSOCIATED WITH HMIX
*----
        IPMRG=IPTRK1
        CALL EDIHMX(IPTRK1 ,NREG  ,NMERGE,IBASE(IMERGE))
      ENDIF
*----
*  TEST ENERGY CONDENSATION INPUT
*----
      CALL EDIENE(NGRP,NGCR,NGCOND,NTENER,IBASE(IGCR),BASE(IECR),
     > IBASE(IGCOND),BASE(IENERG),BASE(IENERV))
*
      CALL LCMLEN(IPEDIT,'MACRO-GEOM',ILONG,ITYLCM)
      LGEO=0
      IF((ILONG.NE.0).OR.(MACGEO.NE.' ')) LGEO=1
      IF(IPRINT.GT.0) THEN
        WRITE(IOUT,200) NMERGE,NGCOND,IFFAC,ILUPS,NACTI,NSTATS,IADF,
     >  IEUR,NBMICR,IPRINT
        WRITE(IOUT,210) NSAVES,NWGTH,MAXPTS,IHF,NDEL,LGEO,IADJ
        WRITE(IOUT,'(//15H MERGING INDEX:/(1X,14I5))')
     >    (IBASE(IMERGE+I-1),I=1,NREG)
        IF(CURNAM.NE.' ') WRITE(IOUT,'(/27H EDI: SAVE MICROLIB INFO ON,
     >  12H DIRECTORY '',A12,2H''.)') CURNAM
      ENDIF
      CALL RLSARA(BASE(IECR))
      CALL RLSARA(IBASE(IGCR))
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMERGE
      ISTATE(2)=NGCOND
      ISTATE(3)=IFFAC
      ISTATE(4)=ILUPS
      ISTATE(5)=NACTI
      ISTATE(6)=NSTATS
      ISTATE(7)=IADF
      ISTATE(8)=IEUR
      ISTATE(9)=NBMICR
      ISTATE(10)=IPRINT
      ISTATE(11)=NSAVES
      ISTATE(12)=NWGTH
      ISTATE(13)=MAXISK
      ISTATE(14)=MAX(NGCOND,MAXCND)
      ISTATE(15)=MAX(NMERGE,MAXMRG)
      ISTATE(16)=IXEDI+ISOTXS*NMERGE
      ISTATE(17)=MAXPTS
      ISTATE(18)=IHF
      ISTATE(19)=NDEL
      ISTATE(20)=LGEO
      ISTATE(21)=IADJ
      CALL LCMPUT(IPEDIT,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(OLDGEO.NE.' ') THEN
         READ(OLDGEO,'(3A4)') (KCHAR(I),I=1,3)
         CALL LCMPUT(IPEDIT,'LINK.GEOM',3,3,KCHAR)
      ENDIF
      IF(NSAVES.GE.2) THEN
         READ(CURNAM,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPEDIT,'LAST-EDIT',3,3,KCHAR)
      ENDIF
      IF(NBMICR.GT.0) THEN
         CALL SETARA(BASE,3*NBMICR,IJCEDM)
         IDEB=IJCEDM
         DO 120 J=1,NBMICR
           READ(CARISO(J),'(3A4)') (BASE(IDEB+I1),I1=0,2)
           IDEB=IDEB+3
  120    CONTINUE
         CALL LCMPUT(IPEDIT,'CARISO',3*NBMICR,3,BASE(IJCEDM))
         CALL RLSARA(BASE(IJCEDM))
      ENDIF
      IF(NACTI.GT.0) CALL LCMPUT(IPEDIT,'IACTI',NACTI,1,IBASE(IACTI))
      IF(MACGEO.NE.' ') THEN
         IF(HENTRY(IKGEO1).EQ.MACGEO) THEN
            IPGEO2=IPGEO1
            IKGEO2=IKGEO1
         ENDIF
         IF(IPGEO2.EQ.0) CALL XABORT('EDI: MISSING LCM OBJECT FOR THE '
     >   //'MACRO-GEOMETRY.')
         IF(IKGEO2.NE.0) THEN
           IF(HENTRY(IKGEO2).NE.MACGEO) THEN
             WRITE(HSMG,'(37HEDI: WRONG SPH MACRO-GEOMETRY NAMED '',
     >       A12,17H'' FOUND ON RHS. '',A12,11H'' EXPECTED.)')
     >       HENTRY(IKGEO2),MACGEO
             CALL XABORT(HSMG)
           ENDIF
         ENDIF
         CALL LCMGET(IPGEO2,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_GEOM') THEN
            CALL XABORT('EDI: SIGNATURE OF '//MACGEO//' IS '//HSIGN//
     >      '. L_GEOM EXPECTED.')
         ENDIF
*
*        COPY THE MACRO-GEOMETRY INTO THE EDITION OBJECT.
         CALL LCMSIX(IPEDIT,'MACRO-GEOM',1)
         CALL LCMEQU(IPGEO2,IPEDIT)
         CALL LCMSIX(IPEDIT,' ',2)
      ENDIF
*----
*  EDITION
*----
      IF(NREG.EQ.0) CALL XABORT('EDI: NREG = 0.')
      IF(NGRP.EQ.0) CALL XABORT('EDI: NGRP = 0.')
      CALL SETARA(BASE,NREG*NGRP*(NWGTH+1),JFLUXE)
      CALL SETARA(BASE,NREG*NGRP*(NWGTH+1),JAFLUX)
      CALL EDIDRV(IPEDIT,IPFLUX,IPLIB,IPSYS,NGRP,NBMIX,NREG,IBASE(IMAT),
     1 BASE(IVOL),IBASE(IIDL),NIFISS,NEDMAC,NL,NDEL,ITRANC,NGCOND,
     2 NMERGE,IADF,NWGTH,IHF,IFFAC,ILUPS,NSAVES,NSTATS,IXEDI,ISOTXS,
     3 IBASE(IGCOND),IBASE(IMERGE),CURNAM,OLDNAM,NBMICR,CARISO,NACTI,
     4 IBASE(IACTI),IPRINT,LISO,IADJ,BASE(JFLUXE),BASE(JAFLUX))
      CALL RLSARA(BASE(JAFLUX))
      CALL RLSARA(BASE(JFLUXE))
*----
*  DESTROY THE TEMPORARY MACRO-GEOMETRY
*----
      IF(LNEWGE) THEN
         CALL LCMCL(IPGEO2,1)
         TEXT12='MACRO$GEO'
         CALL LCMOP(IPGEO2,TEXT12,1,1,0)
         CALL LCMCL(IPGEO2,2)
      ENDIF
*----
*  COMPLETE THE EDITION LCM OBJECT
*----
      CALL LCMPUT(IPEDIT,'REF:IMERGE',NREG,1,IBASE(IMERGE))
      CALL LCMPUT(IPEDIT,'REF:MATCOD',NREG,1,IBASE(IMAT))
      CALL LCMPUT(IPEDIT,'REF:VOLUME',NREG,2,BASE(IVOL))
      CALL LCMPUT(IPEDIT,'REF:IGCOND',NGCOND,1,IBASE(IGCOND))
*
      CALL RLSARA(IBASE(IACTI))
      CALL RLSARA(IBASE(IMERGE))
      CALL RLSARA(IBASE(IGCOND))
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      CALL RLSARA(BASE(IENERV))
      CALL RLSARA(BASE(IENERG))
      IF(IPRINT.GT.2) CALL LCMLIB(IPEDIT)
*----
*  RELEASE TEMPORARY L_TRACK AND L_FLUX OBJECTS
*----
      IF((IKTRK1.EQ.0).AND.(IKFLUX.EQ.0)) THEN
         CALL LCMCL(IPFLUX,2)
         CALL LCMCL(IPTRK1,2)
      ENDIF
      RETURN
*
  190 FORMAT (/16H EDI: GEOMETRY ',A12,28H' WAS PREVIOUSLY TRACKED BY ,
     > 7HMODULE ,A12,1H.)
  200 FORMAT(/24H EDITION-RELATED OPTIONS/1X,23(1H-)/
     1  7H NMERGE,I8,29H   (NUMBER OF MERGED REGIONS)/
     2  7H NGCOND,I8,38H   (NUMBER OF CONDENSED ENERGY GROUPS)/
     3  7H IFFAC ,I8,40H   (=1: 4 FACTORS CALCULATION REQUESTED)/
     4  7H ILUPS ,I8,43H   (=1: REMOVE UP-SCATTERING CONTRIBUTIONS)/
     5  7H NACTI ,I8,45H   (NUMBER OF MIXTURES WITH ACTIVATION EDITS)/
     6  7H NSTATS,I8,35H   (TYPE OF STATISTIC CALCULATIONS)/
     7  7H IADF  ,I8,47H   (=0: DO NOT COMPUTE ADF INFO; =1: USE ALBS I,
     8  48HNFO; =2: USE BOUNDARY REGION AVERAGED FLUX INFO)/
     9  7H IEUR  ,I8,47H   (=1/2/3: SYBIL OR EXCELL MACRO-TRACKING/NXT ,
     1  20HMACRO-TRACKING/ELSE)/
     2  7H NBMICR,I8,47H   (=-1: PROCESS ALL ISOTOPES; >1 NUMBER OF ISO,
     3  17HTOPES TO PROCESS)/
     4  7H IPRINT,I8,16H   (PRINT LEVEL))
  210 FORMAT(
     1  7H NSAVES,I8,47H   (=0: NO COMPUTE/NO SAVE; =1: COMPUTE/NO SAVE,
     2  19H; =2: COMPUTE/SAVE)/
     3  7H NWGTH ,I8,47H   (=0: FLUX WEIGHTING FOR P1 INFO; =1: CURRENT,
     4  23H WEIGHTING FOR P1 INFO)/
     5  7H MAXPTS,I8,47H   (ALLOCATED STORAGE LENGTH FOR REGION-DEPENDE,
     6  10HNT ARRAYS)/
     7  7H IHF   ,I8,39H   (=1: H-FACTOR CALCULATION REQUESTED)/
     8  7H NDEL  ,I8,39H   (NUMBER OF DELAYED PRECURSOR GROUPS)/
     9  7H LGEO  ,I8,47H   (=0: MACRO-GEOMETRY NOT AVAILABLE; =1: IS AV,
     1  8HAILABLE)/
     2  7H IADJ  ,I8,47H   (=0: DIRECT FLUX; =1: DIRECT-ADJOINT WEIGHTI,
     3  3HNG))
      END
