*DECK EVO
      SUBROUTINE EVO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for an isotopic depletion calculation.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): modification type(L_LIBRARY);
*         HENTRY(3): read-only type(L_TRACK);
*         HENTRY(4): optional read-only type(L_FLUX)
*         or
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): creation type(L_LIBRARY);
*         HENTRY(3): read-only type(L_LIBRARY);
*         HENTRY(4): read-only type(L_TRACK);
*         HENTRY(5): optional read-only type(L_FLUX).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER HSMG*131,TEXT12*12,CDOOR*12,HSIGN*12
      INTEGER IPAR(NSTATE),KCHAR(3),IGP(NSTATE),IBASE(1)
      INTEGER          ITYPE,ITIXS,IFLMAC,IEXTR,IGLOB,ISAT,
     >                 IDIRAC,ISAVE,ISET,INR
      REAL             RPAR(5),XT(5)
      LOGICAL LOG,LMACRO
      COMMON RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.1) CALL XABORT('EVO: TWO PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT FIRST LHS.')
      IF(JENTRY(1).EQ.2) CALL XABORT('EVO: BURNUP HISTORY STORAGE IN CR'
     1 //'EATION OR MODIFICATION MODE EXPECTED.')
      IPDEPL=KENTRY(1)
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT SECOND LHS.')
      IF(JENTRY(2).EQ.0) THEN
*        INTERNAL LIBRARY CREATION. COPY THE FIRST RHS ON THIS LHS.
         IF(NENTRY.LE.3) CALL XABORT('EVO: FOUR PARAMETERS EXPECTED.')
         IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.
     1   (IENTRY(3).NE.2)))  CALL XABORT('EVO: LCM OBJECT IN READ-ONLY '
     2   //'MODE EXPECTED AT FIRST RHS.')
         CALL LCMEQU(KENTRY(3),KENTRY(2))
         IENT=4
      ELSE IF(JENTRY(2).EQ.1) THEN
*        INTERNAL LIBRARY MODIFICATION.
         IENT=3
      ELSE
         CALL XABORT('EVO: INTERNAL LIBRARY IN CREATE OR MODIFICATION M'
     1   //'ODE EXPECTED.')
      ENDIF
      IPLIB=KENTRY(2)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPDEPL,'LINK.LIB',3,3,KCHAR)
*----
*  RECOVER IPTRK AND IPFLUX POINTERS
*----
      IPFLUX=0
      IPTRK=0
      IPPOW=0
      DO 10 I=IENT,NENTRY
      IF((JENTRY(I).EQ.2).AND.((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)))
     1 THEN
         CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.EQ.'L_FLUX') THEN
            IPFLUX=KENTRY(I)
         ELSE IF(HSIGN.EQ.'L_TRACK') THEN
            IPTRK=KENTRY(I)
         ELSE IF(HSIGN.EQ.'L_POWER') THEN
            IPPOW=KENTRY(I)
         ELSE
            WRITE(HSMG,'(35HEVO: UNEXPECTED SIGNATURE AT RHS ('',A12,
     1      3H'').)') HSIGN
            CALL XABORT(HSMG)
         ENDIF
      ELSE
         CALL XABORT('EVO: LCM OBJECT IN READ-ONLY MODE EXPECTED AT '
     1   //'RHS.')
      ENDIF
   10 CONTINUE
      IF((IPFLUX.NE.0).AND.(IPTRK.EQ.0)) CALL XABORT('EVO: UNABLE TO F'
     1 //'IND A POINTER TO A L_TRACK OBJECT.')
*----
*  RECOVER GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_LIBRARY') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('EVO: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_LIBRARY EXPECTED.')
      ENDIF
      CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
      NBMIX=IPAR(1)
      NBISO=IPAR(2)
      NGRO=IPAR(3)
      NDEPL=IPAR(11)
      NCOMB=IPAR(12)
      LMACRO=(IPAR(17).GE.0).AND.(IPAR(18).EQ.1)
      CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
         CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
         IF(IPAR(1).NE.NDEPL) CALL XABORT('EVO: INVALID NUMBER OF D'
     1   //'EPLETING ISOTOPES.')
         NSUPS=IPAR(7)
         NREAC=IPAR(8)
      CALL LCMSIX(IPLIB,' ',2)
      CALL SETARA(IBASE,NBISO,JJMIX)
      CALL SETARA(RBASE,NBISO,JJDEN)
      CALL SETARA(RBASE,NBISO,JIEVOL)
      CALL SETARA(IBASE,NBMIX,MIXBRN)
      CALL SETARA(IBASE,NBMIX,MIXPWR)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',IBASE(JJMIX))
      CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(JJDEN))
      CALL LCMGET(IPLIB,'ISOTOPESTODO',IBASE(JIEVOL))
*----
*  RECOVER LOCAL ISOTOPES NAMES
*----
      CALL SETARA(IBASE,3*NBISO,IISONA)
      CALL SETARA(IBASE,3*NBISO,IISONR)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(IISONA))
      CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(IISONR))
*----
*  READ THE INPUT DATA
*----
*     DEFAULT OPTIONS:
      IMPX=1
      IDEPL=0
      INR=-1
      XT(4)=0.0
      IF(JENTRY(1).EQ.0) THEN
         INDREC=1
         RPAR(1)=1.0E-5
         RPAR(2)=1.0E-4
         RPAR(3)=80.0
         RPAR(4)=1.0E-4
         RPAR(5)=0.0
         ITYPE=2
         IEXTR=1
         IGLOB=0
         ISAT=0
         IDIRAC=0
         ITIXS=0
         IFLMAC=0
         HSIGN='L_BURNUP'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPDEPL,'SIGNATURE',3,3,KCHAR)
         CALL LCMPUT(IPDEPL,'ISOTOPESMIX',NBISO,1,IBASE(JJMIX))
         CALL LCMPUT(IPDEPL,'ISOTOPESUSED',3*NBISO,3,IBASE(IISONA))
         CALL XDISET(IBASE(MIXBRN),NBMIX,1)
         CALL XDISET(IBASE(MIXPWR),NBMIX,1)
         CALL LCMPUT(IPDEPL,'MIXTURESBurn',NBMIX,1,IBASE(MIXBRN))
         CALL LCMPUT(IPDEPL,'MIXTURESPowr',NBMIX,1,IBASE(MIXPWR))
      ELSE IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(IPDEPL,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_BURNUP') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('LIB: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_BURNUP EXPECTED.')
         ENDIF
         INDREC=2
         CALL LCMGET(IPDEPL,'STATE-VECTOR',IPAR)
         ITYPE=IPAR(1)
         NTIME=IPAR(3)
         IEXTR=IPAR(9)
         IGLOB=IPAR(10)
         ISAT=IPAR(11)
         IDIRAC=IPAR(12)
         ITIXS=IPAR(13)
         IFLMAC=IPAR(14)
         IF(IPAR(4).NE.NBISO) CALL XABORT('EVO: INVALID NUMBER OF ISOTO'
     1   //'PES.')
         CALL LCMGET(IPDEPL,'EVOLUTION-R',RPAR)
         IF(NTIME.GT.0) THEN
            CALL SETARA(RBASE,NTIME+2,ITIMES)
            CALL LCMGET(IPDEPL,'DEPL-TIMES',RBASE(ITIMES))
            XT(4)=RBASE(ITIMES+NTIME-1)
            CALL RLSARA(RBASE(ITIMES))
         ENDIF
         CALL LCMLEN(IPDEPL,'MIXTURESBurn',ILONG,ITYLCM)
         IF(ILONG.EQ.NBMIX) THEN
           CALL LCMGET(IPDEPL,'MIXTURESBurn',IBASE(MIXBRN))
           CALL LCMGET(IPDEPL,'MIXTURESPowr',IBASE(MIXPWR))
         ELSE
           CALL XDISET(IBASE(MIXBRN),NBMIX,1)
           CALL XDISET(IBASE(MIXPWR),NBMIX,1)
         ENDIF
         CALL LCMLEN(IPDEPL,'ISOTOPESUSED',ILONG,ITYLCM)
         IF(ILONG.NE.3*NBISO) CALL XABORT('EVO: INCONSISTENT RECORD IS'
     1   //'OTOPESUSED.')
      ENDIF
*----
*  READ INPUT OPTIONS
*----
      IF(NBMIX.EQ.0) CALL XABORT('EVO: NBMIX NOT YET DEFINED.')
      CALL EVOGET(IMPX,ITYPE,ITIXS,IEXTR,IGLOB,ISAT,
     >            IDIRAC,ISAVE,ISET,INR,IDEPL,IFLMAC,RPAR,XT,
     >            NBMIX,IBASE(MIXBRN),IBASE(MIXPWR))
*
      XTI=XT(3)
      XTF=XT(5)
      EPS1=RPAR(1)
      EPS2=RPAR(2)
      EXPMAX=RPAR(3)
      H1=RPAR(4)
      FIT=RPAR(5)
      LOG=(ISAVE.GE.0).OR.(IDEPL.GT.0)
      IF(LOG.AND.(INR.EQ.-1)) CALL XABORT('EVO: TYPE OF DEPLETION NO'//
     1 'T DEFINED.')
      IF(NDEPL.EQ.0) CALL XABORT('EVO: NO DEPLETING ISOTOPES.')
      IF(NCOMB.EQ.0) CALL XABORT('EVO: NO DEPLETING MIXTURES.')
*----
*  RECOVER MIXTURE VOLUMES AND AVERAGE NEUTRON FLUXES
*----
      CALL SETARA(RBASE,NBMIX,IVX)
      CALL SETARA(RBASE,NBMIX*NGRO,JFLMIX)
      IF(IFLMAC.EQ.0) THEN
*        RECOVER GENERAL TRACKING INFORMATION
         IF(IPTRK.EQ.0) CALL XABORT('EVO: L_TRACK OBJECT NOT DEFINED.')
         CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
         NREG=IGP(1)
         IF(IGP(4).GT.NBMIX) THEN
            WRITE(HSMG,'(42HEVO: THE NUMBER OF MIXTURES IN THE TRACKIN,
     1      3HG (,I5,46H) IS GREATER THAN THE NUMBER OF MIXTURES IN TH,
     2      20HE INTERNAL LIBRARY (,I5,2H).)') IGP(4),NBMIX
            CALL XABORT(HSMG)
         ENDIF
         IF(NREG.EQ.0) CALL XABORT('EVO: NREG = 0')
         CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
         WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
         IF(CDOOR.EQ.'MCCG') THEN
            CALL LCMLEN(IPTRK,'KEYFLX',LKFL,ITYLCM)
            NFUNL=LKFL/NREG
         ELSE
            NFUNL=1
         ENDIF
         CALL SETARA(IBASE,NREG,IMAT)
         CALL SETARA(RBASE,NREG,IVOL)
         CALL SETARA(IBASE,NREG*NFUNL,IIDL)
         CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
         CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
         CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
*
*        RECOVER MIXTURE VOLUMES FROM L_TRACK
         CALL XDRSET(RBASE(IVX),NBMIX,0.0)
         CALL XDRSET(RBASE(JFLMIX),NBMIX*NGRO,0.0)
         DO 20 I=1,NREG
         IBM=IBASE(IMAT+I-1)
         IF(IBM.GT.0) RBASE(IVX+IBM-1)=RBASE(IVX+IBM-1)+RBASE(IVOL+I-1)
   20    CONTINUE
         IF(IPFLUX.NE.0) THEN
*           RECOVER MIXTURE FLUXES FROM L_FLUX
            CALL SETARA(RBASE,NREG*NGRO,JFLUXE)
            JPFLUX=LCMGID(IPFLUX,'FLUX')
            CALL LCMLEL(JPFLUX,1,ILONG,ITYLCM)
            CALL SETARA(RBASE,ILONG,IFUNKN)
            DO 30 IGR=1,NGRO
            CALL LCMGDL(JPFLUX,IGR,RBASE(IFUNKN))
            DO 30 I=1,NREG
            KEYFLX=IBASE(IIDL+I-1)
   30       RBASE(JFLUXE+(IGR-1)*NREG+I-1)=RBASE(IFUNKN+KEYFLX-1)
            CALL RLSARA(RBASE(IFUNKN))
            DO 50 IBM=1,NBMIX
            DO 50 IGR=1,NGRO
            FLXMIX=0.0
            DO 40 I=1,NREG
            IF(IBASE(IMAT+I-1).EQ.IBM) THEN
               VOLTMP=RBASE(IVOL+I-1)
               FLXMIX=FLXMIX+RBASE(JFLUXE+(IGR-1)*NREG+I-1)*VOLTMP
            ENDIF
   40       CONTINUE
            VOLTMP=RBASE(IVX+IBM-1)
            IF(VOLTMP.NE.0.0) THEN
               RBASE(JFLMIX+(IBM-1)*NGRO+IGR-1)=FLXMIX/VOLTMP
            ENDIF
   50       CONTINUE
            CALL RLSARA(RBASE(JFLUXE))
         ENDIF
         CALL RLSARA(IBASE(IIDL))
         CALL RLSARA(RBASE(IVOL))
         CALL RLSARA(IBASE(IMAT))
      ELSE IF(IFLMAC.EQ.1) THEN
*        RECOVER MIXTURE VOLUMES AND FLUXES FROM L_MACROLIB
         CALL LCMLEN(IPLIB,'MACROLIB',ILONG,ITYLCM)
         IF(ILONG.EQ.0) CALL XABORT('EVO: NO MACROLIB IN MICROLIB.')
         IPMACR=LCMGID(IPLIB,'MACROLIB')
         CALL LCMGET(IPMACR,'STATE-VECTOR',IPAR)
         IF(IPAR(1).NE.NGRO) CALL XABORT('EVO: INVALID NGRO.')
         IF(IPAR(2).NE.NBMIX) CALL XABORT('EVO: INVALID NBMIX.')
         CALL LCMGET(IPMACR,'VOLUME',RBASE(IVX))
         CALL SETARA(RBASE,NBMIX,IFUNKN)
         JPMACR=LCMGID(IPMACR,'GROUP')
         DO 60 IGR=1,NGRO
         KPMACR=LCMGIL(JPMACR,IGR)
         CALL LCMGET(KPMACR,'FLUX-INTG',RBASE(IFUNKN))
         DO 60 IBM=1,NBMIX
         RBASE(JFLMIX+(IBM-1)*NGRO+IGR-1)=RBASE(IFUNKN+IBM-1)/
     1   RBASE(IVX+IBM-1)
   60    CONTINUE
         CALL RLSARA(RBASE(IFUNKN))
      ELSE IF(IFLMAC.EQ.2) THEN
*        RECOVER MIXTURE VOLUMES AND FLUXES FROM L_POWER
         IF(IPPOW.EQ.0) CALL XABORT('EVO: L_POWER OBJECT NOT DEFINED.')
         CALL LCMGET(IPPOW,'STATE-VECTOR',IGP)
         NCH=IGP(6)
         NB=IGP(7)
         IGEO=IGP(8)
         IF((IGEO.NE.7).AND.(IGEO.NE.9))CALL XABORT('EVO: INVALID GEOM'
     1   //'ETRY IN FUEL MAP : ONLY 3-D CARTESIAN OR 3-D HEXAGONAL GEO'
     2   //'METRIES AVAILABLE') 
         IF(IGP(1).NE.NGRO)CALL XABORT('EVO: INVALID NUMBER OF ENERGY '
     1   //'GROUPS IN L_POWER AND MICROLIB.')
         CALL SETARA(IBASE,NCH*NB,IFMIX)
         CALL SETARA(RBASE,NCH*NB,IVMAP)
         CALL SETARA(RBASE,NCH*NB*NGRO,IFMAP)
         CALL LCMGET(IPPOW,'FLMIX',IBASE(IFMIX))
         CALL LCMGET(IPPOW,'VOLU-BUND',RBASE(IVMAP))
         CALL LCMGET(IPPOW,'FLUX-BUND',RBASE(IFMAP))
         NTOT=0
         DO 65 IB=1,NB*NCH
         IF(IBASE(IFMIX+IB-1).EQ.0) GO TO 70
         NTOT=NTOT+1
         IF(NTOT.GT.NBMIX) CALL XABORT('EVO: NBMIX OVERFLOW.')
         RBASE(IVX+NTOT-1)=RBASE(IVMAP+IB-1)
         DO 65 IGR=1,NGRO
   65    RBASE(JFLMIX+(NTOT-1)*NGRO+IGR-1)
     1              =RBASE(IFMAP+(IGR-1)*NB*NCH+IB-1)
   70    CONTINUE
         IF(NTOT.NE.NBMIX) CALL XABORT('EVO: ALGORITHM FAILURE.')
         CALL RLSARA(RBASE(IFMAP))
         CALL RLSARA(RBASE(IVMAP))
         CALL RLSARA(IBASE(IFMIX))
      ELSE
         CALL XABORT('EVO: INVALID VALUE OF IFLMAC.')
      ENDIF
*----
*  COMPUTE EXISTING FLUX NORMALIZATION FACTOR (KEEP OPTION)
*----
      IF(INR.EQ.4) THEN
         VPH=0.0
         VTOT=0.0
         DO 80 IBM=1,NBMIX
         IF(IBASE(MIXPWR+IBM-1).EQ.1) THEN
            VTOT=VTOT+RBASE(IVX+IBM-1)
            DO 75 IGR=1,NGRO
            IOF=NGRO*(IBM-1)+IGR
   75       VPH=VPH+RBASE(IVX+IBM-1)*RBASE(JFLMIX+IOF-1)
         ENDIF
   80    CONTINUE
         FIT=(VPH/VTOT)*1.0E13
         INR=1
      ENDIF
*----
*  PERFORM DEPLETION CALCULATION
*----
      CALL SETARA(IBASE,NBMIX,LMASKM)
      CALL SETARA(IBASE,NGRO,LMASKG)
      CALL SETARA(IBASE,NDEPL*NBMIX,IJM)
      CALL SETARA(IBASE,NCOMB,IMILVO)
      CALL SETARA(RBASE,(NDEPL-NSUPS+1)*NCOMB*2,IYDPL)
      CALL SETARA(IBASE,NBISO,ISOCMB)
      CALL SETARA(IBASE,3*NDEPL,IINADP)
      CALL SETARA(IBASE,NDEPL,IEVOL2)
      CALL SETARA(RBASE,3*(NDEPL-NSUPS+1)*NBMIX*(NREAC+1),ISIG)
      CALL SETARA(RBASE,3*NBMIX,IVPHV)
      CALL SETARA(RBASE,NBMIX,IENERG)
      CALL SETARA(RBASE,NDEPL,IAWR)
      CALL EVODRV(IPDEPL,IPLIB,INDREC,IMPX,NBISO,NGRO,NBMIX,
     1 IBASE(IISONA),IBASE(IISONR),IBASE(JJMIX),RBASE(JJDEN),
     2 IBASE(JIEVOL),RBASE(IVX),NDEPL,NSUPS,NREAC,NCOMB,EPS1,
     3 EPS2,EXPMAX,H1,ITYPE,INR,IEXTR,IGLOB,ISAT,IDIRAC,ITIXS,
     4 IFLMAC,FIT,ISAVE,ISET,IDEPL,XTI,XTF,XT,LMACRO,RBASE(JFLMIX),
     5 IBASE(MIXBRN),IBASE(MIXPWR),IBASE(LMASKM),IBASE(LMASKG),
     6 IBASE(IJM),IBASE(IMILVO),RBASE(IYDPL),IBASE(ISOCMB),
     7 IBASE(IINADP),IBASE(IEVOL2),RBASE(ISIG),RBASE(IVPHV),
     8 RBASE(IENERG),RBASE(IAWR))
      CALL RLSARA(RBASE(IAWR))
      CALL RLSARA(RBASE(IENERG))
      CALL RLSARA(RBASE(IVPHV))
      CALL RLSARA(RBASE(ISIG))
      CALL RLSARA(IBASE(IEVOL2))
      CALL RLSARA(IBASE(IINADP))
      CALL RLSARA(IBASE(ISOCMB))
      CALL RLSARA(RBASE(IYDPL))
      CALL RLSARA(IBASE(IMILVO))
      CALL RLSARA(IBASE(IJM))
      CALL RLSARA(IBASE(LMASKG))
      CALL RLSARA(IBASE(LMASKM))
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,RBASE(JJDEN))
*----
*  RELEASE GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL RLSARA(RBASE(JFLMIX))
      CALL RLSARA(RBASE(IVX))
      CALL RLSARA(IBASE(IISONR))
      CALL RLSARA(IBASE(IISONA))
      CALL RLSARA(IBASE(JIEVOL))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      RETURN
      END
