*DECK NXTHCP
      SUBROUTINE NXTHCP(IPRINT,MXMESH,NCROW,NCELL,SIDEH,DMESH)
*
*----------
*
*Purpose:
* Find position of each cell in an hexagonal assembly.
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal.
*
*Author(s): G. Marleau.
*
*Reference:
*  G. Marleau,
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260,
*  \'{E}cole Polytechnique de Montr\'{e}al, Montr\'{e}al,
*  2010.
*
*Parameters: input
* IPRINT  print level.
* MXMESH  maximum number of spatial subdivision in
*         $X$, $Y$ or $Z$.
* NCROW   number of crowns of hexagons in assembly.
* NCELL   number of hexagons in assembly.
* SIDEH   width of one side of an hexagon.
*
*Parameters: output
* DMESH   position of hexagons in assembly.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPRINT,MXMESH,NCROW,NCELL
      DOUBLE PRECISION SIDEH,DMESH(-1:MXMESH,3)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTHCP')
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Local variables
*----
      INTEGER          ICELL,ICROW,JSET
      DOUBLE PRECISION SQ3,SO2,SO4,SQ3O2S,SQ3O4S
*----
*  Print header if required
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      SQ3=SQRT(3.0D0)
      SO2=SIDEH/DTWO
      SO4=SO2/DTWO
      SQ3O2S=SQ3*SO2
      SQ3O4S=SQ3*SO4
      ICELL=-1
*----
*  Element -1 initialized to 0.0
*----
      DMESH(ICELL,1)=DZERO
      DMESH(ICELL,2)=DZERO
*----
*  Element 0 initialized to width of one side of the hexagon
*----
      ICELL=0
      DMESH(ICELL,1)=SIDEH
      DMESH(ICELL,2)=SIDEH
*----
*  For first crown
*  This cell is located at origin.
*----
      ICELL=1
      DMESH(ICELL,1)=DZERO
      DMESH(ICELL,2)=DZERO
*----
*  Scan over remaining crown
*----
      DO ICROW=2,NCROW
*----
*  First hexagonal sector
*----
        DO JSET=1,ICROW-1
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(2*ICROW-JSET-1)*SQ3O2S
          DMESH(ICELL,2)=DBLE(3*JSET-3)*SO2
        ENDDO
*----
*  Second hexagonal sector
*----
        DO JSET=ICROW,2*(ICROW-1)
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(3*ICROW-2*JSET-1)*SQ3O2S
          DMESH(ICELL,2)=DBLE(3*ICROW-3)*SO2
        ENDDO
*----
*  Third hexagonal sector
*----
        DO JSET=2*ICROW-1,3*(ICROW-1)
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(ICROW-JSET)*SQ3O2S
          DMESH(ICELL,2)=DBLE(9*ICROW-3*JSET-6)*SO2
        ENDDO
*----
*  Fourth hexagonal sector
*----
        DO JSET=3*ICROW-2,4*(ICROW-1)
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(4-5*ICROW+JSET)*SQ3O2S
          DMESH(ICELL,2)=DBLE(9*ICROW-3*JSET-6)*SO2
        ENDDO
*----
*  Fifth hexagonal sector
*----
        DO JSET=4*ICROW-3,5*(ICROW-1)
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(7-9*ICROW+2*JSET)*SQ3O2S
          DMESH(ICELL,2)=DBLE(3-3*ICROW)*SO2
        ENDDO
*----
*  Sixth hexagonal sector
*----
        DO JSET=5*ICROW-4,6*(ICROW-1)
          ICELL=ICELL+1
          DMESH(ICELL,1)=DBLE(3-4*ICROW+JSET)*SQ3O2S
          DMESH(ICELL,2)=DBLE(15-18*ICROW+3*JSET)*SO2
        ENDDO
      ENDDO
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6011) 'HexagonSIDE={         '
        WRITE(IOUT,6012) DMESH(0,1)
        WRITE(IOUT,6013)
        WRITE(IOUT,6011) 'HexagonX={         '
        WRITE(IOUT,6012) (DMESH(ICELL,1),ICELL=1,NCELL)
        WRITE(IOUT,6013)
        WRITE(IOUT,6011) 'HexagonY={         '
        WRITE(IOUT,6012) (DMESH(ICELL,2),ICELL=1,NCELL)
        WRITE(IOUT,6013)
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6011 FORMAT(A20)
 6012 FORMAT(6(1X,F25.16,:,','))
 6013 FORMAT('};')
      END
