*DECK SPH
      SUBROUTINE SPH(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* superhomogeneisation (SPH) procedure.
*
*Copyright:
* Copyright (C) 2011 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): Edition (L_EDIT), Microlib (L_LIBRARY),
*         Macrolib (L_MACROLIB) or Saphyb (L_SAPHYB) object;
*         HENTRY(I): I>1 read-only type (Edition (L_EDIT) and/or
*         L_MACROLIB and/or L_LIBRARY and/or L_SAPHYB and/or L_TRACK).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6,MAXISD=400)
      INTEGER IPOUT,IPRHS,IPTRK,IPEDIT,IPMACR,IPMICR,IPSAP
      INTEGER ISTATE(NSTATE),DIMSAP(50)
      CHARACTER CNDOOR*12,HSIGN*12,HSMG*131,TEXT12*12,HEDIT*12,HEQUI*80
      COMMON RBASE(1)
      INTEGER KCHAR(3),IBASE(1)
      LOGICAL LARM,LTEMP,LNEW
      REAL REALIR
      DOUBLE PRECISION DBLLIR
      EQUIVALENCE(RBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION.
*----
      IPTRK=0
      IFTRK=0
      IPFLX=0
      IF(NENTRY.EQ.0) CALL XABORT('SPH: MISSING PARAMETERS(1).')
      IF((IENTRY(1).LE.2).AND.(JENTRY(1).EQ.0)) THEN
         IMIN=3
         IPOUT=KENTRY(1)
         IF(NENTRY.EQ.1) CALL XABORT('SPH: MISSING PARAMETERS(2).')
         IF((IENTRY(2).LE.2).AND.(JENTRY(2).EQ.2)) THEN
            IPRHS=KENTRY(2)
         ELSE
            CALL XABORT('SPH: LHS LCM OBJECT EXPECTED.')
         ENDIF
      ELSE IF((IENTRY(1).LE.2).AND.(JENTRY(1).EQ.1)) THEN
         IMIN=2
         IPOUT=KENTRY(1)
         IPRHS=IPOUT
      ELSE
         CALL XABORT('SPH: RHS LCM OBJECT EXPECTED.')
      ENDIF
      CALL LCMGTC(KENTRY(IMIN-1),'SIGNATURE',12,1,HSIGN)
      IF(HSIGN.EQ.'L_EDIT') THEN
         ITYPE=1
      ELSE IF(HSIGN.EQ.'L_LIBRARY') THEN
         ITYPE=2
      ELSE IF(HSIGN.EQ.'L_MACROLIB') THEN
         ITYPE=3
      ELSE IF(HSIGN.EQ.'L_SAPHYB') THEN
         ITYPE=4
      ELSE IF(HSIGN.EQ.'L_MULTICOMPO') THEN
         ITYPE=5
      ELSE
         CALL XABORT('SPH: L_EDIT, L_LIBRARY, L_MACROLIB OR L_SAPHYB E'
     1   //'XPECTED AT FIRST RHS PARAMETER.')
      ENDIF
      DO 10 I=IMIN,NENTRY
      IF((IENTRY(I).LE.2).AND.(JENTRY(I).EQ.2)) THEN
         CALL LCMGTC(KENTRY(I),'SIGNATURE',12,1,HSIGN)
         IF(HSIGN.EQ.'L_TRACK') THEN
            IPTRK=KENTRY(I)
         ELSE IF(HSIGN.EQ.'L_FLUX') THEN
            IPFLX=KENTRY(I)
         ELSE
            WRITE(HSMG,'(40HSPH: UNKNOWN TYPE OF PARAMETER(1). NAME=,
     1      A12,11H<--- HSIGN=,A12,5H<---.)') HENTRY(I),HSIGN
            CALL XABORT(HSMG)
         ENDIF
      ELSE IF((IENTRY(I).EQ.3).AND.(JENTRY(I).EQ.2)) THEN
         IFTRK=KENTRY(I)
      ELSE
         WRITE(HSMG,'(40HSPH: UNKNOWN TYPE OF PARAMETER(2). NAME=,A12,
     1   5H<---.)') HENTRY(I)
         CALL XABORT(HSMG)
      ENDIF
   10 CONTINUE
*----
*  INITIALIZE OR RECOVER EXISTING SPH STATE-VECTOR
*----
      IF(ITYPE.EQ.1) THEN
*        TRY TO RECOVER SPH STATE-VECTOR FROM LAST-EDIT DIRECTORY
         CALL LCMGTC(IPRHS,'LAST-EDIT',12,1,HEDIT)
         IF(IPRINT.GT.0) THEN
            WRITE(6,'(32H SPH: STEP UP L_EDIT DIRECTORY '',A,5H''(1).)')
     1      HEDIT
         ENDIF
         IPEDIT=IPRHS
         IPMICR=LCMGID(IPEDIT,HEDIT)
         IPMACR=LCMGID(IPMICR,'MACROLIB')
         CALL LCMLEN(IPMICR,'SIGNATURE',ILONG,ITYLCM)
         IF(ILONG.EQ.0) IPMICR=0
         IPSAP=0
      ELSE IF(ITYPE.EQ.2) THEN
         IPEDIT=0
         IPMICR=IPRHS
         IPMACR=LCMGID(IPMICR,'MACROLIB')
         IPSAP=0
      ELSE IF(ITYPE.EQ.3) THEN
         IPEDIT=0
         IPMICR=0
         IPMACR=IPRHS
         IPSAP=0
      ELSE IF((ITYPE.EQ.4).OR.(ITYPE.EQ.5)) THEN
         IPEDIT=0
         IPMICR=0
         IPMACR=0
         IPSAP=IPRHS
      ENDIF
      ILEN=0
      IF(IPMACR.NE.0) CALL LCMLEN(IPMACR,'SPH',ILEN,ITYLCM)
      IF(ILEN.NE.0) THEN
         IPSPH=LCMGID(IPMACR,'SPH')
         CALL LCMGET(IPSPH,'STATE-VECTOR',ISTATE)
         NSPH=ISTATE(1)
         KSPH=ISTATE(2)
         MAXIT=ISTATE(3)
         MAXNBI=ISTATE(4)
         IBELL=ISTATE(5)
         ILHS=ISTATE(6)
         NWGTH=ISTATE(7)
         IF(NSPH.GE.2) THEN
            CALL LCMGET(IPSPH,'SPH$TRK',KCHAR)
            WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
            CALL LCMGET(IPSPH,'SPH-EPSILON',EPSPH)
         ELSE
            CNDOOR=' '
            EPSPH=0.0
         ENDIF
         LARM=(NSPH.EQ.4)
      ELSE
         NSPH=3
         KSPH=1
         MAXIT=200
         MAXNBI=10
         IBELL=0
         ILHS=0
         NWGTH=0
         EPSPH=1.0E-4
         CNDOOR=' '
         LARM=.FALSE.
      ENDIF
*----
*  SPH DIRECTIVE ANALYSIS
*----
      IPRINT=1
      HEDIT=' '
      HEQUI=' '
      ICAL=0
      B2=0.0
   20 CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
      IF(ITYPLU.EQ.10) GO TO 50
      IF(ITYPLU.NE.3) CALL XABORT('SPH: READ ERROR - CHARACTER VA'
     > //'RIABLE EXPECTED')
   30 IF(TEXT12.EQ.';') THEN
         GO TO 50
      ELSE IF(TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYPLU,IPRINT,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPH: READ ERROR - INTEGER'
     >   //' VARIABLE EXPECTED')
      ELSE IF(TEXT12.EQ.'IDEM') THEN
         ILHS=0
      ELSE IF(TEXT12.EQ.'MICRO') THEN
         IF((ITYPE.EQ.3).OR.(ITYPE.EQ.4)) THEN
            CALL XABORT('SPH: UNABLE TO PRODUCE A MICROLIB')
         ENDIF
         ILHS=2
      ELSE IF(TEXT12.EQ.'MACRO') THEN
         ILHS=3
      ELSE IF(TEXT12.EQ.'ASYM') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPH: READ ERROR - INTEGER V'
     >   //'ARIABLE EXPECTED')
         IF(INTLIR.LE.0) CALL XABORT('SPH: INVALID ASYMPTOTIC MIX'
     >   //'TURE SET')
         NSPH=3
         KSPH=-INTLIR
      ELSE IF(TEXT12.EQ.'STD') THEN
         KSPH=1
      ELSE IF(TEXT12.EQ.'SELE_ALB') THEN
         KSPH=2
      ELSE IF(TEXT12.EQ.'SELE_FD') THEN
         KSPH=3
      ELSE IF(TEXT12.EQ.'SELE_EDF') THEN
         KSPH=4
      ELSE IF(TEXT12.EQ.'SELE_MWG') THEN
         KSPH=6
      ELSE IF(TEXT12.EQ.'OFF') THEN
*        NO SPH CORRECTION PERFORMED
         NSPH=0
         KSPH=0
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'ON') THEN
*        SPH CORRECTION PERFORMED
         NSPH=3
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'SPRD') THEN
*        THE SPH FACTORS ARE READ FROM INPUT OR RECOVERED FROM RHS
         IF(NWGTH.NE.0) CALL XABORT('SPH: EXPECTING NWGTH=0.')
         NSPH=1
         KSPH=0
         CNDOOR=' '
         CALL REDGET(ITYPLU,NMERGO,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.EQ.3) THEN
           NSPH=0
           GO TO 30
         ELSE IF(ITYPLU.NE.1) THEN
           CALL XABORT('SPH: READ ERROR - INTEGER VARIABLE EXPECTED')
         ENDIF
         CALL REDGET(ITYPLU,NGCONO,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPH: READ ERROR - INTEGER'
     >   //' VARIABLE EXPECTED')
         CALL SETARA(RBASE,NMERGO*NGCONO*(NWGTH+1),ISPOLD)
         DO I=1,NMERGO*NGCONO
           CALL REDGET(ITYPLU,INTLIR,RBASE(ISPOLD+I-1),TEXT12,DBLLIR)
           IF(ITYPLU.NE.2) CALL XABORT('SPH: READ ERROR - REAL'
     >     //' VARIABLE EXPECTED')
         ENDDO
      ELSE IF(TEXT12.EQ.'HOMO') THEN
*        HOMOGENEOUS MACRO CALCULATION (NO ITERATIONS ARE PERFORMED)
         NSPH=2
         KSPH=0
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'ALBS') THEN
         NSPH=2
         KSPH=5
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'ITER') THEN
*        SPH ITERATION MAIN CONTROL PARAMETERS
   40    CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.EQ.1) THEN
            MAXIT=INTLIR
         ELSE IF(ITYPLU.EQ.2) THEN
            EPSPH=REALIR
         ELSE IF(ITYPLU.EQ.3) THEN
            GO TO 30
         ENDIF
         GO TO 40
      ELSE IF(TEXT12.EQ.'MAXNB') THEN
*        SPH ITERATION AUXILIARY CONTROL PARAMETERS
         CALL REDGET(ITYPLU,MAXNBI,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPH: READ ERROR - INTEGER'
     >   //' VARIABLE EXPECTED')
      ELSE IF(TEXT12.EQ.'NOBELL') THEN
         IBELL=0
      ELSE IF(TEXT12.EQ.'BELL') THEN
         IBELL=1
      ELSE IF(TEXT12.EQ.'ARM') THEN
         LARM=.TRUE.
      ELSE IF(TEXT12.EQ.'P0W') THEN
         NWGTH=0
      ELSE IF(TEXT12.EQ.'P1W') THEN
         NWGTH=1
      ELSE IF(TEXT12.EQ.'STEP') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPH: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
         IF(TEXT12.EQ.'UP') THEN
            IF((ITYPE.NE.1).AND.(ITYPE.NE.5)) THEN
              CALL XABORT('SPH: L_EDIT OR L_MULTICOMPO EXPECTED AT RHS')
            ENDIF
            CALL REDGET(ITYPLU,INTLIR,REALIR,HEDIT,DBLLIR)
            IF(ITYPLU.NE.3) CALL XABORT('SPH: READ ERROR - CHARACTER'
     >      //' VARIABLE EXPECTED')
         ELSE IF(TEXT12.EQ.'AT') THEN
            IF((ITYPE.NE.4).AND.(ITYPE.NE.5)) THEN
               CALL XABORT('SPH: L_SAPHYB OR L_MULTICOMPO EXPECTED AT '
     >         //'RHS')
            ENDIF
            CALL REDGET(ITYPLU,ICAL,REALIR,TEXT12,DBLLIR)
            IF(ITYPLU.NE.1) CALL XABORT('SPH: READ ERROR - INTEGER'
     >      //' VARIABLE EXPECTED')
            IF(ICAL.LE.0) CALL XABORT('SPH: INVALID VALUE OF ICAL')
         ELSE
            CALL XABORT('SPH: KEYWORD UP OR AT EXPECTED')
         ENDIF
      ELSE IF(TEXT12.EQ.'EQUI') THEN
         IF(ITYPE.NE.4) CALL XABORT('SPH: L_SAPHYB EXPECTED AT RHS')
         CALL REDGET(ITYPLU,INTLIR,REALIR,HEQUI,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPH: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
      ELSE IF(TEXT12.EQ.'LEAK') THEN
        CALL REDGET(ITYPLU,NITMA,B2,TEXT12,DFLOTT)
        IF(ITYPLU.NE.2) CALL XABORT('SPH: REAL DATA EXPECTED.')
      ELSE
         CALL XABORT('SPH: INVALID KEYWORD='//TEXT12)
      ENDIF
      GO TO 20
*----
*  RESET TO MICROLIB IN DIRECTORY HEDIT
*----
   50 IF(ITYPE.EQ.1) THEN
         IF(HEDIT.EQ.' ') CALL LCMGTC(IPEDIT,'LAST-EDIT',12,1,HEDIT)
         IF(IPRINT.GT.0) THEN
            WRITE(6,'(32H SPH: STEP UP L_EDIT DIRECTORY '',A,5H''(2).)')
     1      HEDIT
         ENDIF
         CALL LCMLEN(IPEDIT,HEDIT,ILONG,ITYLCM)
         IF(ILONG.EQ.0) THEN
            CALL LCMLIB(IPEDIT)
            CALL XABORT('SPH: MISSING DIRECTORY: '//HEDIT)
         ENDIF
         IPMICR=LCMGID(IPEDIT,HEDIT)
         IPMACR=LCMGID(IPMICR,'MACROLIB')
         CALL LCMLEN(IPMICR,'SIGNATURE',ILONG,ITYLCM)
         IF(ILONG.EQ.0) IPMICR=0
         IPSAP=0
      ENDIF
*----
*  SET POINTERS TO MACROLIB (IPMACR) AND OUTPUT (IPOUT) LCM OBJECTS
*----
      IF(ILHS.EQ.0) THEN
         ILHS2=ITYPE
      ELSE
         ILHS2=ILHS
      ENDIF
      IF((IPRHS.EQ.IPOUT).AND.(ITYPE.NE.ILHS2)) THEN
         IF(ILHS2.EQ.1) THEN
            CALL XABORT('SPH: CANNOT EXTRACT AN EDITION OBJECT FROM A '
     1      //'LCM OBJECT IN MODIFICATION MODE.')
         ELSE IF(ILHS2.EQ.2) THEN
            CALL XABORT('SPH: CANNOT EXTRACT A MICROLIB FROM A LCM OBJ'
     1      //'ECT IN MODIFICATION MODE.')
         ELSE IF(ILHS2.EQ.3) THEN
            CALL XABORT('SPH: CANNOT EXTRACT A MACROLIB FROM A LCM OBJ'
     1      //'ECT IN MODIFICATION MODE.')
         ELSE IF(ILHS2.EQ.4) THEN
            CALL XABORT('SPH: CANNOT EXTRACT A SAPHYB FROM A LCM OBJEC'
     1      //'T IN MODIFICATION MODE.')
         ELSE IF(ILHS2.EQ.5) THEN
            CALL XABORT('SPH: CANNOT EXTRACT A MULTICOMPO FROM A LCM O'
     1      //'BJECT IN MODIFICATION MODE.')
         ENDIF
      ELSE IF((IPRHS.NE.IPOUT).AND.(ILHS2.EQ.1)) THEN
         IF(IPEDIT.EQ.0) CALL XABORT('SPH: NO EDITION OBJECT ON RHS')
         CALL LCMEQU(IPEDIT,IPOUT)
         IF(IPRINT.GT.0) THEN
            WRITE(6,'(32H SPH: STEP UP L_EDIT DIRECTORY '',A,5H''(3).)')
     >      HEDIT
         ENDIF
         IPEDIT=IPOUT
         IPMICR=LCMGID(IPEDIT,HEDIT)
         IPMACR=LCMGID(IPMICR,'MACROLIB')
         CALL LCMLEN(IPMICR,'SIGNATURE',ILONG,ITYLCM)
         IF(ILONG.EQ.0) IPMICR=0
      ELSE IF((IPRHS.NE.IPOUT).AND.(ILHS2.EQ.2)) THEN
         IF(ITYPE.EQ.2) THEN
            IF(IPMICR.EQ.0) CALL XABORT('SPH: NO MICROLIB ON RHS')
            CALL LCMEQU(IPMICR,IPOUT)
            IPMACR=LCMGID(IPMICR,'MACROLIB')
         ELSE IF(ITYPE.EQ.5) THEN
            IPMACR=0
         ELSE
            CALL XABORT('SPH: RHS CANNOT BE CONVERTED TO A MICROLIB')
         ENDIF
         IPEDIT=0
         IPMICR=IPOUT
      ELSE IF((IPRHS.NE.IPOUT).AND.(ILHS2.EQ.3)) THEN
         IF((ITYPE.NE.4).AND.(ITYPE.NE.5)) CALL LCMEQU(IPMACR,IPOUT)
         IPEDIT=0
         IPMICR=0
         IPMACR=IPOUT
      ELSE IF((IPRHS.NE.IPOUT).AND.(ILHS2.EQ.4)) THEN
         IF(IPSAP.EQ.0) CALL XABORT('SPH: NO SAPHYB ON RHS')
         CALL LCMEQU(IPSAP,IPOUT)
         IPEDIT=0
         IPMICR=0
         IPMACR=0
      ELSE IF((IPRHS.NE.IPOUT).AND.(ILHS2.EQ.5)) THEN
         IF(IPSAP.EQ.0) CALL XABORT('SPH: NO MULTICOMPO ON RHS')
         CALL LCMEQU(IPSAP,IPOUT)
         IPEDIT=0
         IPMICR=0
         IPMACR=0
      ENDIF
*----
*  BUILD A MACROLIB IF NEEDED, ASSIGN AND INITIALIZE SPH-FACTOR ARRAY
*----
      LTEMP=.FALSE.
      IF(ITYPE.EQ.4) THEN
*        A Saphyb is given at RHS
         IF(ILHS2.EQ.3) THEN
            IPMACR=IPOUT
         ELSE IF(ILHS2.EQ.4) THEN
            LTEMP=.TRUE.
            CALL LCMOP(IPMACR,'*TEMPORARY*',0,1,0)
         ELSE
            CALL XABORT('SPH: OPTION NOT IMPLEMENTED(1).')
         ENDIF
         CALL LCMLEN(IPSAP,'DIMSAP',ILENG,ITYLCM)
         IF(ILENG.EQ.0) CALL XABORT('SPH: INVALID SAPHYB.')
         CALL LCMGET(IPSAP,'DIMSAP',DIMSAP)
         NMERGE=DIMSAP(7)   ! number of mixtures
         NGCOND=DIMSAP(20)  ! number of energy groups
         NWGTH=0
         CALL SETARA(RBASE,NMERGE*NGCOND,ISPH)
         CALL SETARA(RBASE,NMERGE*NGCOND,ISIGS)
         CALL SPHSAP(IPSAP,IPMACR,ICAL,IPRINT,HEQUI,NMERGE,NGCOND,
     >   RBASE(ISPH),RBASE(ISIGS))
         CALL RLSARA(RBASE(ISIGS))
      ELSE IF(ITYPE.EQ.5) THEN
*        A Multicompo is given at RHS
         IF(ILHS2.EQ.2) THEN
            IPMICR=IPOUT
         ELSE IF((ILHS2.EQ.3).OR.(ILHS2.EQ.5)) THEN
            LTEMP=.TRUE.
            CALL LCMOP(IPMICR,'*TEMPORARY*',0,1,0)
         ELSE
            CALL XABORT('SPH: OPTION NOT IMPLEMENTED(2).')
         ENDIF
         IF(HEDIT.EQ.' ') HEDIT='default'
         IF(IPRINT.GT.0) THEN
            WRITE(6,'(38H SPH: STEP UP L_MULTICOMPO DIRECTORY '',A,
     1      2H''.)') HEDIT
         ENDIF
         IPCPO=LCMGID(IPSAP,HEDIT)
         CALL LCMLEN(IPCPO,'STATE-VECTOR',ILENG,ITYLCM)
         IF(ILENG.EQ.0) CALL XABORT('SPH: INVALID MULTICOMPO.')
         CALL LCMGET(IPCPO,'STATE-VECTOR',ISTATE)
         NMERGE=ISTATE(1)  ! number of mixtures
         NGCOND=ISTATE(2)  ! number of energy groups
         NWGTH=0
         MAXISO=MAXISD*NMERGE
         CALL SETARA(IBASE,3*MAXISO,IHUSE1)
         CALL SETARA(IBASE,3*MAXISO,IHNAM1)
         CALL SETARA(RBASE,MAXISO,IDENS1)
         CALL SETARA(IBASE,MAXISO,IITYP1)
         CALL SETARA(IBASE,MAXISO,IITOD1)
         CALL SETARA(RBASE,MAXISO,ITEMP1)
         CALL SETARA(RBASE,MAXISO,IVOL1)
         CALL SETARA(IBASE,3*MAXISO,IHUSE2)
         CALL SETARA(IBASE,3*MAXISO,IHNAM2)
         CALL SETARA(RBASE,MAXISO,IDENS2)
         CALL SETARA(IBASE,MAXISO,IIMIX2)
         CALL SETARA(IBASE,MAXISO,IITYP2)
         CALL SETARA(IBASE,MAXISO,IITOD2)
         CALL SETARA(RBASE,MAXISO,ITEMP2)
         CALL SETARA(RBASE,MAXISO,IVOL2)
         CALL SETARA(RBASE,NGCOND+1,IENER)
         CALL SETARA(RBASE,NGCOND,IDELT)
         CALL SETARA(IBASE,NMERGE,IMILVO)
         CALL SETARA(RBASE,NMERGE,IVOLMI)
         CALL SPHCPO(MAXISO,IPMICR,IPCPO,NMERGE,NGCOND,IPRINT,ICAL,
     1   RBASE(IHUSE1),RBASE(IHNAM1),RBASE(IDENS1),RBASE(IITYP1),
     2   RBASE(IITOD1),RBASE(ITEMP1),RBASE(IVOL1),RBASE(IHUSE2),
     3   RBASE(IHNAM2),RBASE(IDENS2),RBASE(IIMIX2),RBASE(IITYP2),
     4   RBASE(IITOD2),RBASE(ITEMP2),RBASE(IVOL2),RBASE(IENER),
     5   RBASE(IDELT),RBASE(IMILVO),RBASE(IVOLMI))
         CALL RLSARA(RBASE(IVOLMI))
         CALL RLSARA(IBASE(IMILVO))
         CALL RLSARA(RBASE(IDELT))
         CALL RLSARA(RBASE(IENER))
         CALL RLSARA(RBASE(IVOL2))
         CALL RLSARA(RBASE(ITEMP2))
         CALL RLSARA(IBASE(IITOD2))
         CALL RLSARA(IBASE(IITYP2))
         CALL RLSARA(IBASE(IIMIX2))
         CALL RLSARA(RBASE(IDENS2))
         CALL RLSARA(IBASE(IHNAM2))
         CALL RLSARA(IBASE(IHUSE2))
         CALL RLSARA(RBASE(IVOL1))
         CALL RLSARA(RBASE(ITEMP1))
         CALL RLSARA(IBASE(IITOD1))
         CALL RLSARA(IBASE(IITYP1))
         CALL RLSARA(RBASE(IDENS1))
         CALL RLSARA(IBASE(IHNAM1))
         CALL RLSARA(IBASE(IHUSE1))
         IPMACR=LCMGID(IPMICR,'MACROLIB')
         IF(ILHS2.EQ.3) CALL LCMEQU(IPMACR,IPOUT)
         CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),ISPH)
         CALL XDRSET(RBASE(ISPH),NMERGE*NGCOND*(NWGTH+1),1.0)
      ELSE
*        A Edition/Microlib/Macrolib is given at RHS
         CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
         NGCOND=ISTATE(1)
         NMERGE=ISTATE(2)
         IF(NWGTH.GT.ISTATE(10)) CALL XABORT('SPH: TME MACROLIB IS NOT'
     >   //' CONSISTENT WITH OPTION P1W')
         CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),ISPH)
         CALL XDRSET(RBASE(ISPH),NMERGE*NGCOND*(NWGTH+1),1.0)
      ENDIF
*----
*  STORE SPH-RELATED INFORMATION
*----
      IF(NSPH.GT.0) THEN
         IPSPH=LCMDID(IPMACR,'SPH')
         IF((NSPH.GE.2).AND.(IPTRK.NE.0)) THEN
            CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
            WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(TEXT12.NE.'L_TRACK') THEN
               CALL XABORT('SPH: TRACKING DATA STRUCTURE EXPECTED.')
            ENDIF
            CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
            CALL LCMPUT(IPSPH,'SPH$TRK',3,3,KCHAR)
            CALL LCMPUT(IPSPH,'SPH-EPSILON',1,2,EPSPH)
            WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
            IF(CNDOOR.EQ.'SYBIL') THEN
*              SYBIL TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=3
               IF(LARM) NSPH=4
            ELSE IF(CNDOOR.EQ.'NXT') THEN
*              NXT TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=3
               IF(IFTRK.EQ.0) CALL XABORT('SPH: MISSING TRACKING FILE')
            ELSE IF(CNDOOR.EQ.'EXCELL') THEN
*              EXCELL TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=3
               IF(IFTRK.EQ.0) CALL XABORT('SPH: MISSING TRACKING FILE')
            ELSE IF(CNDOOR.EQ.'MCCG') THEN
*              MCCG TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=4
               IF(IFTRK.EQ.0) CALL XABORT('SPH: MISSING TRACKING FILE')
            ELSE IF(CNDOOR.EQ.'SN') THEN
*              SN TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=4
            ELSE IF(CNDOOR.EQ.'BIVAC') THEN
*              BIVAC TRANSPORT-DIFFUSION EQUIVALENCE
               NSPH=4
            ELSE IF(CNDOOR.EQ.'TRIVAC') THEN
*              TRIVAC TRANSPORT-DIFFUSION EQUIVALENCE
               NSPH=4
            ELSE
               CALL XABORT('SPH: '//CNDOOR//' IS AN INVALID TRACKING M'
     >         //'ODULE')
            ENDIF
         ENDIF
         CALL XDISET(ISTATE,NSTATE,0)
         ISTATE(1)=NSPH
         ISTATE(2)=KSPH
         ISTATE(3)=MAXIT
         ISTATE(4)=MAXNBI
         ISTATE(5)=IBELL
         ISTATE(6)=ILHS
         ISTATE(7)=NWGTH
         CALL LCMPUT(IPSPH,'STATE-VECTOR',NSTATE,1,ISTATE)
         IF(IPRINT.GT.0) WRITE(IOUT,200) (ISTATE(I),I=1,7),EPSPH,CNDOOR
      ENDIF
*----
*  COMPUTE SPH FACTORS
*----
      IF(NSPH.EQ.1) THEN
        IF(NWGTH.NE.0) CALL XABORT('SPH: EXPECTING NWGTH=0.')
        IF(NMERGE.NE.NMERGO) CALL XABORT('SPH: INVALID NUMBER OF REGIO'
     >  //'NS AFTER SPRD.')
        IF(NGCOND.NE.NGCONO) CALL XABORT('SPH: INVALID NUMBER OF GROUP'
     >  //'S AFTER SPRD.')
        DO I=1,NMERGE*NGCOND
          RBASE(ISPH+I-1)=RBASE(ISPOLD+I-1)
        ENDDO
        CALL RLSARA(RBASE(ISPOLD))
      ELSE IF(NSPH.GE.2) THEN
        CALL LCMLEN(IPMACR,'SPH',ILONG,ITYLCM)
        IF(ILONG.EQ.0) THEN
           CALL LCMLIB(IPMACR)
           CALL XABORT('SPH: NO SPH DIRECTORY AVAILABLE.')
        ENDIF
        CALL SPHDRV(IPTRK,IFTRK,IPMACR,IPFLX,IPRINT,NGCOND,NMERGE,NWGTH,
     >  RBASE(ISPH))
      ENDIF
*----
*  APPLY SPH CORRECTION
*----
      IF((ILHS2.LE.3).AND.(IPMICR.EQ.0)) THEN
*       Correction of Macrolib information
        CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
        NIFISS=ISTATE(4)
        NED=ISTATE(5)
        CALL SETARA(IBASE,2*(NED+1),IIHEDI)
        CALL SETARA(RBASE,NMERGE*NGCOND,IGAR1)
        CALL SETARA(RBASE,NMERGE*NIFISS,IGAR2)
        CALL SETARA(RBASE,NGCOND,IDIFHO)
        CALL SETARA(RBASE,NGCOND,ISPHHO)
        CALL SETARA(IBASE,NMERGE,INJJ)
        CALL SETARA(IBASE,NMERGE,IIJJ)
        CALL SETARA(IBASE,NMERGE,IIPOS)
        CALL SPHCMA(IPMACR,IPRINT,NMERGE,NGCOND,NWGTH,NIFISS,
     1  NED,RBASE(ISPH),IBASE(IIHEDI),RBASE(IGAR1),RBASE(IGAR2),
     2  RBASE(IDIFHO),RBASE(ISPHHO),IBASE(INJJ),IBASE(IIJJ),
     3  IBASE(IIPOS))
        CALL RLSARA(IBASE(IIPOS))
        CALL RLSARA(IBASE(IIJJ))
        CALL RLSARA(IBASE(INJJ))
        CALL RLSARA(RBASE(ISPHHO))
        CALL RLSARA(RBASE(IDIFHO))
        CALL RLSARA(RBASE(IGAR2))
        CALL RLSARA(RBASE(IGAR1))
        CALL RLSARA(IBASE(IIHEDI))
      ELSE IF(ILHS2.LE.3) THEN
*       Correction of Microlib information
        CALL LCMGET(IPMICR,'STATE-VECTOR',ISTATE)
        NISOT=ISTATE(2)
        NL=ISTATE(4)
        NED=ISTATE(13)
        NDEL=ISTATE(19)
        CALL SETARA(IBASE,2*(NED+1),IIHEDI)
        CALL SETARA(RBASE,NGCOND*(10+NL+NED+2*NDEL),IGAR)
        CALL SETARA(RBASE,NGCOND*NGCOND*NL,IWSCAT)
        CALL SETARA(IBASE,NL,IITYPR)
        CALL SPHCMI(IPMICR,IPRINT,NMERGE,NISOT,NGCOND,NWGTH,NL,NED,
     1  NDEL,RBASE(ISPH),IBASE(IIHEDI),RBASE(IGAR),RBASE(IWSCAT),
     2  IBASE(IITYPR))
        CALL RLSARA(IBASE(IITYPR))
        CALL RLSARA(RBASE(IWSCAT))
        CALL RLSARA(RBASE(IGAR))
        CALL RLSARA(IBASE(IIHEDI))
      ELSE IF((ILHS2.EQ.4).AND.(HEQUI.NE.' ')) THEN
*----
*  STORE A NEW SET OF SPH FACTORS IN THE SAPHYB
*----
        LNEW=(IPRHS.NE.IPOUT)
        CALL SPHSTO(IPOUT,ICAL,IPRINT,LNEW,HEQUI,NMERGE,NGCOND,
     1  RBASE(ISPH))
      ELSE IF(ILHS2.EQ.5) THEN
*----
*  APPLY A NEW SET OF SPH FACTORS IN THE MULTICOMPO
*----
        IPCPO=LCMGID(IPOUT,HEDIT)
        CALL SPHSCO(IPCPO,ICAL,IPRINT,NMERGE,NGCOND,NWGTH,RBASE(ISPH))
      ENDIF
*----
*  RELEASE MEMORY ALLOCATED FOR MACROLIB/MICROLIB AND SPH FACTORS
*----
      IF((IPMICR.NE.0).AND.LTEMP) THEN
        CALL LCMCL(IPMICR,2)
      ELSE IF(LTEMP) THEN
        CALL LCMCL(IPMACR,2)
      ENDIF
      CALL RLSARA(RBASE(ISPH))
*----
*  INCLUDE LEAKAGE IN THE MACROLIB (USED ONLY FOR NON-REGRESSION TESTS)
*----
      IF(B2.NE.0.0) THEN
        IF(ILHS2.EQ.1) THEN
           CALL LCMLEN(IPOUT,HEDIT,ILONG,ITYLCM)
           IF(ILONG.EQ.0) THEN
              CALL LCMLIB(IPEDIT)
              CALL XABORT('SPH: MISSING DIRECTORY: '//HEDIT)
           ENDIF
           IPMICR=LCMGID(IPOUT,HEDIT)
           IPMACR=LCMGID(IPMICR,'MACROLIB')
        ELSE IF(ILHS2.EQ.2) THEN
           IPMACR=LCMGID(IPOUT,'MACROLIB')
        ELSE IF(ILHS2.EQ.3) THEN
           IPMACR=IPOUT
        ELSE
           CALL XABORT('SPH: LHS MACROLIB EXPECTED WITH LEAK OPTION.')
        ENDIF
        CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
        NGRP=ISTATE(1)
        NMIX=ISTATE(2)
        JPMAC=LCMGID(IPMACR,'GROUP')
        CALL SETARA(RBASE,NMIX,IWORK1)
        CALL SETARA(RBASE,NMIX,IWORK2)
        DO 70 IGR=1,NGRP
          KPMAC=LCMGIL(JPMAC,IGR)
          CALL LCMGET(KPMAC,'NTOT0',RBASE(IWORK1))
          CALL LCMGET(KPMAC,'DIFF',RBASE(IWORK2))
          DO 60 IBM=1,NMIX
   60     RBASE(IWORK1+IBM-1)=RBASE(IWORK1+IBM-1)+B2*RBASE(IWORK2+IBM-1)
          CALL LCMPUT(KPMAC,'NTOT0',NMIX,2,RBASE(IWORK1))
   70   CONTINUE
        CALL RLSARA(RBASE(IWORK2))
        CALL RLSARA(RBASE(IWORK1))
      ENDIF
      IF(IPRINT.GT.5) CALL LCMLIB(IPOUT)
      RETURN
*
  200 FORMAT(/20H SPH-RELATED OPTIONS/1X,19(1H-)/
     1  7H NSPH  ,I8,47H   (=0: NO SPH CORRECTION; =1: READ SPH FACTORS,
     2  32H; >1: TYPE OF MACRO-CALCULATION)/
     3  7H KSPH  ,I8,47H   (<0: ASYMPTOTIC SPH NORMALIZATION; =1: AVERA,
     4  54HGE FLUX SPH NORMALIZATION; >1: SELENGUT NORMALIZATION)/
     5  7H MAXIT ,I8,37H   (MAXIMUM NUMBER OF SPH ITERATIONS)/
     6  7H MAXNBI,I8,47H   (MAXIMUM NUMBER OF BAD ITERATIONS BEFORE ABO,
     7  6HRTING)/
     8  7H IBELL ,I8,42H   (=0/1: BELL FACTOR ACCELERATION OFF/ON)/
     9  7H ILHS  ,I8,47H   (=0/1/2/3: PRODUCE A RHS-TYPE/EDITION/MICROL,
     1  19HIB/MACROLIB AT LHS)/
     2  7H NWGTH ,I8,47H   (=0/1: USE P0 SPH FACTORS EVERYWHERE/USE BOT,
     3  24HH P0 AND P1 SPH FACTORS)/
     4  8H EPSPH  ,1P,E7.1,26H   (CONVERGENCE CRITERION)/
     5  8H CNDOOR ,A8,37H  (MACRO-CALCULATION TRACKING MODULE))
      END
