*DECK SPHDRV
      SUBROUTINE SPHDRV(IPTRK,IFTRK,IPMACR,IPFLX,IPRINT,NGCOND,NMERGE,
     1 NWGTH,SPH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the SPH factors. These factors are used to multiply the
* cross sections and to divide the fluxes. The SPH factors calculation
* is generally application dependent. New SPH algorithms should be
* implemented in this driver.
*
*Copyright:
* Copyright (C) 2011 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the macro-tracking LCM object.
* IFTRK   pointer to the macro-tracking binary sequential file.
* IPMACR  pointer to the Macrolib (L_MACROLIB signature).
* IPFLX   pointer towards an initialization flux (L_FLUX signature).
* IPRINT  print flag (equal to 0 for no print).
* NGCOND  number of condensed groups.
* NMERGE  number of merged regions.
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1. P1-weighted SPH factors are computed if NWGTH
*         is greater than zero.
*
*Parameters: output
* SPH     SPH homogenization factors.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   IPTRK,IFTRK,IPMACR,IPFLX,IPRINT,NGCOND,NMERGE,NWGTH
      REAL      SPH(NMERGE,NGCOND,NWGTH+1)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER TEXT12*12,CNDOOR*12,CTITRE*72,SUFF(2)*2
      INTEGER   ISTATE(NSTATE),KCHAR(18),IBASE(1)
      LOGICAL   ILK
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      DATA      SUFF/'00','01'/
*----
*  RECOVER SPH-RELATED INFORMATION
*----
      CALL LCMLEN(IPMACR,'SPH',ILONG,ITYLCM)
      IF(ILONG.EQ.0) CALL XABORT('SPHDRV: MISSING SPH DIRECTORY')
      IPSPH=LCMDID(IPMACR,'SPH')
      CALL LCMGET(IPSPH,'STATE-VECTOR',ISTATE)
      NSPH=ISTATE(1)
      KSPH=ISTATE(2)
      MAXIT=ISTATE(3)
      MAXNBI=ISTATE(4)
      IBELL=ISTATE(5)
      IF((NSPH.EQ.0).OR.(NSPH.EQ.1)) CALL XABORT('SPHDRV: INVALID VALU'
     > //'E OF NSPH.')
*----
*  RECOVER AND USE AN EXISTING MACRO-TRACKING.
*----
      IF(IPTRK.NE.0) THEN
         IF(NSPH.GE.2) THEN
            CALL LCMGET(IPSPH,'SPH$TRK',KCHAR)
            WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
            CALL LCMGET(IPSPH,'SPH-EPSILON',EPSPH)
         ENDIF
         CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
         WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
         IF(TEXT12.NE.'L_TRACK') THEN
            CALL XABORT('SPHDRV: TRACKING DATA STRUCTURE EXPECTED.')
         ENDIF
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         NREG2=ISTATE(1)
         NUN2=ISTATE(2)
         ILK=ISTATE(3).EQ.0
         CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
         WRITE(CNDOOR,'(3A4)') (KCHAR(I),I=1,3)
         IF(CNDOOR.EQ.'MCCG') THEN
            CALL LCMLEN(IPTRK,'KEYFLX',LKFL,ITYLCM)
            NFUNL=LKFL/NREG2
         ELSE
            NFUNL=1
         ENDIF
         IF((CNDOOR.EQ.'SYBIL').AND.(NSPH.EQ.4)) NUN2=NUN2+ISTATE(9)
         IF((CNDOOR.EQ.'EXCELL').OR.(CNDOOR.EQ.'MCCG')) THEN
            ISCAT=ISTATE(6)
         ELSE IF(CNDOOR.EQ.'SN') THEN
            ISCAT=ISTATE(16)
         ELSE IF(CNDOOR.EQ.'BIVAC') THEN
            IF(ISTATE(14).EQ.0) THEN
               ISCAT=1
            ELSE
               ISCAT=ISTATE(16)
            ENDIF
         ELSE IF(CNDOOR.EQ.'TRIVAC') THEN
            IF(ISTATE(30).EQ.0) THEN
               ISCAT=1
            ELSE
               ISCAT=ISTATE(32)
            ENDIF
         ELSE
            ISCAT=1
         ENDIF
         ISCAT=MAX(ISCAT,NWGTH+1)
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2*NFUNL,IKEY2)
         CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL2))
         CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT2))
         CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEY2))
         CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPTRK,'TITLE',KCHAR)
            WRITE(CTITRE,'(18A4)') (KCHAR(I1),I1=1,18)
         ELSE
            CTITRE='*** NO TITLE PROVIDED ***'
         ENDIF
         NBMIX2=0
         IF(KSPH.EQ.5) THEN
*          HEBERT-BENOIST ALBS TECHNIQUE.
           DO 20 IREG=1,NREG2
   20      NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
           CALL SETARA(IBASE,NBMIX2,IMERG2)
           DO 25 IBM=1,NBMIX2
   25      IBASE(IMERG2+IBM-1)=1
           ILK=.FALSE.
         ELSE
           DO 30 IREG=1,NREG2
   30      NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
           IF(NBMIX2.NE.NMERGE) CALL XABORT('SPHDRV: INVALID NUMBER OF'
     1     //' MACRO-REGIONS.')
           CALL SETARA(IBASE,NBMIX2,IMERG2)
           DO 35 IBM=1,NBMIX2
   35      IBASE(IMERG2+IBM-1)=IBM
         ENDIF
*
*        RECOVER TABULATED FUNCTIONS.
         CALL XDRTA2(IPTRK)
      ELSE
         ISCAT=NWGTH+1
         ILK=.FALSE.
         NBMIX2=0
         NREG2=0
         NUN2=0
      ENDIF
*----
*  GENERAL PROCEDURE FOR COMPUTING THE SPH FACTORS
*----
      CALL SETARA(RBASE,(NBMIX2+1)*ISCAT,ISIGMA)
      CALL SETARA(RBASE,(NBMIX2+1)*2,ISIGMS)
      CALL SETARA(RBASE,NBMIX2+1,ISIGMD)
      CALL SETARA(RBASE,NGCOND*(NWGTH+1),ICOURI)
      CALL SETARA(RBASE,NGCOND*(NWGTH+1),ICOURO)
      CALL SETARA(RBASE,NMERGE,IVOLME)
      CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),ISIGT)
      CALL SETARA(RBASE,NMERGE*NGCOND*ISCAT,ISIGW)
      CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),IFLXME)
      CALL SETARA(RBASE,NMERGE*NGCOND,IDIFF)
      CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),IZLEAK)
      CALL SETARA(RBASE,NMERGE*(NGCOND**2)*ISCAT,ISUNME)
      CALL SETARA(RBASE,NUN2*NGCOND,IFUNKN)
      CALL SETARA(RBASE,NUN2*NGCOND,ISUNKN)
      CALL SETARA(RBASE,NMERGE,IFACTO)
      CALL SETARA(RBASE,NGCOND,INPSYS)
      CALL SPHEQU(NBMIX2,IPTRK,IFTRK,IPMACR,IPFLX,CNDOOR,NSPH,KSPH,
     1 MAXIT,MAXNBI,IBELL,EPSPH,IPRINT,NGCOND,NMERGE,ISCAT,NWGTH,NREG2,
     2 NUN2,IBASE(IMAT2),RBASE(IVOL2),IBASE(IKEY2),IBASE(IMERG2),
     3 ILK,CTITRE,SPH,RBASE(ISIGMA),RBASE(ISIGMS),RBASE(ISIGMD),
     4 RBASE(ICOURI),RBASE(ICOURO),RBASE(IVOLME),RBASE(ISIGT),
     5 RBASE(ISIGW),RBASE(IFLXME),RBASE(IDIFF),RBASE(IZLEAK),
     6 RBASE(ISUNME),RBASE(IFUNKN),RBASE(ISUNKN),RBASE(IFACTO),
     7 RBASE(INPSYS))
      CALL RLSARA(RBASE(INPSYS))
      CALL RLSARA(RBASE(IFACTO))
      CALL RLSARA(RBASE(ISUNKN))
      CALL RLSARA(RBASE(IFUNKN))
      CALL RLSARA(RBASE(ISUNME))
      CALL RLSARA(RBASE(IZLEAK))
      CALL RLSARA(RBASE(IDIFF))
      CALL RLSARA(RBASE(IFLXME))
      CALL RLSARA(RBASE(ISIGW))
      CALL RLSARA(RBASE(ISIGT))
      CALL RLSARA(RBASE(IVOLME))
      CALL RLSARA(RBASE(ICOURO))
      CALL RLSARA(RBASE(ICOURI))
      CALL RLSARA(RBASE(ISIGMD))
      CALL RLSARA(RBASE(ISIGMS))
      CALL RLSARA(RBASE(ISIGMA))
      IF(IPTRK.NE.0) THEN
         CALL RLSARA(IBASE(IMERG2))
         CALL RLSARA(IBASE(IKEY2))
         CALL RLSARA(IBASE(IMAT2))
         CALL RLSARA(RBASE(IVOL2))
      ENDIF
*----
*  PRINT SPH FACTORS
*----
      IF(IPRINT.GT.1) THEN
         DO 50 INL=1,NWGTH+1
         WRITE(6,'(/29H SPHDRV: SPH FACTORS (ORDER P,A,2H):)') SUFF(INL)
         WRITE(6,200) ((IKK,IGR,SPH(IKK,IGR,INL),IKK=1,NMERGE),IGR=1,
     >   NGCOND)
   50    CONTINUE
      ENDIF
      RETURN
*
  200 FORMAT(4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5)
      END
