*DECK SPHSAP
      SUBROUTINE SPHSAP(IPSAP,IPMAC,ICAL,IMPX,HEQUI,NMIL,NGROUP,SPH,
     1 SIGS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* extract a Macrolib corresponding to an elementary calculation in a
* Saphyb.
*
*Copyright:
* Copyright (C) 2011 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSAP   pointer to the Saphyb (L_SAPHYB signature).
* ICAL    index of the elementary calculation being considered.
* IMPX    print parameter (equal to zero for no print).
* HEQUI   name of SPH-factor set to be recovered.
* NMIL    number of mixtures in the elementary calculation.
* NGROUP  number of energy groups in the elementary calculation.
*
*Parameters: output
* IPMAC   pointer to the Macrolib (L_MACROLIB signature).
* SPH     SPH-factor set extracted from the Saphyb.
*
*Parameters: scratch
* SIGS    P0 scattering cross sections.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSAP,IPMAC
      INTEGER ICAL,IMPX,NMIL,NGROUP
      REAL SPH(NMIL,NGROUP),SIGS(NMIL,NGROUP)
      CHARACTER HEQUI*80
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,MAXREA=25,MAXMAC=2,MAXDIV=3,MAXLOC=10,
     1 MAXMIL=100)
      INTEGER ISTATE(NSTATE),DIMSAP(50)
      INTEGER IPOS(MAXMIL),NJJM(MAXMIL),IJJM(MAXMIL)
      REAL BASE(1)
      REAL VALDIV(MAXDIV)
      LOGICAL LSTRD,LDIFF,LSPH,LBASE(1)
      CHARACTER TEXT12*12,HSMG*131,NOMREA(MAXREA)*12,CM*2,
     1 IDVAL(MAXDIV)*4,LOCTYP(MAXLOC)*4,LOCNAM(MAXLOC)*80,
     2 NOMMIL(MAXMIL)*20,TEXT8*8,TEXT9*8
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1),LBASE(1))
*----
*  RECOVER SAPHYB CHARACTERISTICS
*----
      CALL LCMLEN(IPSAP,'DIMSAP',ILENG,ITYLCM)
      IF(ILENG.EQ.0) CALL XABORT('SPHSAP: INVALID SAPHYB.')
      CALL LCMGET(IPSAP,'DIMSAP',DIMSAP)
      IF(NMIL.NE.DIMSAP(7)) THEN
         CALL XABORT('SPHSAP: INVALID VALUE OF NMIL.')
      ELSE IF(NGROUP.NE.DIMSAP(20)) THEN
         CALL XABORT('SPHSAP: INVALID VALUE OF NGROUP.')
      ENDIF
      NREA=DIMSAP(4)   ! number of reactions
      NISO=DIMSAP(5)   ! number of particularized isotopes
      NMAC=DIMSAP(6)   ! number of macroscopic sets
      NPARL=DIMSAP(11) ! number of local variables
      NADRX=DIMSAP(18) ! number of address sets
      NCALS=DIMSAP(19) ! number of elementary calculations in the Saphyb
      NPRC=DIMSAP(31)  ! number of delayed neutron precursor groups
      NISOTS=DIMSAP(32) ! number of isotopes in edition tables
      IF(IMPX.GT.1) THEN
        WRITE(6,'(30H SPHSAP: number of reactions =,I3)') NREA
        WRITE(6,'(44H SPHSAP: number of particularized isotopes =,I4)')
     1  NISO
        WRITE(6,'(37H SPHSAP: number of macroscopic sets =,I2)') NMAC
        WRITE(6,'(29H SPHSAP: number of mixtures =,I5)') NMIL
        WRITE(6,'(36H SPHSAP: number of local variables =,I4)') NPARL
        WRITE(6,'(33H SPHSAP: number of address sets =,I4)') NADRX
        WRITE(6,'(33H SPHSAP: number of calculations =,I5)') NCALS
        WRITE(6,'(34H SPHSAP: number of energy groups =,I4)') NGROUP
        WRITE(6,'(37H SPHSAP: number of precursor groups =,I4)') NPRC
        WRITE(6,'(46H SPHSAP: number of isotopes in output tables =,
     1  I4)') NISOTS
      ENDIF
      IF(NREA.GT.MAXREA) CALL XABORT('SPHSAP: MAXREA OVERFLOW')
      IF(NMAC.GT.MAXMAC) CALL XABORT('SPHSAP: MAXMAC OVERFLOW')
      INDX=NISO+NMAC
      IF(INDX.EQ.0) CALL XABORT('SPHSAP: NO CROSS SECTIONS FOUND.')
*----
*  RECOVER INFORMATION FROM constphysiq DIRECTORY.
*----
      CALL SETARA(BASE,NGROUP+1,IENER)
      CALL LCMSIX(IPSAP,'constphysiq',1)
      CALL LCMGET(IPSAP,'ENRGS',BASE(IENER))
      CALL LCMSIX(IPSAP,' ',2)
      DO IGR=1,NGROUP+1
        BASE(IENER+IGR-1)=BASE(IENER+IGR-1)/1.0E-6
      ENDDO
      CALL LCMPUT(IPMAC,'ENERGY',NGROUP+1,2,BASE(IENER))
      CALL RLSARA(BASE(IENER))
*----
*  RECOVER INFORMATION FROM contenu DIRECTORY.
*----
      CALL SETARA(IBASE,NMIL,ITOTM)
      CALL SETARA(IBASE,NMIL,IRESM)
      CALL LCMSIX(IPSAP,'contenu',1)
      IF(NREA.GT.0) THEN
        CALL LCMGTC(IPSAP,'NOMREA',12,NREA,NOMREA)
        IF(IMPX.GT.1) THEN
          WRITE(6,'(29H SPHSAP: Available reactions:/(1X,10A13))')
     1    (NOMREA(I),I=1,NREA)
        ENDIF
      ENDIF
      CALL LCMGET(IPSAP,'TOTMAC',IBASE(ITOTM))
      CALL LCMGET(IPSAP,'RESMAC',IBASE(IRESM))
      IF(NISO.GT.0) THEN
        CALL SETARA(IBASE,NISO*2,INOMIS)
        CALL LCMGET(IPSAP,'NOMISO',IBASE(INOMIS))
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
*----
*  RECOVER INFORMATION FROM adresses DIRECTORY.
*----
      NL=0
      IF(NADRX.GT.0) THEN
         CALL SETARA(IBASE,(NREA+2)*(NISO+NMAC)*NADRX,IADRX)
         CALL LCMSIX(IPSAP,'adresses',1)
         CALL LCMLIB(IPSAP)
         CALL LCMGET(IPSAP,'ADRX',IBASE(IADRX))
         CALL LCMSIX(IPSAP,' ',2)
         DO IAD=1,NADRX
          DO ISO=1,NISO+NMAC
            IOF=(NREA+2)*(NISO+NMAC)*(IAD-1)+(NREA+2)*(ISO-1)+NREA+1
            NL=MAX(NL,IBASE(IADRX+IOF-1))
            IOF=(NREA+2)*(NISO+NMAC)*(IAD-1)+(NREA+2)*(ISO-1)+NREA+2
            NL=MAX(NL,IBASE(IADRX+IOF-1))
          ENDDO
         ENDDO
      ENDIF
      IF(IMPX.GT.1) THEN
        WRITE(6,'(36H SPHSAP: number of legendre orders =,I4)') NL
      ENDIF
*----
*  RECOVER INFORMATION FROM geom DIRECTORY.
*----
      CALL LCMSIX(IPSAP,'geom',1)
      CALL SETARA(BASE,NMIL,IXVOLM)
      CALL LCMGET(IPSAP,'XVOLMT',BASE(IXVOLM))
      CALL LCMGTC(IPSAP,'NOMMIL',20,NMIL,NOMMIL)
      CALL LCMSIX(IPSAP,' ',2)
*----
*  RECOVER INFORMATION FROM caldir DIRECTORY.
*----
      WRITE(TEXT12,'(4Hcalc,I8)') ICAL
      CALL LCMLEN(IPSAP,TEXT12,ILENG,ITYLCM)
      IF(ILENG.EQ.0) THEN
         WRITE(HSMG,'(29HSPHSAP: MISSING CALCULATION '',A12,2H''.)')
     1   TEXT12
         CALL XABORT(HSMG)
      ENDIF
      CALL LCMSIX(IPSAP,TEXT12,1)
      CALL LCMSIX(IPSAP,'info',1)
      LSPH=.FALSE.
      IF(NPARL.GT.0) THEN
        CALL LCMGET(IPSAP,'NLOC',NLOC)
        IF(NLOC.GT.MAXLOC) CALL XABORT('SPHSAP: MAXLOC OVERFLOW')
        CALL LCMGTC(IPSAP,'LOCTYP',4,NLOC,LOCTYP)
        CALL LCMGTC(IPSAP,'LOCNAM',80,NLOC,LOCNAM)
        CALL SETARA(IBASE,NLOC+1,ILOCAD)
        CALL LCMGET(IPSAP,'LOCADR',IBASE(ILOCAD))
        DO ILOC=1,NLOC
          LSPH=((LOCTYP(ILOC).EQ.'EQUI').AND.(LOCNAM(ILOC).EQ.HEQUI))
          IF(LSPH) EXIT
        ENDDO
      ENDIF
      CALL SETARA(IBASE,NMIL,ISADRX)
      CALL SETARA(IBASE,NMIL,LENGDX)
      CALL SETARA(IBASE,NMIL,LENGDP)
      CALL LCMGET(IPSAP,'ISADRX',IBASE(ISADRX))
      CALL LCMGET(IPSAP,'LENGDX',IBASE(LENGDX))
      CALL LCMGET(IPSAP,'LENGDP',IBASE(LENGDP))
      IF(NISOTS.GT.0) THEN
        CALL SETARA(IBASE,NISOTS*2,IISOTS)
        CALL LCMGET(IPSAP,'ISOTS',IBASE(IISOTS))
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
      CALL LCMSIX(IPSAP,'divers',1)
      CALL LCMLEN(IPSAP,'NVDIV',ILENG,ITYLCM)
      IF(ILENG.EQ.0) THEN
        NVDIV=0
      ELSE
        CALL LCMGET(IPSAP,'NVDIV',NVDIV)
      ENDIF
      B2=0.0
      IF(NVDIV.GT.0) THEN
        IF(NVDIV.GT.MAXDIV) CALL XABORT('SPHSAP: MAXDIV OVERFLOW.')
        CALL LCMGTC(IPSAP,'IDVAL',4,NVDIV,IDVAL)
        CALL LCMGET(IPSAP,'VALDIV',VALDIV)
        DO I=1,NVDIV
          IF(IMPX.GT.1) THEN
            WRITE(6,'(9H SPHSAP: ,I3,2X,A,1H=,1P,E13.5)') I,IDVAL(I),
     1      VALDIV(I)
          ENDIF
          IF(IDVAL(I).EQ.'KEFF') THEN
            CALL LCMPUT(IPMAC,'K-EFFECTIVE',1,2,VALDIV(I))
          ELSE IF(IDVAL(I).EQ.'KINF') THEN
            CALL LCMPUT(IPMAC,'K-INFINITY',1,2,VALDIV(I))
          ELSE IF(IDVAL(I).EQ.'B2') THEN
            B2=VALDIV(I)
            CALL LCMPUT(IPMAC,'B2  B1HOM',1,2,VALDIV(I))
          ENDIF
        ENDDO
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
*----
*  ALLOCATE MACROLIB WORKING ARRAYS.
*----
      CALL SETARA(BASE,NGROUP*NMIL,INWT0)
      CALL SETARA(BASE,NGROUP*NMIL*NL,ISIGS)
      CALL SETARA(BASE,NGROUP*NGROUP*NMIL*NL,ISS2D)
      CALL SETARA(BASE,NGROUP*NMIL*NREA,IXS)
      CALL SETARA(LBASE,NREA,ILXS)
      CALL XDRSET(BASE(INWT0),NGROUP*NMIL,0.0)
      CALL XDRSET(BASE(ISIGS),NGROUP*NMIL*NL,0.0)
      CALL XDRSET(BASE(ISS2D),NGROUP*NGROUP*NMIL*NL,0.0)
      CALL XDRSET(BASE(IXS),NGROUP*NMIL*NREA,0.0)
      CALL XDLSET(LBASE(ILXS),NREA,.FALSE.)
*----
*  ALLOCATE DELAYED NEUTRON WORKING ARRAYS.
*----
      CALL SETARA(BASE,NPRC,ILAMB)
      CALL SETARA(BASE,NGROUP*NMIL*NPRC,ICHIRS)
      CALL SETARA(BASE,NMIL*NPRC,IBETAR)
      CALL SETARA(BASE,NMIL*NGROUP,INVELS)
      CALL XDRSET(BASE(ILAMB),NPRC,0.0)
      CALL XDRSET(BASE(ICHIRS),NGROUP*NMIL*NPRC,0.0)
      CALL XDRSET(BASE(IBETAR),NMIL*NPRC,0.0)
      CALL XDRSET(BASE(INVELS),NMIL*NGROUP,0.0)
      CALL LCMSIX(IPSAP,'divers',1)
      CALL LCMLEN(IPSAP,'NPR',ILONG,ITYLCM)
      IF((NPRC.GT.0).AND.(ILONG.EQ.1)) THEN
        CALL LCMGET(IPSAP,'NPR',NPR)
        IF(NPR.NE.NPRC) CALL XABORT('SPHSAP: NPR INCONSISTENCY(1).')
        CALL LCMGET(IPSAP,'LAMBRS',BASE(ILAMB))
        DO IBM=1,NMIL
          CALL LCMGET(IPSAP,'CHIRS',BASE(ICHIRS+(IBM-1)*NPRC*NGROUP))
          CALL LCMGET(IPSAP,'BETARS',BASE(IBETAR+(IBM-1)*NPRC))
          CALL LCMGET(IPSAP,'INVELS',BASE(INVELS+(IBM-1)*NGROUP))
        ENDDO
      ENDIF
      CALL LCMSIX(IPSAP,' ',2)
*----
*  LOOP OVER SAPHYB MIXTURES.
*----
      IF(NADRX.EQ.0) CALL XABORT('SPHSAP: NO ADDRESS SETS AVAILABLE.')
      DO IBM=1,NMIL
        WRITE(TEXT12,'(4Hmili,I8)') IBM
        CALL LCMLEN(IPSAP,TEXT12,ILENG,ITYLCM)
        IF(ILENG.EQ.0) THEN
          WRITE(HSMG,'(29HSPHSAP: MISSING MIXTURE '',A12,2H''.)')
     1    TEXT12
          CALL XABORT(HSMG)
        ENDIF
        CALL LCMSIX(IPSAP,TEXT12,1)
        IMAC=IBASE(ITOTM+IBM-1)
        IRES=IBASE(IRESM+IBM-1)
        IAD=IBASE(ISADRX+IBM-1)
        NDATAX=IBASE(LENGDX+IBM-1)
        NDATAP=IBASE(LENGDP+IBM-1)
        CALL SETARA(BASE,NGROUP,IFLUXS)
        CALL SETARA(BASE,NDATAX,IRDATA)
        CALL SETARA(IBASE,NDATAP,IIDATA)
        CALL LCMGET(IPSAP,'FLUXS',BASE(IFLUXS))
        CALL LCMGET(IPSAP,'RDATAX',BASE(IRDATA))
        CALL LCMGET(IPSAP,'IDATAP',IBASE(IIDATA))
        DO I=1,NGROUP
          J=(I-1)*NMIL+IBM
	  BASE(INWT0+J-1)=BASE(INWT0+J-1)+BASE(IFLUXS+I-1)
	ENDDO
        CALL SETARA(BASE,NGROUP*NL,ISIGSB)
        CALL SETARA(BASE,NGROUP*NGROUP*NL,ISS2DB)
        CALL SETARA(BASE,NGROUP*NREA,IXSB)
        IF(IMAC.NE.0) THEN
          CALL SPHSXS(NREA,INDX,NADRX,NGROUP,NL,NDATAX,NDATAP,
     1    NISO+IMAC,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),
     2    NOMREA,BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
          DO I=1,NGROUP*NL
            J=(I-1)*NMIL+IBM
            BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+BASE(ISIGSB+I-1)
          ENDDO
          DO I=1,NGROUP*NGROUP*NL
            J=(I-1)*NMIL+IBM
            BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+BASE(ISS2DB+I-1)
          ENDDO
          DO I=1,NGROUP*NREA
            J=(I-1)*NMIL+IBM
            BASE(IXS+J-1)=BASE(IXS+J-1)+BASE(IXSB+I-1)
          ENDDO
        ELSE IF(NISO.NE.0) THEN
          IF(NISOTS.EQ.0) CALL XABORT('SPHSAP: MISSING CONCES INFO.')
          CALL SETARA(BASE,NISOTS,ICONCE)
          CALL LCMGET(IPSAP,'CONCES',BASE(ICONCE))
          DO ISO=1,NISO
            WRITE(TEXT8,'(2A4)') (IBASE(INOMIS+2*(ISO-1)+I0),I0=0,1)
            ISOKEP=0
            DO IS2=1,NISOTS
              ISOKEP=IS2
              WRITE(TEXT9,'(2A4)') (IBASE(IISOTS+2*(IS2-1)+I0),I0=0,1)
              IF(TEXT9.EQ.TEXT8) GO TO 10
            ENDDO
            CYCLE
   10       DEN=BASE(ICONCE+ISOKEP-1)
            IF(DEN.NE.0.0) THEN
              CALL SPHSXS(NREA,INDX,NADRX,NGROUP,NL,NDATAX,NDATAP,
     1        ISO,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),NOMREA,
     2        BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
              DO I=1,NGROUP*NL
		J=(I-1)*NMIL+IBM
                BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+DEN*BASE(ISIGSB+I-1)
              ENDDO
              DO I=1,NGROUP*NGROUP*NL
                J=(I-1)*NMIL+IBM
                BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+DEN*BASE(ISS2DB+I-1)
              ENDDO
              DO I=1,NGROUP*NREA
		J=(I-1)*NMIL+IBM
                BASE(IXS+J-1)=BASE(IXS+J-1)+DEN*BASE(IXSB+I-1)
              ENDDO
            ENDIF
          ENDDO
          CALL RLSARA(BASE(ICONCE))
          IF(IRES.NE.0) THEN
            CALL SPHSXS(NREA,INDX,NADRX,NGROUP,NL,NDATAX,NDATAP,
     1      NISO+IRES,IAD,IBASE(IADRX),BASE(IRDATA),IBASE(IIDATA),
     2      NOMREA,BASE(ISIGSB),BASE(ISS2DB),BASE(IXSB),LBASE(ILXS))
            DO I=1,NGROUP*NL
              J=(I-1)*NMIL+IBM
              BASE(ISIGS+J-1)=BASE(ISIGS+J-1)+BASE(ISIGSB+I-1)
            ENDDO
            DO I=1,NGROUP*NGROUP*NL
              J=(I-1)*NMIL+IBM
              BASE(ISS2D+J-1)=BASE(ISS2D+J-1)+BASE(ISS2DB+I-1)
            ENDDO
            DO I=1,NGROUP*NREA
              J=(I-1)*NMIL+IBM
              BASE(IXS+J-1)=BASE(IXS+J-1)+BASE(IXSB+I-1)
            ENDDO
          ENDIF
        ELSE
          CALL XABORT('SPHSAP: NO MACROSCOPIC SET.')
        ENDIF
        CALL RLSARA(BASE(IXSB))
        CALL RLSARA(BASE(ISS2DB))
        CALL RLSARA(BASE(ISIGSB))
        CALL RLSARA(IBASE(IIDATA))
        CALL RLSARA(BASE(IRDATA))
        CALL RLSARA(BASE(IFLUXS))
*
        IF(LSPH) THEN
          CALL SETARA(BASE,IBASE(ILOCAD+NLOC)-1,IRVALO)
          CALL LCMGET(IPSAP,'RVALOC',BASE(IRVALO))
          DO ILOC=1,NLOC
            IF((LOCTYP(ILOC).EQ.'EQUI').AND.(LOCNAM(ILOC).EQ.HEQUI))
     1      THEN
              IF(IBASE(ILOCAD+ILOC)-IBASE(ILOCAD+ILOC-1).NE.NGROUP) THEN
                CALL XABORT('SPHSAP: INVALID NUMBER OF COMPONENTS FOR '
     1          //'SPH FACTORS')
              ENDIF
              DO IGR=1,NGROUP
                SPH(IBM,IGR)=BASE(IRVALO+IBASE(ILOCAD+ILOC-1)+IGR-2)
              ENDDO
            ENDIF
          ENDDO
          CALL RLSARA(BASE(IRVALO))
        ELSE
          DO IGR=1,NGROUP
            SPH(IBM,IGR)=1.0
          ENDDO
        ENDIF
*
        CALL LCMLEN(IPSAP,'cinetique',ILONG,ITYLCM)
        IF((NPRC.GT.0).AND.(ILONG.NE.0)) THEN
          CALL LCMSIX(IPSAP,'cinetique',1)
          CALL LCMGET(IPSAP,'NPR',NPR)
          IF(NPR.NE.NPRC) CALL XABORT('SPHSAP: NPR INCONSISTENCY(2).')
          CALL LCMGET(IPSAP,'LAMBRS',BASE(ILAMB))
          CALL LCMGET(IPSAP,'CHIRS',BASE(ICHIRS+(IBM-1)*NPRC*NGROUP))
          CALL LCMGET(IPSAP,'BETARS',BASE(IBETAR+(IBM-1)*NPRC))
          CALL LCMGET(IPSAP,'INVELS',BASE(INVELS+(IBM-1)*NGROUP))
          CALL LCMSIX(IPSAP,' ',2)
        ENDIF
        CALL LCMSIX(IPSAP,' ',2)
*       END OF LOOP OVER SAPHYB MIXTURES
      ENDDO
      IF(NPARL.GT.0) CALL RLSARA(IBASE(ILOCAD))
      CALL LCMSIX(IPSAP,' ',2)
*----
*  IDENTIFY SPECIAL FLUX EDITS
*----
      CALL SETARA(IBASE,2*NREA,IIHEDI)
      NED=0
      DO IREA=1,NREA
        IF(.NOT.LBASE(ILXS+IREA-1)) CYCLE
        IF(NOMREA(IREA).EQ.'TOTALE') CYCLE
        IF(NOMREA(IREA).EQ.'TOTALE P1') CYCLE
        IF(NOMREA(IREA).EQ.'EXCESS') CYCLE
        IF(NOMREA(IREA).EQ.'FISSION') CYCLE
        IF(NOMREA(IREA).EQ.'SPECTRE') CYCLE
        IF(NOMREA(IREA).EQ.'NU*FISSION') CYCLE
        IF(NOMREA(IREA).EQ.'ENERGIE') CYCLE
        IF(NOMREA(IREA).EQ.'SELF') CYCLE
        IF(NOMREA(IREA).EQ.'TRANSP-CORR') CYCLE
        IF(NOMREA(IREA).EQ.'FUITES') CYCLE
        IF(NOMREA(IREA).EQ.'DIFFUSION') CYCLE
        IF(NOMREA(IREA).EQ.'TRANSFERT') CYCLE
        NED=NED+1
        READ(NOMREA(IREA),'(2A4)') IBASE(IIHEDI+2*NED-2),
     1  IBASE(IIHEDI+2*NED-1)
      ENDDO
*----
*  STORE MACROLIB.
*----
      CALL LCMPUT(IPMAC,'VOLUME',NMIL,2,BASE(IXVOLM))
      IF(NPRC.GT.0) CALL LCMPUT(IPMAC,'LAMBDA-D',NPRC,2,BASE(ILAMB))
      IFISS=0
      ITRANC=0
      LSTRD=(B2.EQ.0.0)
      LDIFF=.FALSE.
      CALL SETARA(BASE,NMIL,ISTR)
      CALL SETARA(BASE,NMIL,IWRK)
      CALL SETARA(BASE,NGROUP,IDIFHO)
      CALL SETARA(BASE,NGROUP,IFLUHO)
      CALL XDRSET(BASE(IDIFHO),NGROUP,0.0)
      CALL XDRSET(BASE(IFLUHO),NGROUP,0.0)
      CALL XDRSET(SIGS,NMIL*NGROUP,0.0)
      JPMAC=LCMLID(IPMAC,'GROUP',NGROUP)
      DO IGR=1,NGROUP
        CALL XDRSET(BASE(ISTR),NMIL,0.0)
        KPMAC=LCMDIL(JPMAC,IGR)
        IOF=NMIL*(IGR-1)
        CALL LCMPUT(KPMAC,'FLUX-INTG',NMIL,2,BASE(INWT0+IOF))
        IF(LSPH) CALL LCMPUT(KPMAC,'SPH00',NMIL,2,SPH(1,IGR))
        IF(NPRC.GT.0) THEN
          DO IBM=1,NMIL
            BASE(IWRK+IBM-1)=BASE(INVELS+(IBM-1)*NGROUP+IGR-1)
          ENDDO
          CALL LCMPUT(KPMAC,'OVERV',NMIL,2,BASE(IWRK))
        ENDIF
        DO IREA=1,NREA
          IF(.NOT.LBASE(ILXS+IREA-1)) CYCLE
          IF(NOMREA(IREA).EQ.'TOTALE') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            IF(LSTRD) THEN
              DO IBM=1,NMIL
                BASE(ISTR+IBM-1)=BASE(ISTR+IBM-1)+BASE(IXS+IOF+IBM-1)
              ENDDO
            ENDIF
            CALL LCMPUT(KPMAC,'NTOT0',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'TOTALE P1') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'NTOT1',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'EXCESS') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
*           correct scattering XS with excess XS
            DO IBM=1,NMIL
              SIGS(IBM,IGR)=SIGS(IBM,IGR)+BASE(IXS+IOF+IBM-1)
            ENDDO
            CALL LCMPUT(KPMAC,'N2N',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'FISSION') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'NFTOT',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'SPECTRE') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'CHI',NMIL,2,BASE(IXS+IOF))
            DO IPRC=1,NPRC
              DO IBM=1,NMIL
                IOF=(IBM-1)*NGROUP*NPRC+(IPRC-1)*NGROUP+IGR
                BASE(IWRK+IBM-1)=BASE(ICHIRS+IOF-1)
              ENDDO
              WRITE(TEXT12,'(A3,I2.2)') 'CHI',IPRC
              CALL LCMPUT(KPMAC,TEXT12,NMIL,2,BASE(IWRK))
            ENDDO
          ELSE IF(NOMREA(IREA).EQ.'NU*FISSION') THEN
            IFISS=1
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'NUSIGF',NMIL,2,BASE(IXS+IOF))
            DO IPRC=1,NPRC
              DO IBM=1,NMIL
                BASE(IWRK+IBM-1)=BASE(IXS+IOF+IBM-1)
     1                            *BASE(IBETAR+(IBM-1)*NPRC+IPRC-1)
              ENDDO
              WRITE(TEXT12,'(A6,I2.2)') 'NUSIGF',IPRC
              CALL LCMPUT(KPMAC,TEXT12,NMIL,2,BASE(IWRK))
            ENDDO
          ELSE IF(NOMREA(IREA).EQ.'ENERGIE') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'H-FACTOR',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'SELF') THEN
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,'SIGW00',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'TRANSP-CORR') THEN
            ITRANC=2
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            IF(LSTRD) THEN
              DO IBM=1,NMIL
                BASE(ISTR+IBM-1)=BASE(ISTR+IBM-1)-BASE(IXS+IOF+IBM-1)
              ENDDO
            ENDIF
            CALL LCMPUT(KPMAC,'TRANC',NMIL,2,BASE(IXS+IOF))
          ELSE IF(NOMREA(IREA).EQ.'FUITES') THEN
            LDIFF=LSTRD
            IF(.NOT.LSTRD) THEN
              IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
              DO IBM=1,NMIL
	        LDIFF=LDIFF.OR.(BASE(IXS+IOF+IBM-1).NE.0.0)
                BASE(ISTR+IBM-1)=BASE(IXS+IOF+IBM-1)/B2
              ENDDO
            ENDIF
          ELSE IF(NOMREA(IREA).EQ.'DIFFUSION') THEN
            DO IL=1,NL
              WRITE(CM,'(I2.2)') IL-1
              IOF=NMIL*NGROUP*(IL-1)+NMIL*(IGR-1)
              IF(IL.EQ.1) THEN
                DO IBM=1,NMIL
                  SIGS(IBM,IGR)=SIGS(IBM,IGR)+BASE(ISIGS+IOF+IBM-1)
                ENDDO
              ELSE
                CALL LCMPUT(KPMAC,'SIGS'//CM,NMIL,2,BASE(ISIGS+IOF))
              ENDIF
            ENDDO
          ELSE IF(NOMREA(IREA).EQ.'TRANSFERT') THEN
            CALL SETARA(BASE,NGROUP*NMIL,ISCAT)
            CALL SETARA(BASE,NMIL,IGAR)
            DO IL=1,NL
              WRITE(CM,'(I2.2)') IL-1
              IPOSDE=0
              DO IBM=1,NMIL
                IPOS(IBM)=IPOSDE+1
                IGMIN=IGR
                IGMAX=IGR
                DO JGR=NGROUP,1,-1
                  IOF=NMIL*NGROUP*NGROUP*(IL-1)+NMIL*NGROUP*(JGR-1)+
     1                NMIL*(IGR-1)
                  IF(BASE(ISS2D+IOF).NE.0.0) THEN
                    IGMIN=MIN(IGMIN,JGR)
                    IGMAX=MAX(IGMAX,JGR)
                  ENDIF
                ENDDO
                IJJM(IBM)=IGMAX
                NJJM(IBM)=IGMAX-IGMIN+1
                DO JGR=IGMAX,IGMIN,-1
                  IPOSDE=IPOSDE+1
                  IOF=NMIL*NGROUP*NGROUP*(IL-1)+NMIL*NGROUP*(JGR-1)+
     1                NMIL*(IGR-1)
                  BASE(ISCAT+IPOSDE-1)=BASE(ISS2D+IOF+IBM-1)
                ENDDO
                BASE(IGAR+IBM-1)=BASE(ISCAT+IPOS(IBM)+IJJM(IBM)-IGR-1)
              ENDDO
              CALL LCMPUT(KPMAC,'SCAT'//CM,IPOSDE,2,BASE(ISCAT))
              CALL LCMPUT(KPMAC,'NJJS'//CM,NMIL,1,NJJM)
              CALL LCMPUT(KPMAC,'IJJS'//CM,NMIL,1,IJJM)
              CALL LCMPUT(KPMAC,'IPOS'//CM,NMIL,1,IPOS)
              CALL LCMPUT(KPMAC,'SIGW'//CM,NMIL,2,BASE(IGAR))
            ENDDO
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(ISCAT))
          ELSE
            IOF=NMIL*NGROUP*(IREA-1)+NMIL*(IGR-1)
            CALL LCMPUT(KPMAC,NOMREA(IREA),NMIL,2,BASE(IXS+IOF))
          ENDIF
        ENDDO
        IF(LSTRD) THEN
          IF((ITRANC.EQ.0).AND.(NL.GT.1)) THEN
*           Apollo-type transport correction
            IOF=NMIL*(NGROUP+IGR-1)
            DO IBM=1,NMIL
              BASE(ISTR+IBM-1)=BASE(ISTR+IBM-1)-BASE(ISIGS+IOF+IBM-1)
            ENDDO
          ENDIF
          DO IBM=1,NMIL
            BASE(ISTR+IBM-1)=1.0/(3.0*BASE(ISTR+IBM-1))
          ENDDO
        ENDIF
        IF((ITRANC.EQ.0).AND.(NL.GT.1)) THEN
*         Apollo-type transport correction
          IF(IGR.EQ.NGROUP) ITRANC=2
          IOF=NMIL*(NGROUP+IGR-1)
          CALL LCMPUT(KPMAC,'TRANC',NMIL,2,BASE(ISIGS+IOF))
        ENDIF
        IF(LDIFF) THEN
          CALL LCMPUT(KPMAC,'DIFF',NMIL,2,BASE(ISTR))
          IOF=NMIL*(IGR-1)
          DO IBM=1,NMIL
            BASE(IFLUHO+IGR-1)=BASE(IFLUHO+IGR-1)+BASE(INWT0+IOF+IBM-1)
            BASE(IDIFHO+IGR-1)=BASE(IDIFHO+IGR-1)+BASE(INWT0+IOF+IBM-1)
     1      *BASE(ISTR+IBM-1)
          ENDDO
        ENDIF
      ENDDO
      IF(LDIFF) THEN
        DO IGR=1,NGROUP
          BASE(IDIFHO+IGR-1)=BASE(IDIFHO+IGR-1)/BASE(IFLUHO+IGR-1)
        ENDDO
        CALL LCMPUT(IPMAC,'DIFFB1HOM',NGROUP,2,BASE(IDIFHO))
      ENDIF
      CALL RLSARA(BASE(IFLUHO))
      CALL RLSARA(BASE(IDIFHO))
      CALL RLSARA(BASE(IWRK))
      CALL RLSARA(BASE(ISTR))
*----
*  RELEASE MEMORY
*----
      CALL RLSARA(BASE(INVELS))
      CALL RLSARA(BASE(IBETAR))
      CALL RLSARA(BASE(ICHIRS))
      CALL RLSARA(BASE(ILAMB))
      CALL RLSARA(LBASE(ILXS))
      CALL RLSARA(BASE(IXS))
      CALL RLSARA(BASE(ISS2D))
      CALL RLSARA(BASE(ISIGS))
      CALL RLSARA(BASE(INWT0))
      IF(NISOTS.GT.0) CALL RLSARA(IBASE(IISOTS))
      CALL RLSARA(IBASE(LENGDP))
      CALL RLSARA(IBASE(LENGDX))
      CALL RLSARA(IBASE(ISADRX))
      CALL RLSARA(BASE(IXVOLM))
      IF(NADRX.GT.0) CALL RLSARA(IBASE(IADRX))
      IF(NISO.GT.0) CALL RLSARA(IBASE(INOMIS))
      CALL RLSARA(IBASE(IRESM))
      CALL RLSARA(IBASE(ITOTM))
*----
*  SAVE SCATTERING P0 INFO
*----
      DO IGR=1,NGROUP
        KPMAC=LCMDIL(JPMAC,IGR)
        CALL LCMPUT(KPMAC,'SIGS00',NMIL,2,SIGS(1,IGR))
      ENDDO
*----
*  WRITE STATE VECTOR
*----
      TEXT12='L_MACROLIB'
      CALL LCMPTC(IPMAC,'SIGNATURE',12,1,TEXT12)
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NGROUP
      ISTATE(2)=NMIL
      ISTATE(3)=NL ! 1+scattering anisotropy
      ISTATE(4)=IFISS
      ISTATE(5)=NED
      ISTATE(6)=ITRANC
      ISTATE(7)=NPRC
      IF(LDIFF) ISTATE(9)=1
      CALL LCMPUT(IPMAC,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(NED.GT.0) CALL LCMPUT(IPMAC,'ADDXSNAME-P0',2*NED,3,
     1 IBASE(IIHEDI))
      CALL RLSARA(IBASE(IIHEDI))
      RETURN
      END
