*DECK VAL
      SUBROUTINE VAL(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* interpolate the flux distribution.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create type(L_FVIEW);
*         HENTRY(2): read-only type(L_TRACK);
*         HENTRY(3): read-only type(L_FLUX).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY,IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
      CHARACTER    HENTRY(NENTRY)*12
*----
*  LOCAL VARIABLES
*----
      INTEGER NSTATE
      PARAMETER (NSTATE=40)
      CHARACTER TEXT12*12,HSIGN*12,CMODUL*12
      INTEGER INDIC,NITMA
      DOUBLE PRECISION DFLOT
      REAL FLOT
      REAL DX,DY,DZ
      LOGICAL L3D
      INTEGER IGP(NSTATE),IFL(NSTATE),IFV(NSTATE),NXD,NYD,NZD,IELEM,NUN,
     1  IMPX,DIM,NG,NLF,NXI,NYI,NZI,NREG,ICHX,IDIM,ITYPE,L4,MAXKN,MKN,
     2  LC,ITYLCM
      INTEGER I,IG,J,K,IOF
      INTEGER IPFVW,IPTRK,IPFLU,JPFVW,JPFLU
      INTEGER LCMGID,LCMLID
      REAL E(25)
      REAL BASE(1)
      INTEGER IBASE
      COMMON IBASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  SETARA POINTERS
*----
      INTEGER IMAT,IKFLX,IKN,IWRK
      INTEGER IXX,IYY,IZZ,IMXD,IMYD,IMZD,IMXI,IMYI,IMZI,IFXYZ,IFLXD,
     1 ICOEF
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.NE.3) CALL XABORT('VAL: 3 PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('FLD: LCM '
     1 //'OBJECT EXPECTED AT LHS.')
      IF(JENTRY(1).NE.0) CALL XABORT('VAL: ENTRY IN CREATE MODE '
     1 //'EXPECTED.')
      IPFVW=KENTRY(1)
      DO I=2,3
        IF(JENTRY(I).NE.2) CALL XABORT('VAL: LCM OBJECT IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
        CALL LCMGTC(KENTRY(I),'SIGNATURE',12,1,HSIGN)
        IF(HSIGN.EQ.'L_FLUX') THEN
           IPFLU=KENTRY(I)
        ELSEIF(HSIGN.EQ.'L_TRACK') THEN
           IPTRK=KENTRY(I)
           CALL LCMGTC(IPTRK,'TRACK-TYPE',12,1,CMODUL)
        ELSE
           TEXT12=HENTRY(I)
           CALL XABORT('VAL: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1     '. L_FLUX or L_TRACK EXPECTED.')
        ENDIF
      ENDDO
      HSIGN='L_FVIEW'
      CALL LCMPTC(KENTRY(1),'SIGNATURE',12,1,HSIGN)
      L3D=.TRUE.
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NREG=IGP(1)
      NUN=IGP(2)
      ITYPE=IGP(6)
      NLF=0
      ICHX=0
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IF(CMODUL.EQ.'BIVAC') THEN
         L3D=.FALSE.
         IELEM=IGP(8)
         NLF=IGP(14)
         NXD=IGP(12)
         NYD=IGP(13)
         NZD=1
         CALL XABORT('VAL: BIVAC is currently not supported.')
      ELSE IF(CMODUL.EQ.'TRIVAC') THEN
         L3D=.TRUE.
         IELEM=IGP(9)
         L4=IGP(11)
         ICHX=IGP(12)
         NLF=IGP(30)
         NXD=IGP(14)
         NYD=IGP(15)
         NZD=IGP(16)
         IF(NZD.EQ.0) L3D=.FALSE.
         NZD=MAX(1,NZD)
      ENDIF
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(IBASE,NREG,IKFLX)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKFLX))
      CALL SETARA(BASE,NXD+1,IMXD)
      CALL SETARA(BASE,NYD+1,IMYD)
      CALL SETARA(BASE,NZD+1,IMZD)
      CALL SETARA(BASE,NREG,IXX)
      CALL SETARA(BASE,NREG,IYY)
      CALL SETARA(BASE,NREG,IZZ)
      CALL LCMGET(IPTRK,'XX',BASE(IXX))
      CALL LCMGET(IPTRK,'YY',BASE(IYY))
      IF(L3D) CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
*
      CALL LCMGET(IPFLU,'STATE-VECTOR',IFL)
      NG=IFL(1)
*----
*  READ INPUTS
*----
      IMPX=0
      DX=1.
      DY=1.
      DZ=1.
      CALL REDGET(INDIC,NITMA,FLOT,TEXT12,DFLOT)
      IF(INDIC.NE.3) CALL XABORT('VAL: character data expected.')
      IF(TEXT12.EQ.'EDIT') THEN
        CALL REDGET(INDIC,IMPX,FLOT,TEXT12,DFLOT)
        IF(INDIC.NE.1) CALL XABORT('VAL: integer data expected.')
        CALL REDGET(INDIC,NITMA,FLOT,TEXT12,DFLOT)
        IF(INDIC.NE.3) CALL XABORT('VAL: character data expected.')
      ENDIF
      IF(TEXT12.NE.'DIM') CALL XABORT('VAL: ''DIM'' expected.')
      CALL REDGET(INDIC,DIM,FLOT,TEXT12,DFLOT)
      IF((DIM.LE.1).OR.(DIM.GE.4)) CALL XABORT('VAL: DIM .ne. 2 or 3'
     1 //' expected.')
      CALL REDGET(INDIC,NITMA,DX,TEXT12,DFLOT)
      CALL REDGET(INDIC,NITMA,DY,TEXT12,DFLOT)
      IF(DIM.EQ.3) CALL REDGET(INDIC,NITMA,DZ,TEXT12,DFLOT)
      CALL REDGET(INDIC,NITMA,FLOT,TEXT12,DFLOT)
      IF(TEXT12.NE.';') CALL XABORT('VAL: '';'' expected.')
*----
*  Get Data in L_TRACK
*----
*     compute X and Y mesh from L_TRACK
      BASE(IMXD)=0.0
      DO I=1,NXD
        BASE(IMXD+I)=BASE(IMXD+I-1)+BASE(IXX+I-1)
      ENDDO
      BASE(IMYD)=0.0
      DO I=1,NYD
        BASE(IMYD+I)=BASE(IMYD+I-1)+BASE(IYY+(I-1)*NXD)
      ENDDO
      BASE(IMZD)=0.0
      IF(L3D) THEN
        DO I=1,NZD
          BASE(IMZD+I)=BASE(IMZD+I-1)+BASE(IZZ+(I-1)*NXD*NYD)
        ENDDO
      ELSE
        BASE(IMZD+1)=0.0
      ENDIF
*----
*  Perform interpolation
*----
*     Compute points to interpolate
      NXI=INT((BASE(IMXD+NXD)-BASE(IMXD))/DX)+1
      NYI=INT((BASE(IMYD+NYD)-BASE(IMYD))/DY)+1
      NZI=INT((BASE(IMZD+NZD)-BASE(IMZD))/DZ)+1
      CALL SETARA(BASE,NXI,IMXI)
      CALL SETARA(BASE,NYI,IMYI)
      CALL SETARA(BASE,NZI,IMZI)
      CALL SETARA(BASE,NXI*NYI*NZI*NG,IFXYZ)
      DO I=1,NXI
        BASE(IMXI+I-1)=BASE(IMXD)+DX*REAL(I-1)
      ENDDO
      DO I=1,NYI
        BASE(IMYI+I-1)=BASE(IMYD)+DY*REAL(I-1)
      ENDDO
      DO I=1,NZI
        BASE(IMZI+I-1)=BASE(IMZD)+DZ*REAL(I-1)
      ENDDO
      JPFLU=LCMGID(IPFLU,'FLUX')
*     Get Data in L_FLUX
      CALL SETARA(BASE,NUN,IFLXD)
      DO IG=1,NG
        CALL LCMGDL(JPFLU,IG,BASE(IFLXD))
*       Perform interpolation
        IF(L3D) THEN
          IF(ICHX.EQ.1) THEN
*           Variational collocation method
            CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
            MKN=MAXKN/(NXD*NYD*NZD)
            CALL SETARA(IBASE,MAXKN,IKN)
            CALL LCMGET(IPTRK,'KN',IBASE(IKN))
            CALL LCMSIX(IPTRK,'BIVCOL',1)
            CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
            CALL LCMGET(IPTRK,'E',E)
            CALL LCMSIX(IPTRK,' ',2)
            CALL SETARA(BASE,NXD*NYD*NZD,IWRK)
            CALL SETARA(BASE,MAXKN,ICOEF)
            CALL VALUE2(LC,MKN,NXD,NYD,NZD,L4,BASE(IMXI),BASE(IMYI),
     1      BASE(IMZI),BASE(IMXD),BASE(IMYD),BASE(IMZD),BASE(IFLXD),
     2      BASE(IMAT),IBASE(IKN),NXI,NYI,NZI,E,
     3      BASE(IFXYZ+(IG-1)*NXI*NYI*NZI),BASE(IWRK),BASE(ICOEF))
            CALL RLSARA(BASE(ICOEF))
            CALL RLSARA(BASE(IWRK))
            CALL RLSARA(IBASE(IKN))
          ELSE IF(ICHX.EQ.2) THEN
*           Raviart-Thomas finite element method
            CALL VALUE4(IELEM,NUN,NXD,NYD,NZD,BASE(IMXI),BASE(IMYI),
     1      BASE(IMZI),BASE(IMXD),BASE(IMYD),BASE(IMZD),BASE(IFLXD),
     2      BASE(IMAT),BASE(IKFLX),NXI,NYI,NZI,
     3      BASE(IFXYZ+(IG-1)*NXI*NYI*NZI))
          ELSE IF(ICHX.EQ.3) THEN
*           Nodal collocation method (MCFD)
            CALL SETARA(BASE,NXD*NYD*NZD,IWRK)
            CALL VALUE1(IDIM,NXD,NYD,NZD,L4,BASE(IMXI),BASE(IMYI),
     1      BASE(IMZI),BASE(IMXD),BASE(IMYD),BASE(IMZD),BASE(IFLXD),
     2      BASE(IMAT),IELEM,NXI,NYI,NZI,
     3      BASE(IFXYZ+(IG-1)*NXI*NYI*NZI),BASE(IWRK))
            CALL RLSARA(BASE(IWRK))
          ELSE
            CALL XABORT('VAL: INTERPOLATION NOT IMPLEMENTED(1).')
          ENDIF
        ELSE
          IF(ICHX.EQ.2) THEN
*           Raviart-Thomas finite element method
            CALL VALUE5(IELEM,NUN,NXD,NYD,BASE(IMXI),BASE(IMYI),
     1      BASE(IMXD),BASE(IMYD),BASE(IFLXD),BASE(IMAT),
     2      BASE(IKFLX),NXI,NYI,BASE(IFXYZ+(IG-1)*NXI*NYI))
          ELSE
            CALL XABORT('VAL: INTERPOLATION NOT IMPLEMENTED(2).')
          ENDIF
        ENDIF
      ENDDO
*----
*  Save results
*----
      CALL LCMPUT(IPFVW,'MXI',NXI,2,BASE(IMXI))
      CALL LCMPUT(IPFVW,'MYI',NYI,2,BASE(IMYI))
      CALL LCMPUT(IPFVW,'MZI',NZI,2,BASE(IMZI))
      CALL XDISET(IFV,NSTATE,0)
      IFV(1)=NG
      IFV(2)=NXI
      IFV(3)=NYI
      IFV(4)=NZI
      CALL LCMPUT(IPFVW,'STATE-VECTOR',NSTATE,1,IFV)
      JPFVW=LCMLID(IPFVW,'FLUX',NG)
      DO IG=1,NG
        CALL LCMPDL(JPFVW,IG,NXI*NYI*NZI,2,
     1    BASE(IFXYZ+(IG-1)*NXI*NYI*NZI))
      ENDDO
*----
*  Save results
*----
      IF(IMPX.GE.1)THEN
        WRITE(6,*) 'Mesh along X-direction'
        WRITE(6,*) (BASE(IMXI+I-1),I=1,NXI)
        WRITE(6,*) 'Mesh along Y-direction'
        WRITE(6,*) (BASE(IMYI+I-1),I=1,NYI)
        WRITE(6,*) 'Mesh along Z-direction'
        WRITE(6,*) (BASE(IMZI+I-1),I=1,NZI)
        IF(IMPX.GE.2)THEN
          WRITE(6,*) 'Flux distribution:'
          DO IG=1,NG
            WRITE(6,*) 'Group',IG
            DO K=1,NZI
              WRITE(6,*) 'Plane',K
              DO J=1,NYI
                IOF=IFXYZ+(IG-1)*NXI*NYI*NZI+(J-1+(K-1)*NYI)*NXI-1
                WRITE(6,*) (BASE(IOF+I),I=1,NXI)
              ENDDO
            ENDDO
          ENDDO
        ENDIF
      ENDIF
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(BASE(IFXYZ))
      CALL RLSARA(BASE(IFLXD))
      CALL RLSARA(BASE(IMZI))
      CALL RLSARA(BASE(IMYI))
      CALL RLSARA(BASE(IMXI))
      CALL RLSARA(BASE(IMZD))
      CALL RLSARA(BASE(IMYD))
      CALL RLSARA(BASE(IMXD))
      CALL RLSARA(BASE(IZZ))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(IBASE(IMAT))
      CALL RLSARA(IBASE(IKFLX))
      RETURN
      END
